#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine drystress(kpti, kptj)
! ---------------------------------------------------------------------
      use inland_ibispar, only: nsoilay, stressfac
      use inland_comveg
      use inland_comsoi

      implicit none

!-----------------------------------------------------------------------
! input variables
      integer kpti            ! index of 1st point of little vector in big lpt vector
      integer kptj            ! index of last point of little vector

! local variables
      integer i, k

      real*8 awc,   & ! available water content (fraction)
             znorm, & ! normalizing factor
             zwilt    ! function of awc, =1 if awc = 1 (no stress)

! stressfac determines the 'strength' of the soil moisture
! stress on physiological processes
!
! strictly speaking, stresst* is multiplied to the vmax
! parameters used in the photosynthesis calculations
!
! stressfac here determines the shape of the soil moisture response
      znorm = 1.0 - exp(stressfac)
      do 100 i = kpti, kptj
! initialize stress parameter
         stresstl(i) = 0.0
         stresstu(i) = 0.0

! fraction of soil water uptake in each layer
         do 110 k = 1, nsoilay

! plant available water content (fraction)
            awc = min(dble(1.0),max(dble(0.0),(wsoi(i,k)*(1 - wisoi(i,k)) - swilt(i,k)) / &
                      (sfield(i,k) - swilt(i,k))))
            zwilt = (1. - exp(stressfac * awc)) / znorm

! update for each layer
            stressl(i,k) = froot(k,1) * max (dble(0.0), min (dble(1.0), zwilt))
            stressu(i,k) = froot(k,2) * max (dble(0.0), min (dble(1.0), zwilt))

! integral over rooting profile
            stresstl(i) = stresstl(i) + stressl(i,k)
            stresstu(i) = stresstu(i) + stressu(i,k)
110      continue
100   continue
      return
end subroutine drystress
