#include "inland_config.h"
! TODO: This subroutine has sensitive differences in calculation in comparison
!      to SAGE's IBIS. These differences might and will change final output,
!      so we will need to evaluate if these results are better or worse than
!      original SAGE's.
! ---------------------------------------------------------------------
subroutine dynaveg () 
! ---------------------------------------------------------------------
      use inland_ibispar
      use inland_ibisctl, only: isimfire
      use inland_compft
      use inland_comsoi
      use inland_comsum
      use inland_comveg

      implicit none

! local variables
      integer i,   &    ! gridcell counter
              j,   &
              nit, &    ! number of iterations
              niter     ! total number of iterations

      real*8 denswood, & ! kg/m**3
             saparea,  & ! in m**2
             sapspeed, & ! in mm/day
             sapvolume,& ! in m**3
             seedbio,  & ! correction term
             taufin,   & !
             trans,    & ! (2.5 mm/day)
             wood,     & ! total amount of woody biomass in gridcell
             rwork,    & ! working variable (1/total number of iterations)
             cdistinit   ! carbon lost to atmosphere by disturbance as estimated
                         ! in sumnow (used to compute correction term)  

!      real*8 
!     >  aleaf(npft),   ! allocation fraction to leaves
!     >  aroot(npft),   ! allocation fraction to fine roots
!     >  awood(npft),   ! allocation fraction to wood
!     >  tauleaf(npft), ! turnover time of carbon in leaves (years)
!     >  tauroot(npft), ! turnover time of carbon in fine roots (years)
!     >  tauwood(npft), ! turnover time of carbon in wood (years)
!     >  tauwood0(npft) ! normal (unstressed) turnover time
      real*8 cbiolmin(lbeg:lend,npft) ! minimum leaf biomass used as seed.

! ibis uses a small number of plant functional types:
!
!  1: tropical broadleaf evergreen tree
!  2: tropical broadleaf drought-deciduous trees
!  3: warm-temperate broadleaf evergreen tree
!  4: temperate conifer evergreen tree
!  5: temperate broadleaf cold-deciduous tree
!  6: boreal conifer evergreen tree
!  7: boreal broadleaf cold-deciduous tree
!  8: boreal conifer cold-deciduous tree
!  9: evergreen shrub
! 10: deciduous shrub
! 11: warm (c4) grass
! 12: cool (c3) grass

! ---------------------------------------------------------------------
! * * * specify biomass turnover parameters (years) * * *
! ---------------------------------------------------------------------
!      data tauleaf / 1.01,   ! tropical broadleaf evergreen trees
!     >               1.00,   ! tropical broadleaf drought-deciduous trees
!     >               1.00,   ! warm-temperate broadleaf evergreen trees
!     >               2.00,   ! temperate conifer evergreen trees
!     >               1.00,   ! temperate broadleaf cold-deciduous trees
!     >               2.50,   ! boreal conifer evergreen trees
!     >               1.00,   ! boreal broadleaf cold-deciduous trees
!     >               1.00,   ! boreal conifer cold-deciduous trees
!     >               1.50,   ! evergreen shrubs
!     >               1.00,   ! deciduous shrubs
!     >               1.25,   ! warm (c4) grasses
!     >               1.50 /  ! cool (c3) grasses
!
!      data tauwood0 / 25.0,  ! tropical broadleaf evergreen trees
!     >                25.0,  ! tropical broadleaf drought-deciduous trees
!     >                25.0,  ! warm-temperate broadleaf evergreen trees
!     >                50.0,  ! temperate conifer evergreen trees
!     >                50.0,  ! temperate broadleaf cold-deciduous trees
!     >               100.0,  ! boreal conifer evergreen trees
!     >               100.0,  ! boreal broadleaf cold-deciduous trees
!     >               100.0,  ! boreal conifer cold-deciduous trees
!     >                 5.0,  ! evergreen shrubs
!     >                 5.0,  ! deciduous shrubs
!     >               999.0,  ! warm (c4) grasses
!     >               999.0 / ! cool (c3) grasses

! FIXME: useless initialization (it's reset at the start of the do 100 below)
      wood = 0.001

! iteration
      niter = 10
      rwork = 1. / float(niter)

! begin global grid
      do 100 i = lbeg, lend

! initialize wood for gridcell
         wood = 0.001
! ---------------------------------------------------------------------
! * * * initialize vegetation dynamics pools * * *
! ---------------------------------------------------------------------
!
! zero out litter fall fields
         falll(i) = 0.0
         fallr(i) = 0.0
         fallw(i) = 0.0
         caccount(i) = 0.0

! zero out carbon lost due to disturbance
         cdisturb(i) = 0.0
         cdistinit = 0.0

! set fixed disturbance regime
         disturbf(i) = 0.005
         disturbo(i) = 0.005

!*** DTP 2000/08/10. One can do a decent test of ACME by setting
!    these disturbance rates to zero. With these values, the area
!    disturbed each year will be zero so the distribution of biomass
!    and PFTs across the domain should be identical to those 
!    resulting from a run of standard IBIS with zero disturbance

!        disturbf(i) = 0.0  ! Test with zero disturbance rate 
!        disturbo(i) = 0.0  ! (This should equal reference sim).

! call fire disturbance routine
         if (isimfire.eq.1) call fire

! ---------------------------------------------------------------------
! * * * update npp, and pool losses  * * *
! ---------------------------------------------------------------------

! go through all the pfts
         do 110 j = 1, npft

! maintain minimum value of leaf carbon in areas where plants exist
            cbiolmin(i,j) = exist(i,j)*xminlai/specla(j)
!
! existence arrays applied to npp in sumnow
!        aynpp(i,j)  = exist(i,j) * aynpp(i,j)

! determine above-ground npp for each plant type
            ayanpp(i,j) = (aleaf(j) + awood(j)) * aynpp(i,j)

! determine turnover rates for woody biomass:
! if pft can exist,    then tauwood = tauwood0 (normal turnover),
! if pft cannot exist, then tauwood = taufin years (to kill off trees)
!           taufin     = 5.0
            taufin     = tauwood0(j) * 0.5

            tauwood(j) = tauwood0(j)-(tauwood0(j)-taufin) * (1.0-exist(i,j))

! assume a constant fine root turnover time
!          tauroot(j) = 1.0

! calculate carbon lost to atmosphere by disturbance (non iterated) :
! corresponds to value calculated by sumnow and used in the instantaneous nee
! used to balance carbon
            cdistinit = cdistinit + (                                     &
                           (cbiol(i,j) - cbiolmin(i,j)) * (               &
                              disturbf(i) + disturbo(i)                   &
                           ) + cbiow(i,j) * (disturbf(i) + disturbo(i)) + &
                           cbior(i,j) * (disturbf(i) + disturbo(i)))

! iteration loop
            do 10 nit = 1, niter

! determine litter fall rates
               falll(i) = falll(i) + (cbiol(i,j) - cbiolmin(i,j)) / &
                          tauleaf(j) * rwork
               fallr(i) = fallr(i) + cbior(i,j) / tauroot(j) * rwork
               fallw(i) = fallw(i) + cbiow(i,j) / tauwood(j) * rwork

! ---------------------------------------------------------------------
! * * * apply disturbances * * *
! ---------------------------------------------------------------------
!
! calculate biomass (vegetations) carbon lost to atmosphere   
! used to balance net ecosystem exchange  
               cdisturb(i) = cdisturb(i) + (                                  &
                                (cbiol(i,j) - cbiolmin(i,j)) * (disturbf(i) + &
                                disturbo(i)) + cbiow(i,j) * (disturbf(i) +    &
                                disturbo(i)) + cbior(i,j) * (disturbf(i) +    &
                                disturbo(i))                                  &
                             ) * rwork                  
! ---------------------------------------------------------------------
! * * * update biomass pools  * * *
! ---------------------------------------------------------------------
!
! update carbon reservoirs using an analytical solution
! to the original carbon balance differential equation
               cbiol(i,j) = cbiol(i,j) + (                                     &
                               aleaf(j) * max (dble(0.), aynpp(i,j)) - (cbiol(i,j) - &
                               cbiolmin(i,j)) / tauleaf(j) - (disturbf(i) +    &
                               disturbo(i)) * (cbiol(i,j) - cbiolmin(i,j))     &
                            ) * rwork 

               cbiow(i,j) = cbiow(i,j) + (                                    &
                               awood(j) * max (dble(0.), aynpp(i,j)) - cbiow(i,j) / &
                               tauwood(j) - (disturbf(i)+disturbo(i)) *       &
                               cbiow(i,j) &
                            ) * rwork

               cbior(i,j) = cbior(i,j) + (                                    &
                               aroot(j) * max (dble(0.), aynpp(i,j)) - cbior(i,j) / &
                               tauroot(j) - (disturbf(i)+disturbo(i)) *       &
                               cbior(i,j) &
                       ) * rwork
10          continue ! end of iteration loop

            if (j.le.8) wood = wood + max (dble(0.0), cbiow(i,j))
               seedbio = max(dble(0.),(cbiolmin(i,j) - cbiol(i,j)))

! account for negative biomass in nee (caccount > 0: carbon that has been accounted for 
! as absorbed in the fluxes but that is not accounted for in the calculation of biomass 
! pools ==> has to be released to atmosphere)
            caccount(i) = caccount(i)+seedbio-min(dble(0.0), cbiow(i,j))-min(dble(0.0), cbior(i,j)) 

! constrain biomass fields to be positive
            cbiol(i,j) = max (cbiolmin(i,j), cbiol(i,j))
            cbiow(i,j) = max (dble(0.0), cbiow(i,j))
            cbior(i,j) = max (dble(0.0), cbior(i,j))

! update vegetation's physical characteristics
            plai(i,j)    = cbiol(i,j) * specla(j)
            biomass(i,j) = cbiol(i,j) + cbiow(i,j) + cbior(i,j)
110      continue

! ---------------------------------------------------------------------
! * * * update annual npp, lai, and biomass * * *
! ---------------------------------------------------------------------

! Disturbance can't result in negative biomass. caccount account for the 
! carbon not to be removed by the disturbance.
!       cdisturb(i) = cdisturb(i) - caccount(i)
!
! determine total ecosystem positive npp (changed by exist at begin
! of subroutine). Different from sum of monthly and daily npp)
         aynpptot(i) = max(dble(0.0),aynpp(i,1))  + max(dble(0.0),aynpp(i,2))  + &
                       max(dble(0.0),aynpp(i,3))  + max(dble(0.0),aynpp(i,4))  + &
                       max(dble(0.0),aynpp(i,5))  + max(dble(0.0),aynpp(i,6))  + &
                       max(dble(0.0),aynpp(i,7))  + max(dble(0.0),aynpp(i,8))  + &
                       max(dble(0.0),aynpp(i,9))  + max(dble(0.0),aynpp(i,10)) + &
                       max(dble(0.0),aynpp(i,11)) + max(dble(0.0),aynpp(i,12))

! adjust annual net ecosystem exchange (calculated in stats.f) 
! by new value of npp (depending on exist), andloss of carbon to
! atmosphere due to biomass burning (fire)
!       ayneetot(i) = aynpptot(i) - ayco2mic(i) - cdisturb(i) + caccount(i)

! the fact that only >0 npp was used has also to be accounted for (only 
! possible 1ce a year) difference between disturbance calculated in dynaveg and
! sumnow has also to be accounted for
         caccount(i) = caccount(i) -                                 &
                       min(dble(0.0),aynpp(i,1))  - min(dble(0.0),aynpp(i,2))  - &
                       min(dble(0.0),aynpp(i,3))  - min(dble(0.0),aynpp(i,4))  - &
                       min(dble(0.0),aynpp(i,5))  - min(dble(0.0),aynpp(i,6))  - &
                       min(dble(0.0),aynpp(i,7))  - min(dble(0.0),aynpp(i,8))  - &
                       min(dble(0.0),aynpp(i,9))  - min(dble(0.0),aynpp(i,10)) - &
                       min(dble(0.0),aynpp(i,11)) - min(dble(0.0),aynpp(i,12)) - &
                       (cdisturb(i) - cdistinit)

! adjust annual net ecosystem exchange (calculated in sumyear)
! by corrections computed in dynaveg
         ayneetot(i) = ayneetot(i) + caccount(i)

! determine total ecosystem above-ground npp
         ayanpptot(i) = ayanpp(i,1)  + ayanpp(i,2)  + ayanpp(i,3) + &
                        ayanpp(i,4)  + ayanpp(i,5)  + ayanpp(i,6) + &
                        ayanpp(i,7)  + ayanpp(i,8)  + ayanpp(i,9) + &
                        ayanpp(i,10) + ayanpp(i,11) + ayanpp(i,12)

! update total canopy leaf area
         totlaiu(i) = plai(i,1) + plai(i,2) + plai(i,3) + plai(i,4) + &
                      plai(i,5) + plai(i,6) + plai(i,7) + plai(i,8)
         totlail(i) = plai(i,9) + plai(i,10) + plai(i,11) + plai(i,12)

! update total biomass
         totbiou(i) = biomass(i,1) + biomass(i,2) + biomass(i,3) + &
                      biomass(i,4) + biomass(i,5) + biomass(i,6) + &
                      biomass(i,7) + biomass(i,8)
         totbiol(i) = biomass(i,9)  + biomass(i,10) + biomass(i,11) + &
                      biomass(i,12)

! ---------------------------------------------------------------------
! * * * update fractional cover and vegetation height parameters * * *
! ---------------------------------------------------------------------

! update fractional cover of forest and herbaceous canopies:
         fu(i) = (1.0 - exp(-wood)) / (1.0 - exp(-woodnorm))
         fl(i) = totlail(i) / 1.0

! apply disturbances to fractional cover
         fu(i) = fu(i) * (1. - disturbf(i) - disturbo(i))
         fl(i) = fl(i) * (1. - disturbf(i) - disturbo(i))

! constrain the fractional cover
         fu(i) = max(dble(0.25), min(dble(0.975), fu(i)))
         fl(i) = max(dble(0.25), min(dble(0.975), fl(i)))

! annual update upper canopy height parameters
! should be calculated based on vegetative fraction and not the
! average over the entire grid cell
         zbot(i,2) = 3.0
         ztop(i,2) = max(zbot(i,2) + 1.00, 2.50 * totbiou(i) / fu(i) * 0.75)

! ---------------------------------------------------------------------
! * * * update stem area index and sapwood fraction * * *
! ---------------------------------------------------------------------

! estimate stem area index (sai) as a fraction of the lai
         sai(i,1) = 0.050 * totlail(i)
         sai(i,2) = 0.250 * totlaiu(i)

! estimate sapwood fraction of woody biomass
         sapspeed  = 25.0                        ! (m/day)
         trans     = 0.0025                      ! (2.5 mm/day) 
         saparea   = (trans / sapspeed)          ! m**2
         sapvolume = saparea * ztop(i,2) * 0.75  ! m**3
         denswood  = 400.0                       ! kg/m**3
         sapfrac(i) = min (dble(0.50), max (dble(0.05), sapvolume * denswood / wood))
100   continue

! ---------------------------------------------------------------------
! * * * map out vegetation classes for this year * * *
! ---------------------------------------------------------------------
      call vegmap
      return
end  subroutine dynaveg
