#include "inland_config.h"
! Scatter a single real variable from master process
! Varglob is the global variable (input)
! Varloc is the local variable (output)
! Londim is the longitudinal (row) dimension (input)
subroutine scatvar (varglob, varloc, londim)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim
      real*8 :: varglob(londim, plata)
      real*8 :: varloc(londim, beglata:endlata)

#ifndef SPMD
      varloc(:,:) = varglob(:,:)

#else
      integer :: sndct(nproc), displ(nproc+1)
      integer :: i, j, n

      displ(1) = 0
      do 100 n = 1, nproc
         sndct(n) = londim * (cuta(2,n-1) - cuta(1,n-1) + 1)
         displ(n+1) = displ(n) + sndct(n)
100   continue
      
      call mpi_scatterv(varglob, sndct, displ, MPI_REAL8, varloc,sndct(myid+1),&
                        MPI_REAL8, 0, mpicomw, iermpi)
#endif
      return
end subroutine scatvar

! Gather a single real variable to master process
! Varglob is the global variable (output)
! Varloc is the local variable (input)
! Londim is the longitudinal (row) dimension (input)
subroutine gathvar (varglob, varloc, londim)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim
      real*8 :: varglob(londim, plata)
      real*8 :: varloc(londim, beglata:endlata)

#ifndef SPMD
      varglob(:,:) = varloc(:,:)

#else
      integer :: sndct(nproc), displ(nproc+1)
      integer :: i, j, n

      displ(1) = 0
      do 200 n = 1, nproc
         sndct(n) = londim * (cuta(2,n-1) - cuta(1,n-1) + 1)
         displ(n+1) = displ(n) + sndct(n)
200   continue
      
      call mpi_gatherv(varloc, sndct(myid+1), MPI_REAL8, varglob, sndct, displ,&
                       MPI_REAL8, 0, mpicomw, iermpi)
#endif
      return
end subroutine gathvar

! Scatter a single integer variable from master process
! Varglob is the global variable (input)
! Varloc is the local variable (output)
! Londim is the longitudinal (row) dimension (input)
subroutine scatvari (varglob, varloc, londim)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim
      integer :: varglob(londim, plata)
      integer :: varloc(londim, beglata:endlata)

#ifndef SPMD
      varloc(:,:) = varglob(:,:)

#else
      integer :: sndct(nproc), displ(nproc+1)

      integer :: i, j, n

      displ(1) = 0
      do 300 n = 1, nproc
         sndct(n) = londim * (cuta(2,n-1)-cuta(1,n-1)+1)
         displ(n+1) = displ(n) + sndct(n)
300   continue
       
      call mpi_scatterv(varglob,sndct,displ,MPI_INTEGER,varloc,sndct(myid+1), &
                        MPI_INTEGER,0,mpicomw,iermpi)

#endif
      return
end subroutine scatvari

! Gather a single integer variable to master process
! Varglob is the global variable (output)
! Varloc is the local variable (input)
! Londim is the longitudinal (row) dimension (input)
subroutine gathvari (varglob, varloc, londim)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim
      integer :: varglob(londim,plata)
      integer :: varloc(londim,beglata:endlata)

#ifndef SPMD
      varglob(:,:) = varloc(:,:)

#else
      integer :: sndct(nproc), displ(nproc+1)

      integer :: i, j, n

      displ(1) = 0
      do 400 n = 1, nproc
         sndct(n) = londim * (cuta(2,n-1) - cuta(1,n-1) + 1)
         displ(n+1) = displ(n) + sndct(n)
400   continue
       
      call mpi_gatherv(varloc,sndct(myid+1),MPI_INTEGER,varglob,sndct,displ, &
                       MPI_INTEGER,0,mpicomw,iermpi)

#endif
     return
end subroutine gathvari

! Scatter several contiguous real variables from master process
! Varglob is the global variable (input)
! Varloc is the local variable (output)
! Londim is the longitudinal (row) dimension (input)
! Nvars is the number of variables (third dimension) (input)
subroutine scatvarn (varglob, varloc, londim, nvars)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim, nvars
      real*8 :: varglob(londim,plata,nvars)
      real*8 :: varloc(londim,beglata:endlata,nvars)

#ifndef SPMD
      varloc(:,:,:) = varglob(:,:,:)

#else
      real*8 :: sendbuf(londim*plata*nvars)
      real*8 :: recvbuf(londim*plata*nvars)

      integer :: i, j, k, n, ipos, irecv, tag
      integer :: mpistatus(MPI_STATUS_SIZE)

      tag = 81001

      if (myid .eq. 0) then
         do 500 k = 1, nvars
            do 600 j = beglata, endlata
               do 700 i = 1, londim
                  varloc(i,j,k) = varglob(i,j,k)
700            continue
600         continue
500      continue

         do 800 n = 2, nproc
            ipos = 0
            do 900 k = 1, nvars
               do 1000 j = cuta(1,n-1), cuta(2,n-1)
                  do 1100 i = 1, londim
                     ipos = ipos + 1
                     sendbuf(ipos) = varglob(i,j,k)
1100              continue
1000           continue
900         continue
            call mpi_send(sendbuf, ipos, MPI_REAL8, n-1, tag, mpicomw, iermpi)
800      continue
      endif
      if (myid .ne. 0) then
         irecv = londim * (endlata - beglata + 1) * nvars
         call mpi_recv(recvbuf, irecv, MPI_REAL8, 0, tag, mpicomw, mpistatus, &
                       iermpi)
         ipos = 0
         do 1200 k = 1, nvars
            do 1300 j = beglata, endlata
               do 1400 i = 1, londim
                  ipos = ipos + 1
                  varloc(i,j,k) = recvbuf(ipos)
1400           continue
1300        continue
1200     continue
      endif

#endif
     return
end subroutine scatvarn

! Gather several contiguous real variables to master process
! Varglob is the global variable (output)
! Varloc is the local variable (input)
! Londim is the longitudinal (row) dimension (input)
! Nvars is the number of variables (third dimension) (input)
subroutine gathvarn (varglob, varloc, londim, nvars)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim, nvars
      real*8 :: varglob(londim, plata, nvars)
      real*8 :: varloc(londim, beglata:endlata, nvars)

#ifndef SPMD
      varglob(:,:,:) = varloc(:,:,:)

#else
      real*8 :: sendbuf(londim * plata * nvars)
      real*8 :: recvbuf(londim * plata * nvars)

      integer :: i, j, k, n, ipos, irecv, tag
      integer :: mpistatus(MPI_STATUS_SIZE)

      tag = 81002

      if (myid .ne. 0) then
         ipos = 0
         do k 1500 = 1, nvars
            do 1600 j = beglata, endlata
               do 1700 i = 1, londim
                  ipos = ipos + 1
                  sendbuf(ipos) = varloc(i,j,k)
1700           continue
1600        continue
1500     continue
         call mpi_send(sendbuf, ipos, MPI_REAL8, 0, tag, mpicomw, iermpi)
      endif

      if (myid .eq. 0) then
         do 1800 k = 1, nvars
            do 1900 j = beglata, endlata
               do 2000 i = 1, londim
               varglob(i, j, k) = varloc(i, j, k)
2000           continue
1900        continue
1800     continue

         do 2100 n = 2, nproc
            irecv = londim * (cuta(2,n-1) - cuta(1,n-1) + 1) * nvars

            call mpi_recv(recvbuf, irecv, MPI_REAL8, n-1, tag, &
                 mpicomw, mpistatus, iermpi)

            ipos = 0
            do 2200 k = 1, nvars
               do 2300 j = cuta(1,n-1), cuta(2,n-1)
                  do 2400 i = 1, londim
                     ipos = ipos + 1
                     varglob(i,j,k) = recvbuf(ipos)
2400              continue
2300           continue
2200        continue
2100     continue
      endif

#endif
      return
end subroutine gathvarn

! Scatter several contiguous integer variables from master process
! Varglob is the global variable (input)
! Varloc is the local variable (output)
! Londim is the longitudinal (row) dimension (input)
! Nvars is the number of variables (third dimension) (input)
subroutine scatvarni (varglob, varloc, londim, nvars)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim, nvars
      integer :: varglob(londim, plata, nvars)
      integer :: varloc(londim, beglata:endlata, nvars)

#ifndef SPMD
      varloc(:,:,:) = varglob(:,:,:)

#else
      integer :: sendbuf(londim * plata * nvars)
      integer :: recvbuf(londim * plata * nvars)

      integer :: i, j, k, n, ipos, irecv, tag
      integer :: mpistatus(MPI_STATUS_SIZE)

      tag = 81001

      if (myid .eq. 0) then
         do 2500 k = 1, nvars
            do 2600 j = beglata, endlata
               do 2700 i = 1, londim
                  varloc(i,j,k) = varglob(i,j,k)
2700           continue
2600        continue
2500     continue

         do 2800 n = 2, nproc
            ipos = 0
            do 2900 k = 1, nvars
               do 3000 j = cuta(1,n-1), cuta(2,n-1)
                  do 3100 i = 1, londim
                     ipos = ipos + 1
                     sendbuf(ipos) = varglob(i,j,k)
3100              continue
3000           continue
2900        continue

            call mpi_send(sendbuf, ipos, MPI_INTEGER, n-1, &
                 tag, mpicomw, iermpi)
2800     continue
      endif

      if (myid .ne. 0) then
         irecv = londim * (endlata - beglata + 1) * nvars

         call mpi_recv(recvbuf,irecv,MPI_INTEGER,0,tag,mpicomw,mpistatus,   &
                       iermpi)

         ipos = 0
         do 3200 k = 1, nvars
            do 3300 j = beglata, endlata
               do 3400 i = 1, londim
                  ipos = ipos + 1
                  varloc(i,j,k) = recvbuf(ipos)
3400           continue
3300        continue
3200     continue
      endif

#endif
      return
end subroutine scatvarni

! Gather several contiguous integer variables to master process
! Varglob is the global variable (output)
! Varloc is the local variable (input)
! Londim is the longitudinal (row) dimension (input)
! Nvars is the number of variables (third dimension) (input)
subroutine gathvarni (varglob, varloc, londim, nvars)
#ifndef PCMIBIS
# ifdef CAMIBIS
      use mpishorthand
# else
      use comspmd
# endif
#endif
      use ibisparm

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif

      integer :: londim, nvars
      integer :: varglob(londim, plata, nvars)
      integer :: varloc(londim, beglata:endlata, nvars)

#ifndef SPMD
      varglob(:,:,:) = varloc(:,:,:)

#else
      integer :: sendbuf(londim * plata * nvars)
      integer :: recvbuf(londim * plata * nvars)

      integer :: i, j, k, n, ipos, irecv, tag
      integer :: mpistatus(MPI_STATUS_SIZE)

      tag = 81002

      if (myid .ne. 0) then
         ipos = 0
         do 3500 k = 1, nvars
            do 3600 j = beglata, endlata
               do 3700 i = 1, londim
                  ipos = ipos + 1
                  sendbuf(ipos) = varloc(i,j,k)
3700           continue
3600        continue
3500     continue
         call mpi_send(sendbuf, ipos, MPI_INTEGER, 0, tag, mpicomw, iermpi)
      endif

      if (myid .eq. 0) then
         do 3800 k = 1, nvars
            do 3900 j = beglata, endlata
               do 4000 i = 1, londim
                  varglob(i,j,k) = varloc(i,j,k)
4000           continue
3900        continue
3800     continue

         do 4100 n = 2, nproc
            irecv = londim * (cuta(2,n-1) - cuta(1,n-1) + 1) * nvars
            call mpi_recv(recvbuf, irecv, MPI_INTEGER, n-1, tag, mpicomw,    &
                          mpistatus, iermpi)
            ipos = 0
            do 4200 k = 1, nvars
               do 4300 j = cuta(1,n-1), cuta(2,n-1)
                  do 4400 i = 1, londim
                     ipos = ipos + 1
                     varglob(i,j,k) = recvbuf(ipos)
4400              continue
4300           continue
4200        continue
4100     continue
      endif

#endif
     return
end subroutine gathvarni
