#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine gdiag
      use inland_ibispar
      use inland_ibisctl, only: iyear, iyear0
      use inland_comveg
      use inland_comsum

      implicit none

! ------------------------------Variables--------------------------------
! local variables
      integer i

      real*8 :: gnee,      &  ! domain total nee (gt-c/yr)
                gnpp,      &  ! domain total npp (gt-c/yr)
                ggpp,      &  !  domain total gpp (gt-c/yr)
                gbiomass,  &  ! domain total biomass (gt-c)
                galitc,    &  !  domain total aboveground litter carbon (gt-c)
                gblitc,    &  ! domain total belowground litter carbon (gt-c)
                gsoic,     &  ! domain total soil carbon (gt-c)
                gco2soi,   &  ! domain total soil surface co2 flux (gt-c)
                galitn,    &  ! domain total aboveground litter nitrogen (gt-c)
                gblitn,    &  ! domain total belowground litter nitrogen (gt-c)
                gsoin,     &  ! domain total soil nitrogen (gt-c)
                gprcp,     &  ! domain average annual precipitation (mm/yr)
                gaet,      &  ! domain average annual evapotranspiration (mm/yr)
                gt,        &  ! domain average annual transpiration (mm/yr)
                gtrunoff,  &  ! domain average total runoff (mm/yr)
                gsrunoff,  &  ! domain average surface runoff (mm/yr)
                gdrainage, &  ! domain average drainage (mm/yr)
                gdwtot,    &  !   "      "     water recharge (mm/yr)
                gco2mic,   &  !
                gcdist,    &  !
                gtotfall,  &  !
                gtarea,    &  ! total land area of the domain (m**2)
                aratio,    &  ! aet / prcp ratio
                rratio,    &  ! runoff / prcp ratio
                sratio,    &  ! surface runoff / drainage ratio
                tratio,    &  ! transpiration / aet ratio
                gleach        ! domain total leaching rate

#ifdef SPMD
      real*8 :: gtall1(23), gtall2(23)
#endif /* SPMD */

! initialize variables
      gtarea    = 0.0
      gnee      = 0.0
      gnpp      = 0.0
      ggpp      = 0.0
      gbiomass  = 0.0
      galitc    = 0.0
      gblitc    = 0.0
      gsoic     = 0.0
      gco2soi   = 0.0
      galitn    = 0.0
      gblitn    = 0.0
      gsoin     = 0.0
      gprcp     = 0.0
      gaet      = 0.0
      gt        = 0.0
      gtrunoff  = 0.0
      gsrunoff  = 0.0
      gdrainage = 0.0
      gdwtot    = 0.0
      gleach    = 0.0
      gco2mic   = 0.0
      gcdist    = 0.0
      gtotfall  = 0.0

      do 100 i = lbeg, lend
         ayrratio(i) = min (dble(1.0), max (dble(0.0), aytrunoff(i)) / max (dble(0.1), ayprcp(i)))
         aytratio(i) = min (dble(1.0), max (dble(0.0), aytrans(i))  / max (dble(0.1), ayaet(i)))
         gtarea    = gtarea    + garea(i)
         gnee      = gnee      + garea(i) * ayneetot(i) * 1.e-12
         gnpp      = gnpp      + garea(i) * aynpptot(i) * 1.e-12
         ggpp      = ggpp      + garea(i) * aygpptot(i) * 1.e-12
         gbiomass  = gbiomass  + garea(i) * totbiou(i)  * 1.e-12 + &
                     garea(i) * totbiol(i)  * 1.e-12
         galitc    = galitc    + garea(i) * totalit(i)  * 1.e-12
         gblitc    = gblitc    + garea(i) * totrlit(i)  * 1.e-12
         gsoic     = gsoic     + garea(i) * totcsoi(i)  * 1.e-12
         gco2soi   = gco2soi   + garea(i) * ayco2soi(i) * 1.e-12
         galitn    = galitn    + garea(i) * ayanlit(i)  * 1.e-12
         gblitn    = gblitn    + garea(i) * aybnlit(i)  * 1.e-12
         gsoin     = gsoin     + garea(i) * aynsoi(i)   * 1.e-12
         gprcp     = gprcp     + garea(i) * ayprcp(i)
         gaet      = gaet      + garea(i) * ayaet(i)
         gt        = gt        + garea(i) * aytrans(i)
         gsrunoff  = gsrunoff  + garea(i) * aysrunoff(i)
         gdrainage = gdrainage + garea(i) * aydrainage(i)
         gdwtot    = gdwtot    + garea(i) * dwtot(i)    * nytime(1)
         gleach    = gleach    + garea(i) * yrleach(i)  * 1.e-12
         gco2mic   = gco2mic   + garea(i) * ayco2mic(i) * 1.e-12
         gcdist    = gcdist    + garea(i) * cdisturb(i) * 1.e-12
         gtotfall  = gtotfall  + garea(i) * (falll(i) + fallr(i) + &
                     fallw(i)) * 1.e-12
100   continue
      gtrunoff  = gsrunoff  + gdrainage

#ifdef SPMD
! sum over parallel tasks
      gtall1(1) = gtarea
      gtall1(2) = gnpp
      gtall1(3) = gnee
      gtall1(4) = ggpp
      gtall1(5) = gbiomass
      gtall1(6) = galitc
      gtall1(7) = gblitc
      gtall1(8) = gsoic
      gtall1(9) = gco2soi
      gtall1(10) = galitn
      gtall1(11) = gblitn
      gtall1(12) = gsoin
      gtall1(13) = gprcp
      gtall1(14) = gaet
      gtall1(15) = gt
      gtall1(16) = gtrunoff
      gtall1(17) = gsrunoff
      gtall1(18) = gdrainage
      gtall1(19) = gdwtot
      gtall1(20) = gleach
      gtall1(21) = gco2mic
      gtall1(22) = gcdist
      gtall1(23) = gtotfall
    
      call MPI_ALLREDUCE(gtall1,gtall2,23,MPI_REAL8,MPI_SUM,mpicomw,iermpi)

      gtarea = gtall2(1)
      gnpp = gtall2(2)
      gnee = gtall2(3)
      ggpp = gtall2(4)
      gbiomass = gtall2(5)
      galitc = gtall2(6)
      gblitc = gtall2(7)
      gsoic = gtall2(8)
      gco2soi = gtall2(9)
      galitn = gtall2(10)
      gblitn = gtall2(11)
      gsoin = gtall2(12)
      gprcp = gtall2(13)
      gaet = gtall2(14)
      gt = gtall2(15)
      gtrunoff = gtall2(16)
      gsrunoff = gtall2(17)
      gdrainage = gtall2(18)
      gdwtot = gtall2(19)
      gleach = gtall2(20)
      gco2mic = gtall2(21)
      gcdist = gtall2(22)
      gtotfall = gtall2(23)
#endif

      gprcp     = gprcp     / gtarea
      gaet      = gaet      / gtarea
      gt        = gt        / gtarea
      gtrunoff  = gtrunoff  / gtarea
      gsrunoff  = gsrunoff  / gtarea
      gdrainage = gdrainage / gtarea
      gdwtot    = gdwtot    / gtarea
      aratio   = gaet     / gprcp
      rratio   = gtrunoff / gprcp
      sratio   = gsrunoff / max(dble(0.01),gtrunoff)
      tratio   = gt       / gaet

! write some diagnostic output to history file if I am the first proccess
      if (myid .eq. 0) then
! Fichiers 200 (ibis.out.annual) et 201 (ibis.out.global) ouverts
! dans ibisini.F !FIXME: what language is this comment in?
         write (200,*) ' '
         write (200,*) '* * * annual diagnostic fields * * *',iyear
         write (200,*) ' '
         write (200,9001) gnee
         write (200,9000) gnpp
         write (200,9002) ggpp
         write (200,9010) gbiomass
         write (200,9020) galitc
         write (200,9021) gblitc
         write (200,9030) gsoic
         write (200,9032) gco2soi
         write (200,9034) galitn
         write (200,9036) gblitn
         write (200,9038) gsoin
         write (200,*) ' '
         write (200,9040) gprcp
         write (200,9050) gaet
         write (200,9060) gt
         write (200,9070) gtrunoff
         write (200,9080) gsrunoff
         write (200,9090) gdrainage
         write (200,9095) gdwtot
         write (200,*) ' '
         write (200,9100) aratio
         write (200,9110) rratio
         write (200,*) ' '
         write (200,9120) tratio
         write (200,9130) sratio
         write (200,*) ' '
!        call flush (200)
         write (201,9500) iyear, gnee, gnpp, gtotfall, gcdist, gco2mic,        &
                          gleach, ggpp, gbiomass, gsoic, gsoin, galitc,        &
                          gblitc, gco2soi, gprcp, gaet, gt, gsrunoff, gdrainage
!        call flush (201)

         write (STDOUT,*) ' '
         write (STDOUT,*) '* * * annual diagnostic fields * * *',iyear
         write (STDOUT,*) ' '
         write (STDOUT,9001) gnee
         write (STDOUT,9000) gnpp
         write (STDOUT,9002) ggpp
         write (STDOUT,9010) gbiomass
         write (STDOUT,9020) galitc
         write (STDOUT,9021) gblitc
         write (STDOUT,9030) gsoic
         write (STDOUT,9032) gco2soi
         write (STDOUT,9034) galitn
         write (STDOUT,9036) gblitn
         write (STDOUT,9038) gsoin
         write (STDOUT,*) ' '
         write (STDOUT,9040) gprcp
         write (STDOUT,9050) gaet
         write (STDOUT,9060) gt
         write (STDOUT,9070) gtrunoff
         write (STDOUT,9080) gsrunoff
         write (STDOUT,9090) gdrainage
         write (STDOUT,9095) gdwtot
         write (STDOUT,*) ' '
         write (STDOUT,9100) aratio
         write (STDOUT,9110) rratio
         write (STDOUT,*) ' '
         write (STDOUT,9120) tratio
         write (STDOUT,9130) sratio
         write (STDOUT,*) ' '
      endif
 
9000  format (1x,'total npp             of the domain (gt-c/yr) : ', f24.14)
9001  format (1x,'total nee             of the domain (gt-c/yr) : ', f24.14)
9002  format (1x,'total gpp             of the domain (gt-c/yr) : ', f24.14)
9010  format (1x,'total biomass         of the domain (gt-c)    : ', f24.14)
9020  format (1x,'aboveground litter    of the domain (gt-c)    : ', f24.14)
9021  format (1x,'belowground litter    of the domain (gt-c)    : ', f24.14)
9030  format (1x,'total soil carbon     of the domain (gt-c)    : ', f24.14)
9032  format (1x,'total soil co2 flux   of the domain (gt-c)    : ', f24.14)
9034  format (1x,'aboveground litter n  of the domain (gt-c)    : ', f24.14)
9036  format (1x,'belowground litter n  of the domain (gt-c)    : ', f24.14)
9038  format (1x,'total soil nitrogen   of the domain (gt-c)    : ', f24.14)
9040  format (1x,'average precipitation of the domain (mm/yr)   : ', f24.14)
9050  format (1x,'average aet           of the domain (mm/yr)   : ', f24.14)
9060  format (1x,'average transpiration of the domain (mm/yr)   : ', f24.14)
9070  format (1x,'average runoff        of the domain (mm/yr)   : ', f24.14)
9080  format (1x,'average surf runoff   of the domain (mm/yr)   : ', f24.14)
9090  format (1x,'average drainage      of the domain (mm/yr)   : ', f24.14)
9095  format (1x,'average moisture recharge of the domain (mm/yr) : ', f24.14)
9100  format (1x,'total aet      / precipitation                : ', f24.14)
9110  format (1x,'total runoff   / precipitation                : ', f24.14)
9120  format (1x,'transpiration  / total aet                    : ', f24.14)
9130  format (1x,'surface runoff / total runoff                 : ', f24.14)
9500  format (1x,i4,18f24.14)
      return
end subroutine gdiag 
