#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine ibis(nstep, loni, lati, mcsec, calday, calday1, doalb, loopi, kpti, kptj)
! ---------------------------------------------------------------------
      use com1d
      use ibispar
      use ibisctl
      use comveg
      use comsoi
      use comatm
      use comsno
      use comsum

      implicit none
!------------------------------Arguments--------------------------------
! Input arguments
      logical :: doalb             	! true if surface albedo calculation time step
      integer :: nstep
      integer :: loopi             	! index of little vector in big vector
      integer :: kpti            	! index of 1st point of little vector in big lpt vector
      integer :: kptj            	! index of last point of little vector
      integer :: mcsec             	! current seconds in day (0 - (86400 - dtime))

      real*8 :: loni(npoi)
      real*8 :: lati(npoi)
      real*8 :: calday               ! current julian day (1-365.99)	decimals=fraction of day
      real*8 :: calday1              ! julian day at nstep + 1

!     (used when doalb is true)

! ---------------------Local variables ----------------------------------
      integer :: spin     ! counter for iterations used to spin up soilbgc
      integer :: i, k, ib ! loop indices
      real*8 :: rn
      real*8 :: slope
!
! ----------------------------------------------------------------------
!
! Compute current time step zenith angle
      call ibiszen (calday, loni, lati, coszen, kpti, kptj)

      if (mcsec .eq. dtime) then

! Calculates phenology once a day (beginning of the day)
         call pheno(kpti, kptj)

!       if (ipointout .eq. 1 .and. myid .eq. 0) then
         if (ipointout .eq. 1) then
            i = 1001
            if (i .ge. kpti .and. i .le. kptj) then
               write (103,'(32f10.3)') lai(i,1),lai(i,2),fu(i),fl(i), &
                      sai(i,1),sai(i,2),(frac(i,k),k=1,12),zbot(i,1), &
                      ztop(i,1)
!             call flush(103)
            end if

            i = 662
            if (i .ge. kpti .and. i .le. kptj) then
               write (1103,'(4e24.14)') lai(i,1),lai(i,2),fu(i),fl(i), &
                      sai(i,1),sai(i,2),(frac(i,k),k=1,12),zbot(i,1),  &
                      ztop(i,1)
!             callflush(1103)
            end if

            i = 1142
            if (i .ge. kpti .and. i .le. kptj) then
               write (2103,'(4e24.14)') lai(i,1),lai(i,2),fu(i),fl(i), &
                      sai(i,1),sai(i,2),(frac(i,k),k=1,12),zbot(i,1),  &
                      ztop(i,1)
!             call flush(2103)
            end if
         end if

         if (isimveg .eq. 1) then

! call soil biogeochemistry model

! Soil carbon acceleration model deleted in favor of spinmax
! specification at each restart (AAM - 3/14/02)
!
!          if (soicspin .eq. 1) then
!             if ((iyear - iyear0) .le.
!    >          (spinfrac * (nspinsoil - iyear0 - eqyears))) then
!                spinmax = int(spincons)
!             else if ((iyear - iyear0) .lt.
!    >              (nspinsoil - iyear0 -  eqyears)) then
!                slope   = spincons / ((nspinsoil - iyear0 - eqyears) -
!    >                    (spinfrac * (nspinsoil - iyear0 - eqyears)))
!                spinmax = int (spincons - (slope * ((iyear - iyear0) -
!    >                (spinfrac * (nspinsoil - iyear0 - eqyears)))))
!                spinmax = max(spinmax,1)
!             else
!                spinmax = 1
!             endif            ! if (iyear - iyear0) ....
!          else
!             spinmax = 1
!          endif               ! if (soicspin = 1)

      do 230 spin = 1, spinmax
         call soilbgc (iyear, iyear0, imonth, iday, nspinsoil, &
                       spin, spinmax, ipointout, kpti, kptj)
230   continue
         endif                  ! if (isimveg = 1)
      endif                    ! if (msec < dtime)

      call lsxmain(loopi, kpti, kptj)

! accumulate variables every timestep

      call sumnow(kpti, kptj)

      call sumday (mcsec, loopi, kpti, kptj)
      call summonth (mcsec,iday, imonth, loopi, kpti, kptj)
      call sumyear  (mcsec,iday, imonth, loopi, kpti, kptj)

      if (doalb) then

! Compute next time step zenith angle
         call ibiszen (calday1, loni, lati, coszen, kpti ,kptj)

! Compute albedos (used in next step radiation computations)
! set up for solar calculations
         call solset(loopi, kpti, kptj)

! solar calculations for each waveband
         do 100 ib = 1, nband

! solsur sets surface albedos for soil and snow
! solalb performs the albedo calculations
! solarf uses the unit-incident-flux results from solalb
! to obtain absorbed fluxes sol[u,s,l,g,i] and
! incident pars sunp[u,l]
            call solsur (ib,loopi)
            call solalb (ib,loopi)
            call solarf (ib,loopi)
100      continue
      end if
!     if (ipointout .eq. 1 .and. myid .eq. 0) then
      if (ipointout .eq. 1) then
         i = 1001
            if (i .ge. kpti .and. i .le. kptj) then
               rn = solad(i,1)*(1-asurd(i,1)) &
                  + solad(i,2)*(1-asurd(i,2)) &
                  + solai(i,1)*(1-asuri(i,1)) &
                  + solai(i,2)*(1-asuri(i,2)) &
                  + fira(i) - firb(i)
               write (200,'(4e24.14)') calday, rn,-fsena(i), &
                  -fvapa(i)*hvap, soihfl(i)
               write (120,'(4e24.14)') coszen(i),topparu(i),topparl(i), &
                  solad(i,1),solai(i,1),solad(i,2),solai(i,2),       &
                  asurd(i,1),asuri(i,1),asurd(i,2),asuri(i,2)
!             call flush(200)
!             call flush(120)
            end if

            i = 662
            if (i .ge. kpti .and. i .le. kptj) then
               rn = solad(i,1)*(1-asurd(i,1))    &
                  + solad(i,2)*(1-asurd(i,2))    &
                  + solai(i,1)*(1-asuri(i,1))    &
                  + solai(i,2)*(1-asuri(i,2))    &
                  + fira(i) - firb(i)
               write (1100,'(4e24.14)') calday, rn,-fsena(i), &
                     -fvapa(i)*hvap,soihfl(i)
               write (1120,'(4e24.14)') coszen(i),topparu(i),topparl(i),&
                  solad(i,1),solai(i,1),solad(i,2),solai(i,2), &
                  asurd(i,1),asuri(i,1),asurd(i,2),asuri(i,2)
!             call flush(1100)
!             call flush(1120)
            end if

            i = 1142
            if (i .ge. kpti .and. i .le. kptj) then
               rn = solad(i,1)*(1-asurd(i,1)) &
                  + solad(i,2)*(1-asurd(i,2)) &
                  + solai(i,1)*(1-asuri(i,1)) &
                  + solai(i,2)*(1-asuri(i,2)) &
                  + fira(i) - firb(i)
               write (2100,'(4e24.14)') calday, rn,-fsena(i), &
                     -fvapa(i)*hvap,soihfl(i)
               write (2120,'(4e24.14)') coszen(i),topparu(i),topparl(i),&
                     solad(i,1),solai(i,1),solad(i,2),solai(i,2), &
                     asurd(i,1),asuri(i,1),asurd(i,2),asuri(i,2)
!         call flush(2100)
!         call flush(2120)
         end if
      end if
      return
end subroutine ibis
