#include "inland_config.h"
! ---------------------------------------------------------------------
     subroutine ibisdrv(pgcmxy, psrfxy, ubotxy, vbotxy, tbotxy, &
     					qbotxy, flwdsxy, prcxy, prlxy, solsxy,  &
     					sollxy, solsdxy, solldxy, doalb,nstep,  &
#ifndef CAMIBIS
                        mdbase, msbase, mbdate, mbsec,          &
#endif
                        dtibis, tsxy, shxy, lhxy, cfxy, tauxxy, &
     					tauyxy, trefxy, asdirxy, asdifxy,       &
     					aldirxy,								&
#ifdef ATMCO2
                        land_atm_exchange,                      &
#endif
                        aldifxy, lwupxy, h2osnoxy,				&
#ifdef COUP_CSM
                        qrunoffxy, qrefxy, swabxy,				&
#endif
#if (defined STAND_ALONE || defined COUP_CSM)
                        restart_written,						&
#endif
#ifdef PCMIBIS
                        qoverxy, qdraixy,						&
#endif
                        co2vmrgcm)

! -----------------------------------------------------------------------
!
! Interface routine for the Integrated Biosphere Simulator (IBIS)
!          Foley et al, 1996, Kucharik et al, 2000
!
! ---------------------------Code history--------------------------------
!
! Original Version: lsmdrv, G. Bonan, Feb. 1996
! Standardized:
!     Multitasked version: A. Hahmann, July 1996
!     IBIS's version: C. Delire, July 1998
!     MPI version: Art Mirin, 2002
!
! ---------------------------------------------------------------------
     use com1d
     use lsmmapib
     use ibisparm
     use ibiscst
     use ibisctl
     use intermap
     use comveg
     use comsoi
     use comatm
     use comsno
     use comhyd
     use comsum
#ifdef CAMIBIS
     use time_manager, only: get_curr_date, get_curr_calday
#endif
#ifdef ATMCO2
     use atmco2, only : 							&
         atmco2_DebugPrint => DebugPrint, 			&
         atmco2_GetCO2Index => GetCO2Index,			&
     	 atmco2_UseProgCO2 => UsePrognosticCO2		&
#endif
!
     implicit none
! ------------------------------Arguments--------------------------------
! Input arguments
!
# ifdef ATMCO2
     real*8 :: land_atm_exchange  ! the globally summed carbon exchange
# endif
     integer :: nstep             ! atm time step index
     logical :: doalb             ! true if surface albedo calculation time step
#ifndef CAMIBIS
     integer :: mdbase            ! Base day of run
     integer :: msbase            ! Base seconds of base day
     integer :: mbdate            ! Base date of run (yymmdd format)
     integer :: mbsec             ! Base seconds of base date
#endif
     real*8 :: dtibis             ! time step as passed from GCM, dtime from
                                  ! ibiscst.F is set equal to dtibis here
     real*8 :: pgcmxy(plonda,beglata:endlata)   ! bottom level gcm pressure [Pa]
     real*8 :: psrfxy(plonda,beglata:endlata)   ! surface gcm pressure [Pa]
     real*8 :: ubotxy(plonda,beglata:endlata)   ! atm bottom level zonal wind [m/s]
     real*8 :: vbotxy(plonda,beglata:endlata)   ! atm bottom level meridional wind [m/s]
     real*8 :: tbotxy(plonda,beglata:endlata)   ! atm bottom level temperature [K]
     real*8 :: qbotxy(plonda,beglata:endlata)   ! atm btm level specifi! humidity [kg/kg]
     real*8 :: flwdsxy(plonda,beglata:endlata)  !atm downward longwave on surface (w/m**2)
     real*8 :: prcxy(plonda,beglata:endlata)    ! atm convective precipitation rate [mm h2o/s]
     real*8 :: prlxy(plonda,beglata:endlata)    ! atm large-scale precipitation rate [mm h2o/s]
     real*8 :: solsxy(plonda,beglata:endlata)   ! atm vis direct beam solar rad onto srf [w/m**2]
     real*8 :: sollxy(plonda,beglata:endlata)   ! atm nir direct beam solar rad onto srf[w/m**2]
     real*8 :: solsdxy(plonda,beglata:endlata)  ! atm vis diffuse solar rad onto srf [W/m**2]
     real*8 :: solldxy(plonda,beglata:endlata)  ! atm nir diffuse solar rad onto srf [W/m**2]
!
! Output arguments
!
     real*8 :: co2vmrgcm            ! modified co2 volume mixing ratio
!
     real*8 :: shxy(plonda,beglata:endlata)     ! sensible heat flux (w/m**2) [+ to atm]
     real*8 :: lhxy(plonda,beglata:endlata)     ! latent heat flux (w/m**2) [+ to atm]
#ifndef PCMIBIS
     real*8 :: cfxy(plonda,pcnsta+pnatsa,beglata:endlata) ! const fluxes (kg/m**2/s) [+ to atm]
#else
     real*8 :: cfxy(plonda,pcnsta,beglata:endlata) ! const fluxes (kg/m**2/s) [+ to atm]
#endif
     real*8 :: tauxxy(plonda,beglata:endlata)   ! zonal surface stress (kg/m/s**2)
     real*8 :: tauyxy(plonda,beglata:endlata)   ! meridional surface stress (kg/m/s**2)
     real*8 :: tsxy(plonda,beglata:endlata)     ! surface radiative temperature (kelvin)
     real*8 :: lwupxy(plonda,beglata:endlata)   ! longwave up flux (W/m**2)
     real*8 :: trefxy(plonda,beglata:endlata)   ! 2 m height air temperature (kelvin)
     real*8 :: asdirxy(plonda,beglata:endlata)  ! albedo, visible, direct
     real*8 :: asdifxy(plonda,beglata:endlata)  ! albedo, visible, diffuse
     real*8 :: aldirxy(plonda,beglata:endlata)  ! albedo, near infrared, direct
     real*8 :: aldifxy(plonda,beglata:endlata)  ! albedo, near infrared, diffuse
!
! Additional output variables
! return these land surface fields to atmospheri! model so they can
! be written to atm history fields
!
     real*8 :: h2osnoxy(plonda,beglata:endlata)!
#ifdef COUP_CSM
     real*8 :: qrunoffxy(plonda,beglata:endlata)! surface runoff rate (kg_h2o m-2 s-1)
     real*8 :: qrefxy(plonda,beglata:endlata)   ! 2 m height specifi! humidity (kg_h2o/kg_air)
     real*8 :: swabxy(plonda,beglata:endlata)   ! incident solar radiation (W/m**2)
#endif
! The variable restart_written is local in some cases
     integer :: restart_written

#ifdef PCMIBIS
! Return these to ccm for transfer to ocean model
     real*8 :: qoverxy(plonda,beglata:endlata)  ! Runoff
     real*8 :: qdraixy(plonda,beglata:endlata)  ! Drainage
#endif

!
! --------Local variables (input/output of IBIS and calendr)---------------
!
     integer :: i,j               ! longitude/latitude index
!      integer :: iyear0            ! initial year (eg: 60)
!      integer :: iyear             ! current year (eg: 68)
     integer :: iyearprev         ! year of previous timestep
!      integer :: imonth            ! current month (1 - 12)
     integer :: imonthprev        ! month of previous timestep
!      integer :: iday              ! current day in month (1-31 or 1-30, or 1-28)
     integer :: idayprev          ! day in month of previous timestep
     integer :: k                 ! grid point index [1-lpt]
     integer :: kend              ! last index of little vector
     integer :: mcsec             ! current seconds in day (0 - (86400 - dtime))
     integer :: mcsecrun          ! seconds since beginning of run
     integer :: mcsec1            ! second of next time step
     integer :: mdcur             ! current day of simulation (0 - #day at end of sim)
#ifndef CAMIBIS
     integer :: mscur
     integer :: mcdate            ! date of current time step
     integer :: mdcur1            ! day of next time step
     integer :: mscur1
     integer :: mcdate1           ! date of next time step
#endif
     integer :: ndyn              ! number of iteration of dynaveg (spin up)
     real*8 :: ratio
     real*8 :: calday             ! current julian day (1-365.99) decimals=fraction of day
     real*8 :: calday1            ! julian day at nstep + 1 (used when doalb is true)
     real*8 ::  gnee              ! domain total nee (gt-c/yr)
     real*8 ::  gleach            ! domain total leaching rate (gt-c/yr)
     real*8, dimension(:,:), allocatable :: tempxy1, tempxy2, tempxy3
     real*8, dimension(:), allocatable :: tempkn1, tempkn2, tempkn3, tempkn4
!
! ---------------------Local variables (output of IBIS)------------------
!
! -----------------------------------------------------------------------
!
     real*8 :: ibis_global_sum
     integer :: co2_idx
!
     integer :: ndaypm(12)
     data ndaypm /31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
!
!      save iyear, imonth, iday, iyear0
!
! Redefenition of dtime to pass into ibis's common block ibiscst.h from
! CCM3's common block, comtim.h
!
     dtime = dtibis
     co2con! = co2vmrgcm
!
     allocate(tempxy1(plonda,beglata:endlata))
     allocate(tempxy2(plonda,beglata:endlata))
     allocate(tempxy3(plonda,beglata:endlata))
     allocate(tempkn1(lbeg:lend))
     allocate(tempkn2(lbeg:lend))
     allocate(tempkn3(lbeg:lend))
     allocate(tempkn4(lbeg:lend))
!
     do 100 j = beglata, endlata
        do 200 i = 1, plonda
           tempxy1(i,j) = 0.
           tempxy2(i,j) = 0.
           tempxy3(i,j) = 0.
 200    continue
 100 continue
!
     do 300 k = lbeg, lend
        tempkn1(k) = 0.
        tempkn2(k) = 0.
        tempkn3(k) = 0.
        tempkn4(k) = 0.
        caccount(k) = 0.
 300 continue
!
     iyearprev = iyear
     imonthprev = imonth
     idayprev = iday

#ifdef CAMIBIS
     call get_curr_date(iyear, imonth, iday, mcsec)
        calday = get_curr_calday()
        mcsecrun = nstep * nint(dtime)
        mdcur = mod(mcsecrun,86400)
#else
!
! Calendar information for current time step
!
     call calendr (nstep, dtime, mdbase, msbase, mbdate,  &
          mbsec, mdcur, mscur, mcdate, mcsec, calday)
!
        iyear = mcdate/10000
        imonth = mod(mcdate,10000)/100
        iday = mod(mcdate,100)
#endif
!
! sigma of 1st atmospheri! level calculated here because nstep >< 0
! at restart
!
     if (iday .ne. idayprev .and. myid .eq. 0) then
        write (6,901) nstep, iday, imonth, iyear, iyear0, calday, &
              idayprev, imonthprev, iyearprev
 901    format('IBISDRV: step day month year year0', 	&
              '         calday   prev d_mo_yr', /,		&
              5x,i8,i4,i6,i5,i6,f15.6,3i5)
     endif
!
     if (doalb) then
#ifdef CAMIBIS
        calday1 = get_curr_calday(nint(dtime))
#else
        call  calendr (nstep+1, dtime, mdbase, msbase, mbdate, 		&
              mbsec  ,mdcur1, mscur1, mcdate1 ,mcsec1  ,calday1  )
#endif
     end if
!
! All variables should be in "long" land vector for multitasking over "short"
! vectors
!
     call ccm_to_ibis(tbotxy, ta, plonda, beglata, endlata, 1)
     call ccm_to_ibis(ubotxy, ux, plonda, beglata, endlata, 1)
     call ccm_to_ibis(vbotxy, uy, plonda, beglata, endlata, 1)
     call ccm_to_ibis(qbotxy, qa, plonda, beglata, endlata, 1)
     call ccm_to_ibis(psrfxy, psurf, plonda, beglata, endlata, 1)
     call ccm_to_ibis(pgcmxy, pbota, plonda, beglata, endlata, 1)
!
     do 400 k = njbeg(myid+1), njend(myid+1)
        i = ixy(k)
        j = jxy(k)
        tempxy1(i,j) = sqrt(ubotxy(i,j)**2 + vbotxy(i,j)**2)
        ratio = 1.0 / (1.0 + 1.61 * (1.35**(tbotxy(i,j)-273.15)))
           if (tbotxy(i,j).gt.278.15) ratio = 0.0
              tempxy2(i,j) = (prcxy(i,j) + prlxy(i,j))* ratio
              tempxy3(i,j) = (prcxy(i,j) + prlxy(i,j))* (1.0 - ratio)
 400 continue
     call ccm_to_ibis(tempxy1, ua, plonda, beglata, endlata, 1)
     call ccm_to_ibis(tempxy2, snowa, plonda, beglata, endlata, 1)
     call ccm_to_ibis(tempxy3, raina, plonda, beglata, endlata, 1)
     call ccm_to_ibis(solsxy, solad(lbeg,1), plonda, beglata, endlata, 1)
     call ccm_to_ibis(sollxy, solad(lbeg,2), plonda, beglata, endlata, 1)
     call ccm_to_ibis(solsdxy, solai(lbeg,1), plonda, beglata, endlata, 1)
     call ccm_to_ibis(solldxy, solai(lbeg,2), plonda, beglata, endlata, 1)
     call ccm_to_ibis(flwdsxy, fira, plonda, beglata, endlata, 1)
!
! For first time step and initial fields not read, re-initialize
! temperatures and snow according to CCM3 initial conditions since
! these were not yet available in BATSINI
!
!
! Call IBIS
!
!$omp  parallel do
!$omp& default(shared),
!$omp& private(i,k,kend),
!$omp& schedule(runtime)
!
     do 500 i = 1, numlv
        k = begpnt(i)
        kend = begpnt(i) + numpnt(i) - 1
        call ibis (nstep, loni, lati, mcsec, calday, &
                  calday1, doalb, i, k, kend)
 500 continue
!
     restart_written = 0
!
     if (nstep .ne. 0) then
!
! write out daily output
!
        if (idayprev .ne. iday) then
           if (idayout.eq.1) then
              if (myid .eq. 0) then
                 write (*,*) ' '
                 write (*,*) 'writing daily output'
                 write (*,*) ' '
              endif
              call wdaily (int(calday-1), iyear, iyear0, iyearprev)
           endif
!
! end of calculations done once a day
!
        end if
!
        if (imonthprev .ne. imonth) then
!
           do 600 i = lbeg, lend
              tcthis(i) = min (tcthis(i), (amts2(i) - 273.16))
              twthis(i) = max (twthis(i), (amts2(i) - 273.16))
 600       continue
!
! write out monthly output (use 1st day of this month because of GrADS bug)
!
           if (imonthout.eq.1) then
              if (myid .eq. 0) then
                 write (*,*) ' '
                 write (*,*) 'writing monthly output'
                 write (*,*) ' '
              endif
              call wmonthly (mdcur-ndaypm(imonthprev)+1, 		&
                             imonthprev, iyearprev, iyear0)
           endif
!
! end of calculations done once a month
!
        end if
!
! calculations done once a year
!
        if (iyearprev .ne. iyear) then
!
           if (myid .eq. 0) then
              write(*,*) 'IBISDRV, yearly',nstep, calday, iyear, iyearprev
           endif
!
! perform vegetation dynamics
!
           ndyn = 1
!
           do 700 j = 1, ndyn
              if (isimveg.ne.0) call dynaveg (isimfire)
 700       continue
!
! calculate simple annual diagnosti! variables for the globe
!
           call gdiag (iyearprev,iyear0, gnee, gleach)
!
!           co2vmrgcm = co2vmrgcm -
!    >                  gnee *1e12 * 28.9644 / 5.267e18 / 12.

           if (myid .eq. 0) then
              write(*,*) 'IBISDRV, yearly', co2vmrgcm
           endif
!
!
! calculate simple annual diagnosti! variables for each vegetation type
!
           call vdiag (iyearprev, iyear0)
!
! write out annual output
!
           if (iyearout.eq.1) then
              if (myid .eq. 0) then
                 write (*,*) ' '
                 write (*,*) 'writing annual output'
                 write (*,*) ' '
              endif
              call wyearly (mdcur, iyearprev, iyear0)
           endif
!
! recalculate bioclimati! parameters (used in dynaveg, calculated
! even in fixed vegetation case when fixed vegetation is an
! initialisation of a dynami! run)
!
           call climanl2 (isimveg)
!
! End of calculation done once a year
!
        end if
!
        if (imonthprev .ne. imonth) then
!
! write restart files
!
           call wrestart (mdcur, imonthprev, iyearprev, iyear0)
           restart_written = 1
        endif
!
!     End of test on 1st time step
!
     end if
!
! Transfer fields from boundary package
! tneetot = (tnpptot - co2mi! - cdisturb) in mol_! / m**2 / s  (positive to land)
! Net carbon flux = tneetot - leaching + correction calculated once a year (see dynaveg)
! Hypothesis: amount of carbon leached is directly and completely released to the atmosphere
!         (degassing from the streams and lakes)
! Leaching calculated in soilbg! in mol_! m-2 s-1
! caccount calculated in dynaveg in kg_! m-2 yr-1 but all applied in the last time step ==>
!          in ibisdrv, caccount in  units of kg_! m-2 timestep-1
!
! Return fluxes and sf! temps to CCM3 ([plon] x [plat] grid) from BATS;
! no fluxes or sf! temp returned for ocean or sea-ice.
!
     do 800 k = lbeg, lend
        tempkn1(k) = -fsena(k)
        tempkn2(k) = -fvapa(k)
        tempkn3(k) = -fvapa(k)*hvap
        tempkn4(k) = tneetot(k) - cleach(k) + caccount(k) / ( dtime * 12.e-3 )
 800 continue
!
#ifdef ATMCO2
!       Now, globally sum this carbon flux with a routine in gdiag,
!       returning it to the atmosphere via the argument list.  Units of
!       land_atm_exchange should be moles, positive to atmosphere.
!
     if ( atmco2_UseProgCO2() ) then
        land_atm_exchange = -1.0 * ibis_global_sum( tempkn4 ) * dtime
        call atmco2_DebugPrint( "tneetot = ",						&
                                ibis_global_sum( tneetot ) * dtime )
        call atmco2_DebugPrint( "cleach = ",						&
                                ibis_global_sum( cleach ) * dtime )
        call atmco2_DebugPrint( "caccount = ",						&
        						ibis_global_sum( caccount ) / 12.e-3 )
     endif
#endif
!
     call ibis_to_ccm(shxy, tempkn1, plonda, beglata, endlata, 1)
     call ibis_to_ccm(tempxy1, tempkn2, plonda, beglata, endlata, 1)
     call ibis_to_ccm(lhxy, tempkn3, plonda, beglata, endlata, 1)
     call ibis_to_ccm(tempxy2, tempkn4, plonda, beglata, endlata, 1)
!
     do 900 k = njbeg(myid+1), njend(myid+1)
        i = ixy(k)
        j = jxy(k)
        cfxy(i,1,j) = tempxy1(i,j)
 900 continue
 !
     if (pcnsta .gt. 1) then
#ifdef ATMCO2
!         tempkn4, and therefore tempxy2, has units of mol / m^2 / s,
!         positive to land.  We convert this to kg CO2 / m^2 / se! (44
!         g/mol = 0.044 kg/mol) and change sign.
        co2_idx = atmco2_GetCO2Index()
        cfxy(:,co2_idx,:) = 0.
!
        do 1000 k = njbeg(myid+1), njend(myid+1)
           i = ixy(k)
           j = jxy(k)
           cfxy(i,co2_idx,j) = -0.044 * tempxy2(i,j)
 1000   continue
#else
!         tempkn4, and therefore tempxy2, has units of mol / m^2 / s,
!         positive to land.  We convert this to kg CO2 / m^2 / se! (44
!         g/mol = 0.044 kg/mol) and change sign.
        do 1100 k = njbeg(myid+1), njend(myid+1)
           i = ixy(k)
           j = jxy(k)
           cfxy(i,2,j) = -0.044 * tempxy2(i,j)
 1100   enddo
#endif
     endif
!
     do 1200 k = lbeg, lend
        tempkn1(k) = -taux(k)
        tempkn2(k) = -tauy(k)
        tempkn3(k) = (firb(k)/stef)**0.25
 1200 continue
!
     call ibis_to_ccm(tauxxy, tempkn1, plonda, beglata, endlata, 1)
     call ibis_to_ccm(tauyxy, tempkn2, plonda, beglata, endlata, 1)
     call ibis_to_ccm(tsxy, tempkn3, plonda, beglata, endlata, 1)
     call ibis_to_ccm(lwupxy, firb, plonda, beglata, endlata, 1)
     call ibis_to_ccm(trefxy, ts2, plonda, beglata, endlata, 1)
     call ibis_to_ccm(asdirxy, asurd(lbeg,1), plonda, beglata, endlata, 1)
     call ibis_to_ccm(aldirxy, asurd(lbeg,2), plonda, beglata, endlata, 1)
     call ibis_to_ccm(asdifxy, asuri(lbeg,1), plonda, beglata, endlata, 1)
     call ibis_to_ccm(aldifxy, asuri(lbeg,2), plonda, beglata, endlata, 1)
!
     do 1300 k = lbeg, lend
        tempkn1(k) = rhos*(hsno(k,1)+hsno(k,2)+hsno(k,3))
 1300 continue
!
     call ibis_to_ccm(h2osnoxy, tempkn1, plonda, beglata, endlata, 1)
#ifdef PCMIBIS
     call ibis_to_ccm(qoverxy, grunof, plonda, beglata, endlata, 1)
     call ibis_to_ccm(qdraixy, gdrain, plonda, beglata, endlata, 1)
#endif
#ifdef COUP_CSM
     do 1400 k = lbeg, lend
        tempkn1(k) = solad(k,1) * (1. - asurd(k,1)) &
                   + solad(k,2) * (1. - asurd(k,2)) &
                   + solai(k,1) * (1. - asuri(k,1)) &
                   + solai(k,2) * (1. - asuri(k,2))
 1400 continue
!
     call ibis_to_ccm(swabxy, tempkn1, plonda, beglata, endlata, 1)
     call ibis_to_ccm(qrunoffxy, grunof, plonda, beglata, endlata, 1)
     call ibis_to_ccm(qrefxy, qs2, plonda, beglata, endlata, 1)
#endif
!
     deallocate(tempxy1)
     deallocate(tempxy2)
     deallocate(tempxy3)
     deallocate(tempkn1)
     deallocate(tempkn2)
     deallocate(tempkn3)
     deallocate(tempkn4)
!
     return
     end subroutine ibisdrv
