#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine ibisini(oroxy, lwupxy, tsxy, asdirxy, asdifxy, aldirxy, aldifxy, snowxy, &
#ifndef CAMIBIS
                         xmdbase, xmsbase, xmbdate, xmbsec, &
#endif
                         nstep, nsrest, twodt)   
!-----------------------------------------------------------------------
!
!     Initialize the Integrated BIosphere Simulator (IBIS)
!
!---------------------------Code history--------------------------------
!
!     Original version: batsini : A. Hahmann, June 1996
!     off-line IBIS (initial.f) developed by CPEP (1996-1998)
!     coupling to CCM3: C. Delire, April 1999
!
! ---------------------------------------------------------------------
#ifndef PCMIBIS
      use pmgrid
# ifdef CAMIBIS
      use mpishorthand
#  if (defined STAND_ALONE || defined COUP_CSM)
      use modmain, only: cut
#  else
      use spmd_dyn
#  endif
      use shr_sys_mod, only : shr_sys_flush
      use abortutils, only: endrun
# else
#  ifdef SPMD
      use comspmd
#  else
      use precision
#  endif
# endif
# if (!defined STAND_ALONE && !defined COUP_CSM)
      use pspect
# endif
#endif
      use comlsmh
      use com1d
      use lsmmapib
      use ibisparm
      use ibisctl
      use intermap
      use comveg
      use comsoi
      use comatm
      use comsno
      use combcs
      use comsum
#ifdef CAMIBIS
# if (defined STAND_ALONE || defined COUP_CSM)
      use modmain, only: clat
#  ifdef SPMD
      use mpishorthand, only: mpicom
#  endif
# else
      use commap, only: clat
# endif
      use time_manager, only: get_ref_date, get_curr_date, get_curr_calday
#endif

      implicit none
#ifdef PCMIBIS
#include "pspect.h"
#include "pmgrid.h"
#include "comspmd.h"
#endif
!-------------------------------Commons (BATS)--------------------------
#ifndef CAMIBIS
#include <commap.h>
#endif
!------------------------------Arguments--------------------------------
! Input arguments
!
      integer nstep             ! Current time step
      integer nsrest            ! 0 = initial run, 
!     > 0 = continuation run (atmoshere)  
#ifdef CAMIBIS
      integer tod0
      integer tod
#else
      integer xmdbase
      integer xmsbase
      integer xmbdate
      integer xmbsec   
#endif
      real*8 twodt                 ! 2 * delta t 
#ifdef CAMIBIS
      real*8 oroxy(plond,beglat:endlat)     ! land/ocean/sea ice flag
#endif
! 
! Output argments
!
      real*8 lwupxy(plond,beglat:endlat)    ! longwave up flux (W/m**2)
#ifndef CAMIBIS
      real*8 oroxy(plond,beglat:endlat)     ! land/ocean/sea ice flag
#endif
      real*8 tsxy(plond,beglat:endlat)      ! surface (skin) temperature
      real*8 asdirxy(plond,beglat:endlat)   ! albedo - visible waveband, direct
      real*8 asdifxy(plond,beglat:endlat)   ! albedo - visible waveband, diffuse
      real*8 aldirxy(plond,beglat:endlat)   ! albedo - near infrared waveband, direct
      real*8 aldifxy(plond,beglat:endlat)   ! albedo - near infrared waveband, diffuse
!                                   direct
      real*8 snowxy(plond,beglat:endlat)    ! water equivalent snow (m)
!     
      logical rdlsf
!      
!------------------------------Local variables--------------------------
      integer kpti,kptj,loopi   ! dummy variables used in parallel portion
!                               ! of code
!
      integer i,j,k,l,m,ib      ! Loop index
      integer ii,ll,ml
      integer nsurdat           ! surface data unit number
      integer npts              ! total number of land points
      integer mdbase
      integer msbase
      integer mbdate
      integer mbsec   
      integer mcdate            ! date of current time step
      integer mcsec             ! second of current time step
      integer mdcur             ! current day of simulation
      integer mscur             
      integer irestart          ! 0: not restart of IBIS, 1 restart of IBIS
      integer iyrlast           ! last year of simulation 
!                                 (restart from Jan 1 of yr iyrlast + 1) 
      integer imolast           ! last month of simulation 
!                                 (restart from day 1 of mo imolast + 1) 
      real*8 oroxy_g(plon,plat) ! orography - global
      real*8 oroxy2(plon,beglat:endlat)     ! oroxy w/o long. ghost points
!
      real*8 calday

      real*8 vegtype(plon,beglat:endlat,mlpt)     ! 2-d vegetation type [1-18]
      real*8 sandpct(plon,beglat:endlat,nsoilay)  ! 2-d sand percentage
      real*8 claypct(plon,beglat:endlat,nsoilay)  ! 2-d clay percentage
      real*8 tmonth(plon,beglat:endlat,12)        ! monthly climatological temperature
      real*8 difftmin(plon,beglat:endlat)         ! absolute min T - min(monthly T)
      real*8 vegtype_g(plon,plat,mlpt)            ! 2-d vegetation type [1-18] - global
      real*8 sandpct_g(plon,plat,nsoilay)         ! 2-d sand percentage - global
      real*8 claypct_g(plon,plat,nsoilay)         ! 2-d clay percentage - global
      real*8 tmonth_g(plon,plat,12)               ! monthly climatological temperature - global
      real*8 difftmin_g(plon,plat)                ! absolute min T - min(monthly T) - global

      real*8, dimension(:,:), allocatable :: xinvegm
      real*8 vegtemp(plon,plat)

      real*8 isnowib
!     
      character*80 fvegdat(mlpt)  ! vegetation data file names
      character*80 fsanddat       ! sand data file name
      character*80 fclaydat       ! clay data file name
      character*80 ftmndat        ! monthly temperature data file name
      character*80 fdttdat        ! difference between min monthly temp
!                                   and absolute min temp.
      character*80 finidat        ! initial conditions file name
      character*80 ibis_stdin
      namelist /ibisexp/ rdlsf, fvegdat, fsanddat, fclaydat, ftmndat, fdttdat, &
                         finidat, irestart, numlv, soicspin, spinmax, iyearout,&
                         imonthout, idayout, ipointout, isimveg, isimfire,     &
                         isimco2, ccmexist
      integer getunit             ! returns unassigned file unit number
      external getunit
      integer lencsib             ! length of character string

      integer iquot, irem, icnt

      real*8, dimension(:), allocatable :: temp_pn1, temp_pn2
      real*8 garea_g(npoi1)

      logical lexist

! ---------------------------------------------------------------------
! local variabes for netcdf files
!
      integer istart(4), icount(4), istat ! for reading netcdf vars
      integer nlevel, zero, nk1, np1
!
      real*8 alons(plon),  alats(plat), vals3d(plon), times(12)
!
      character*1024 filen ! file name
!
      data istart / 1,1,1,1 /, icount / plon,plat,1,1 /

!     create ibis copies of atm variables in ibisparm

      plona = plon
      plata = plat
      plonda = plond
      pleva = plev
      plevpa = pleva + 1
      pcnsta = PCNST
#ifdef PCMIBIS
      pnatsa = 0
#else
      pnatsa = PNATS
#endif
      beglata = beglat
      endlata = endlat
      do j = 1, plat
         clata(j) = clat(j)
         cuta(1,j-1) = cut(1,j-1)
         cuta(2,j-1) = cut(2,j-1)
      enddo

      mpicomw = 0
      myid = 0
      nproc = 1
      numlv = 8

#ifdef SPMD
# if (defined STAND_ALONE || defined COUP_CSM)
      mpicomw = mpicom
# else
      mpicomw = MPI_COMM_WORLD
# endif
      call mpi_comm_rank (mpicomw, myid, iermpi)
      call mpi_comm_size (mpicomw, nproc, iermpi)
#endif

!-----------------------------------------------------------------------
! Begin initialization: echo initialization to standard output
!
      if (myid .eq. 0) then
         write (6,*)
         write (6,*) '***********************************************'
         write (6,*) 'IBIS biosphere model - University of Wisconsin'
         write (6,*) 'Scalable MPI version constructed by LLNL'
         write (6,*)
         if (nsrest .eq. 0) then
            write (6,*) 'Initial atmospheric run'
         else
            write (6,*) 'Continuation run for atmosphere'
         end if
         write (6,*) 'This version supports multiple land points'
         write (6,*) 'per actual land point'
      end if
!
! This version supports multiple land points per actual land points,
! as a means of simulating multiple vegetation types at each land point.
! The variable "npoi1" is set to the total number of actual land points, LPT.
! The degree of multiplicity is "mlpt". The variable "npoi", formerly LPT,
! is set to mlpt*LPT.
!
! A number of the mesh-related arrays have two incarnations - one with
! respect to the full (multiplied) land grid (extent npoi), and the other
! with respect to the singular land grid (extent npoi1). Typically the
! names ending in "1" refer to the singular land grid. These arrays include
! isy, jxy, lati, loni, procj, procn, njbeg, njend, npbeg, npend, npnum,
! lbeg, lend, lnum.
!
! The original input data files are transformed onto the singular ibis
! mesh and then scattered onto the multiple ibis mesh. The multiple
! vegetaiton data is in one netcdf file and is transformed onto the
! multiple ibis mesh.
!
! CCM variables are transformed onto the singular ibis mesh and then
! scattered onto the multiple ibis mesh. Ibis variables are averaged
! over the multiple ibis mesh onto the sinular ibis mesh and then
! transformed to CCM.
!

!
! Set up IBIS domain decomposition
! This must be done before the partition for shared memory
!  parallelization. Both must be done prior to the main allocation.
! Npbeg is first land point index for given subdomain
! Npend is last land point index for given subdomains
! Npnum is number of land points for given subdomain
!
      allocate(npbeg1(nproc))
      allocate(npend1(nproc))
      allocate(npnum1(nproc))
      allocate(npbeg(nproc))
      allocate(npend(nproc))
      allocate(npnum(nproc))
      npbeg1(:) = 0
      npend1(:) = 0
      npnum1(:) = 0
      npbeg(:) = 0
      npend(:) = 0
      npnum(:) = 0

      npbeg1(1) = 1
      npend1(1) = npoi1
      npnum1(1) = npoi1
      npbeg(1) = 1
      npend(1) = npoi
      npnum(1) = npoi

#ifdef SPMD
      iquot = npoi1 / nproc
      irem = npoi1 - iquot*nproc
      do i = 1, irem
         npnum1(i) = iquot + 1
      enddo
      do i = irem+1, nproc
         npnum1(i) = iquot
      enddo
      icnt = 0
      do i = 1, nproc
         npbeg1(i) = icnt + 1
         npend1(i) = npbeg1(i) + npnum1(i) - 1
         icnt = npend1(i)
      enddo
      do i = 1, nproc
         npnum(i) = mlpt * npnum1(i)
         npbeg(i) = mlpt * (npbeg1(i)-1) + 1
         npend(i) = mlpt * npend1(i)
      enddo
#endif
      lbeg1 = npbeg1(myid+1)
      lend1 = npend1(myid+1)
      lnum1 = npnum1(myid+1)
      lbeg = npbeg(myid+1)
      lend = npend(myid+1)
      lnum = npnum(myid+1)

      allocate(temp_pn1(lbeg:lend))
      allocate(temp_pn2(lbeg:lend))
!
! Store CCM latitudinal domain decomposition information in
!   Ibis arrays
! lat_b is first latitudinal index for given CCM subdomain
! lat_e is last latitudinal index for given CCM subdomain
! lat_n is subdomain number for given latitude
!
      allocate(lat_b(nproc))
      allocate(lat_e(nproc))
      allocate(lat_n(plat))
      lat_b(:) = 0
      lat_e(:) = 0
      lat_n(:) = 0

      lat_b(1) = 1
      lat_e(1) = plat
      lat_n(:) = 1

#ifdef SPMD
      do i = 1, nproc
         lat_b(i) = cut(1,i-1)
         lat_e(i) = cut(2,i-1)
         do j = lat_b(i), lat_e(i)
            lat_n(j) = i
         enddo
      enddo
#endif

      if (myid .eq. 0) then
         write (6,*) 'Preset FORTRAN unit numbers:'
         write (6,*) '   unit  5 = Standard Input'
         write (6,*) '   unit  6 = Standard Output'
         write (6,*)
      end if
!
! Read [ibisexp] namelist parameters. Logical switches are preset to
! default values
!     o rdlsf   = true if read in initial dataset
!     o finidat = initial condition file name. Only needed if rdlst = .true.
!     o tsurdat = surface data file name
! Default values
!
      rdlsf = .false.
      finidat = 'arbitrary initialization'
      fvegdat(:) = 'must specify'
!
! Zero namelist constants in ibisctl
!
      iyearout  = 0
      imonthout = 0
      idayout   = 0
      ipointout = 0
      isimveg   = 0
      isimfire  = 0
      isimco2   = 0
      ccmexist  = 0
      soicspin  = 0
      spinmax   = 1
      iyrlast   = 0
      imolast   = 12
!
! read [ibisexp] namelist from standard input (unit 5)  
!      namelist is read by each MPI task
! *** it clearly reads from unit 7, not STDIN. what is reasonable if we are not
!     running the model in an interactive environment - fzm 20110620
      ibis_stdin = 'atm.parm'
      open (unit=7, file=ibis_stdin)
      read (7,ibisexp)
      close(7)

      if (irestart .eq. 1) then
         inquire(file=trim(outdir)//'inland-restart_control.dat',exist=lexist)
         if (.not. lexist) then
            if (myid .eq. 0) then
               print *, 'inland-restart_control.dat does not exist - exiting'
            endif
            call endrun
         endif
         open(unit=8, file=trim(outdir)//'inland-restart_control.dat')
         read(8,*) iyrlast, imolast
         close(8)
      endif

      if (soicspin .eq. 0) spinmax=1

      if (myid .eq. 0) then
         write(*,*) 'numlv = ', numlv
         write(*,*) 'ipointout = ', ipointout
         write(*,*) 'idayout = ',idayout
         write(*,*) 'imonthout = ',imonthout
         write(*,*) 'iyearout = ',iyearout
         write(*,*) 'soicspin = ',soicspin
         write(*,*) 'spinmax =', spinmax
         write(*,*) 'isimveg = ', isimveg
         write(*,*) 'isimfire = ', isimfire
         write(*,*) 'isimco2 = ', isimco2
         write(*,*) 'ccmexit = ', ccmexist
         write(*,*) 'irestart = ', irestart
         write(*,*) 'iyrlast =', iyrlast
         write(*,*) 'imolast =', imolast
!
! Echo control parameters and data sets
!
         write (6,*) 'Logical run control parameters:'
         write (6,*) '   use initial data set = ',rdlsf
         write (6,*)
         write (6,*) 'Data files:'
         write (6,*) '   vegetation data = ',          &
                     (fvegdat(m)(1:lencsib(fvegdat(m))),m=1,mlpt)
         write (6,*) '   sand data = ', fsanddat(1:lencsib(fsanddat))
         write (6,*) '   clay data = ', fclaydat(1:lencsib(fclaydat))
         write (6,*) '   monthly temperature data = ', &
                     ftmndat(1:lencsib(ftmndat))
         write (6,*) '   tmin - min(tmonth) data = ',  &
                     fdttdat(1:lencsib(fdttdat))
         write (6,*) '   initial data = ', finidat(1:lencsib(finidat))

      end if
!
! Error messages
!
      do m = 1, mlpt
         if (fvegdat(m) .eq. 'must specify') then
            write (6,*) 'IBISINI: Need to set srf data file name - m = ', m 
            call endrun
         end if
      enddo

! ----------------------------------------------------------------------
! Set up shared memory parallelism
! Numlv is the number of groups of points assigned to threads
! Begpnt is the beginning point of a group
! Numpnt is the number of points in a group
! Mpt is the maximum number of points in a group - needed for ibisalloc
! ----------------------------------------------------------------------

      allocate(begpnt(numlv))
      allocate(numpnt(numlv))
      begpnt(:) = 0
      numpnt(:) = 0

      iquot = lnum / numlv
      irem = lnum - iquot*numlv
      do i = 1, irem
         numpnt(i) = iquot + 1
      enddo
      do i = irem+1, numlv
         numpnt(i) = iquot
      enddo

      icnt = lbeg - 1
      do i = 1, numlv
         begpnt(i) = icnt + 1
         icnt = begpnt(i) + numpnt(i) - 1
      enddo

      mpt = numpnt(1)

!
! Allocate variables
!
      call ibisalloc

      if (rdlsf) then
         if (myid .eq. 0) then
            write (6,*) 'IBISINI: initializing Antarctica and Greenland' 
         endif
      end if
!
!
! Read land surface time-invariant boundary data on [plon] x [plat] grid
! assign unit number to file. open file. read data. close and release file
!     Vegtype is read even if not used in case of a restart with
!     dynamic vegetation 
!
! Read using master process
!
      if (myid .eq. 0) then
         do m = 1, mlpt
            filen = fvegdat(m)
            write(*,*) filen
            icount(3) = 1
            call readvar(filen,'vegtype','level',istart,icount,vegtemp,alons, &
                         alats,vals3d,times,istat)
            if (istat .ne. 0) then
               write(*,*) 'ERROR in ibisini, vegtype, m = ', m
               call endrun
            end if
            vegtype_g(:,:,m) = vegtemp(:,:)
         enddo
!
         filen = fsanddat
         write(*,*) filen
         icount(3) = nsoilay
         call readvar(filen,'sandpct','layer',istart,icount,sandpct_g,alons, &
                      alats,vals3d,times,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in ibisini, sandpct'
            call endrun
         end if

         filen = fclaydat
         write(*,*) filen
         icount(3) = nsoilay
         call readvar(filen,'claypct','layer',istart,icount,claypct_g,alons, &
                      alats,vals3d,times,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in ibisini, claypct'
            call endrun
         end if
!
         filen = ftmndat
         write(*,*) filen
         icount(3) = 1
         istart(4) = 1
         icount(4) = 12
         call readvar(filen,'temp','level',istart,icount,tmonth_g,alons,alats, &
                      vals3d,times,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in ibisini, monthly temp'
            call endrun
         end if

         filen = fdttdat
         write(*,*) filen
         icount(3) = 1
         icount(4) = 1
         call readvar(filen,'deltat','level',istart,icount,difftmin_g,alons, &
                      alats,vals3d,times,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in ibisini, diftmin'
            call endrun
         end if

         print *, 'IBISINI: Successfully read netcdf data files'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif

!
! Hardwiring for T31 by Bala
!
         if (plon .eq. 96 .and. plat .eq. 48) then
            do k = 1, mlpt
               vegtype_g(46,3,k) = 9.e20
            enddo
            do k = 1, nsoilay
               sandpct_g(46,3,k) = -9999.
               claypct_g(46,3,k) = -9999.
            end do
            do m = 1, 12
               tmonth_g(46,3,m) = 0.0
            end do
            difftmin_g(46,3) = 0.0
         endif
      endif ! (myid .eq. 0)
!
! Scatter to other tasks
!
      call scatvarn(vegtype_g, vegtype, plon, mlpt)
      call scatvarn(sandpct_g, sandpct, plon, nsoilay)
      call scatvarn(claypct_g, claypct, plon, nsoilay)
      call scatvarn(tmonth_g, tmonth, plon, 12)
      call scatvar(difftmin_g, difftmin, plon)

#ifndef CAMIBIS
!     
! Return 2-d orography based on IBIS vegetation type (vegtype=9e21 means 
! ocean)
!
! Oroxy is generally an input variable
! For ccm/pcm initial run, oroxy is an output variable
!
      do j=beglat,endlat
         do i=1,plon
!
! Note: Each vegetation type is assumed to have same land points
!
            if (vegtype(i,j,1) .lt. 100.) then
               oroxy2(i,j) = 1.
            else
               oroxy2(i,j) = 0.
            endif
         end do
      end do
      if (nsrest .eq. 0) then
         do j=beglat,endlat
            do i=1,plon
               oroxy(i,j) = oroxy2(i,j)
            end do
         end do
      endif
#else
      do j=beglat,endlat
         do i=1,plon
            oroxy2(i,j) = oroxy(i,j)
         end do
      end do
#endif
!
! Gather and broadcast global orography
!
      call gathvar(oroxy_g, oroxy2, plon)
#ifdef SPMD
      call mpi_bcast(oroxy_g, plon*plat, MPI_REAL8, 0, mpicomw, iermpi)
#endif
!
! Find [ixy] and [jxy] vectors for mapping between [plon] x [plat]
! grid and land surface vector [lpt]. plon and plat in ibispar.
!
      call ibismap (plon, plat, oroxy_g, clat, ixy1, jxy1, lati1, loni1, npts, garea_g)      

      if (myid .eq. 0) then
         print *, '(IBIS) ibismap completed'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif

      do i = 1, npoi
         ii = (i-1)/mlpt + 1
         ixy(i) = ixy1(ii)
         jxy(i) = jxy1(ii)
         lati(i) = lati1(ii)
         loni(i) = loni1(ii)
      enddo
      do i = lbeg, lend
         ii = (i-1)/mlpt + 1
         garea(i) = garea_g(ii)
      enddo
!
! check for mismatch between lpt (parameters - total number of land 
! points on plon x plat grid) and npts (computed within batsmap).
!
      if (npoi1 .ne. npts) then
         write(6,*) 'IBISINI: error 664, illegal value of npts'
         write(6,*) npoi,npts
         call endrun
      endif

      if (myid .eq. 0) then
         print *, 'IBISINI: npts = ', npts
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif
!
! Set up land point process arrays
! Procn = subdomain number for land decomposition
! Procj = subdomain number for ccm latitudinal decomposition
! This calculation must be done after the call to batsmap, since
!  it makes use of jxy.
!
      do i = 1, nproc
         do k = npbeg(i), npend(i)
            procn(k) = i
         enddo
         do k = npbeg1(i), npend1(i)
            procn1(k) = i
         enddo
      enddo
      do k = 1, npoi
         procj(k) = lat_n(jxy(k))
      enddo
      do k = 1, npoi1
         procj1(k) = lat_n(jxy1(k))
      enddo

!
! Compute latitudinal decomposition index ranges
! Njbeg is first land point index for given CCM subdomain
! Njend is last land point index for given CCM subdomain
! Land points are assumed to be in natural order, arranged by
!  latitude first, and then longitude; not all CCM subdomains
!  are assumed to be represented.
! This calculation makes use of procj.
!
      allocate(njbeg1(nproc))
      allocate(njend1(nproc))
      njbeg1(:) = 0
      njend1(:) = -1

      nk1 = procj1(1)
      do j = 1, nk1-1
         njbeg1(j) = 0
         njend1(j) = -1
      enddo
      njbeg1(nk1) = 1
      do k = 2, npoi1
         if (procj1(k) .ne. nk1) then
            njend1(nk1) = k-1
            nk1 = procj1(k)
            njbeg1(nk1) = k
         endif
      enddo
      njend1(nk1) = npoi1
      do j = nk1+1, nproc
         njbeg1(j) = 0
         njend1(j) = -1
      enddo

      allocate(njbeg(nproc))
      allocate(njend(nproc))

      do j = 1, nproc
         njbeg(j) = mlpt * (njbeg1(j)-1) + 1
         njend(j) = mlpt * njend1(j)
      enddo

!
! Compute mapping between CCM and Ibis domain decompositions
!
      call initmap

      if (myid .eq. 0) then
         print *, '(IBIS) initmap completed'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif

! Transfer vegetation and texture to long vectors following
! mapping of land points
!
      allocate (xinvegm(lbeg:lend,mlpt))
      call ccm_to_ibis(vegtype, xinvegm, plon, beglat, endlat, mlpt)
      do ml = 1, mlpt
         do l = lbeg1, lend1
            ll = mlpt * (l-1) + ml
            xinveg(ll) = xinvegm(ll,ml)
         enddo
      enddo
      call ccm_to_ibis(sandpct, sand, plon, beglat, endlat, nsoilay)
      call ccm_to_ibis(claypct, clay, plon, beglat, endlat, nsoilay)
      call ccm_to_ibis(tmonth, xint, plon, beglat, endlat, 12)
      call ccm_to_ibis(difftmin, deltat, plon, beglat, endlat, 1)
!
! Initializes the time and date variables.
!
      dtime = twodt/2.
#ifdef CAMIBIS
      call get_ref_date(iyear0, imonth0, iday0, tod0)
      call get_curr_date(iyear, imonth, iday, tod)
      calday = get_curr_calday()
#else
      mdbase = xmdbase          ! base day of run
      msbase = xmsbase          ! base seconds of base day
      mbdate = xmbdate          ! base date of run (yymmdd format)
      mbsec  = xmbsec           ! base seconds of base date

      call calendr (nstep, dtime, mdbase, msbase, mbdate, mbsec, mdcur, mscur, &
                    mcdate, mcsec, calday)
!

      iyear0 = mbdate/10000
      iyear = mcdate/10000
      imonth = mod(mcdate,10000)/100
      iday = mod(mcdate,100)
#endif
!
      if (myid .eq. 0) write(6,*) 'IBISINI ', iyear0, iyear, imonth, iday
#ifdef CAMIBIS
      call shr_sys_flush(6)
#endif
!
      call rd_param      

      if (myid .eq. 0) then
         print *, '(IBIS) rd_param completed'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif
!
! climanl is read even in the case of a restart : 
! this is needed for restarts in fixed vegetation runs
! For dynamic vegetation runs, the actual pfts and gdd are read
! after from restart files.
!
      call climanl

      if (myid .eq. 0) then
         print *, '(IBIS) climanl completed'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif

#ifdef SPMD
      call mpi_barrier(mpicomw, iermpi)
#endif

!
!     Initializes IBIS           
!  
      call initialib (isimveg, irestart, iyrlast, imolast, rdlsf, lati, loni)

#ifdef SPMD
      call mpi_barrier(mpicomw, iermpi)
#endif
      if (myid .eq. 0) then
         print *, '(IBIS) initialib completed'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif
!
! CD : put into wtot and initialisation of variables in inisum.F 
! called by initial.F calculates total initial water content of the 
! land-surface (intercepted water and snow, soil water and ice, snow).
!
!
! set up soil carbon spin up procedure
! CD: Removed comments by Bala (Bala commented out these variables because
! they are defined in ibis.F
!
! soil spin up procedure calls soil bgc model spincons times
! for each time step to spin up the process (accelerate C & N pools)
!
      spinfrac  = 0.75    ! fraction of nspinsoil time used to
!                         ! spin up soil at maximum spin up rate
!
      spincons  = 40.0    ! number of times soilbgc subroutine is
!                         ! called for each day in simulation during
!                         ! acceleration procedure
!
      eqyears   = 50      ! years used past spin up to slowly bring
!                         ! entire soil to a steady equilibrium
!
      nspinsoil = iyear0 + 150 ! year of simulation that soil c reaches equilibrium
!
!
! Compute surface albedos;
! First time step only, otherwise kept in restart file
!
      if (nsrest .eq. 0) then
         kpti = lbeg
         kptj = lend
         loopi = 1
         call ibiszen (calday, loni, lati, coszen, kpti, kptj)
!
! Bala added the following call because the quanties fwetl and rliql calculated by
! fwetcal are needed in solalb
!
         call fwetcal (kpti, kptj)
!
!
! set up for solar calculations
!
         call solset (loopi, kpti, kptj)
!
! solar calculations for each waveband
!
         do 100 ib = 1, nband
!
! solsur sets surface albedos for soil and snow
! solalb performs the albedo calculations
! solarf uses the unit-incident-flux results from solalb
! to obtain absorbed fluxes sol[u,s,l,g,i] and
! incident pars sunp[u,l]
!
            call solsur (ib, loopi)
            call solalb (ib, loopi)
!
100      continue

#ifdef CAMIBIS
         do l = lbeg, lend
            if (coszen(l) .lt. 0.) then
               asurd(l,1) = 1.d0
               asurd(l,2) = 1.d0
               asuri(l,1) = 1.d0
               asuri(l,2) = 1.d0
            endif
         enddo
#endif
!
         call ibis_to_ccm(asdirxy, asurd(lbeg,1), plond, beglat, endlat, 1)
         call ibis_to_ccm(asdifxy, asuri(lbeg,1), plond, beglat, endlat, 1)
         call ibis_to_ccm(aldirxy, asurd(lbeg,2), plond, beglat, endlat, 1)
         call ibis_to_ccm(aldifxy, asuri(lbeg,2), plond, beglat, endlat, 1)
         call ibis_to_ccm(tsxy, tg, plond, beglat, endlat, 1)
         temp_pn1(:) = 0.
         do k = lbeg, lend
            do l = 1, nsnolay
               temp_pn1(k) = temp_pn1(k) + rhos*hsno(k,l)
            end do
         enddo
         call ibis_to_ccm(snowxy, temp_pn1, plond, beglat, endlat, 1)
         do k = lbeg, lend
            temp_pn2(k) = stef * (tg(k)**4)
         enddo
         call ibis_to_ccm(lwupxy, temp_pn2, plond, beglat, endlat, 1)
      end if

      if (myid .eq. 0) then
         print *, '(IBIS) initial albedo calculation completed'
#ifdef CAMIBIS
         call shr_sys_flush(6)
#endif
      endif
!     
      if (myid .eq. 0) then
         if (nsrest .eq. 0 .or. irestart .eq. 0) then 
            if (ipointout .eq. 1) then
               open(200,file='flux.france')
               open(201,file='water.france')
               open(202,file='tsoi.france') 
               open(103,file='ecol.france')
               open(104,file='evap.france')
               open(120,file='solar.france')
               open(121,file='carbon.france')
               open(122,file='soic.france')
               open(1100,file='flux.amazon')
               open(1101,file='water.amazon')
               open(1102,file='tsoi.amazon')
               open(1103,file='ecol.amazon')
               open(1104,file='evap.amazon')
               open(1120,file='solar.amazon')
               open(1121,file='carbon.amazon')
               open(1122,file='soic.amazon')
               open(2100,file='flux.canada')
               open(2101,file='water.canada')
               open(2102,file='tsoi.canada')
               open(2103,file='ecol.canada')
               open(2104,file='evap.canada')
               open(2120,file='solar.canada')
               open(2121,file='carbon.canada')
               open(2122,file='soic.canada')
            end if 

            open (200,file='ibis.out.annual',status='unknown')
            open (201,file='ibis.out.global',status='unknown')
            open (202,file='ibis.out.vegtype',status='unknown')

            write (201,*) ' '
            write (201,*) '* * * annual diagnostic fields * * *'
            write (201,*) ' '
            write (201,*) 'year       nee     npp       '//             &
                          'totfall   disturb     co2mic     gleach  '// &
                          'gpp   biomass   scarbon   snitrogen  ' //    &
                          'alitter   blitter   co2soi    prec     '//   &
                          ' aet    transp    runoff  drainage '
 
            write (202,*) ' '
            write (202,*) '** annual diagnostic fields by vegetation type **'
            write (202,*) ' '
            write (202,*) 'year    veg           area       nee       npp       gpp   '// &
                          'leach       co2mic  disturb    totfall '// &
                          'biomass       lai   scarbon    runoff '
         else
#if ( defined CRAY || defined AIX || defined SGI )
            if (ipointout .eq. 1) then
               open(200,file='flux.france',status='unknown',position='append')
               open(201,file='water.france',status='unknown',position='append')
               open(202,file='tsoi.france',status='unknown',position='append')
               open(103,file='ecol.france',status='unknown',position='append')
               open(104,file='evap.france',status='unknown',position='append')
               open(120,file='solar.france',status='unknown',position='append')
               open(121,file='carbon.france',status='unknown',position ='append')
               open(122,file='soic.france',status='unknown',position='append')

               open(1100,file='flux.amazon',status='unknown',position='append')
               open(1101,file='water.amazon',status='unknown',position='append')
               open(1102,file='tsoi.amazon',status='unknown',position='append')
               open(1103,file='ecol.amazon',status='unknown',position='append')
               open(1104,file='evap.amazon',status='unknown',position='append')
               open(1120,file='solar.amazon',status='unknown',position='append')
               open(1121,file='carbon.amazon',status='unknown',position='append')
               open(1122,file='soic.amazon',status='unknown',position='append')

               open(2100,file='flux.canada',status='unknown',position='append')
               open(2101,file='water.canada',status='unknown',position='append')
               open(2102,file='tsoi.canada',status='unknown',position='append')
               open(2103,file='ecol.canada',status='unknown',position='append')
               open(2104,file='evap.canada',status='unknown',position='append')
               open(2120,file='solar.canada',status='unknown',position='append')
               open(2121,file='carbon.canada',status='unknown',position='append')
               open(2122,file='soic.canada',status='unknown',position='append')
            end if
!
            open(200,file='ibis.out.annual',status='unknown',position='append')
            open(201,file='ibis.out.global',status='unknown',position='append')
            open(202,file='ibis.out.vegtype',status='unknown',position='append')
#else
            if (ipointout .eq. 1) then
               open(200,file='flux.france',status='unknown',access='append')
               open(201,file='water.france',status='unknown',access='append')
               open(202,file='tsoi.france',status='unknown',access='append')
               open(103,file='ecol.france',status='unknown',access='append')
               open(104,file='evap.france',status='unknown',access='append')
               open(120,file='solar.france',status='unknown',access='append')
               open(121,file='carbon.france',status='unknown',access ='append')
               open(122,file='soic.france',status='unknown',access='append')
               open(1100,file='flux.amazon',status='unknown',access='append')
               open(1101,file='water.amazon',status='unknown',access='append')
               open(1102,file='tsoi.amazon',status='unknown',access='append')
               open(1103,file='ecol.amazon',status='unknown',access='append')
               open(1104,file='evap.amazon',status='unknown',access='append')
               open(1120,file='solar.amazon',status='unknown',access='append')
               open(1121,file='carbon.amazon',status='unknown',access='append')
               open(1122,file='soic.amazon',status='unknown',access='append')
               open(2100,file='flux.canada',status='unknown',access='append')
               open(2101,file='water.canada',status='unknown',access='append')
               open(2102,file='tsoi.canada',status='unknown',access='append')
               open(2103,file='ecol.canada',status='unknown',access='append')
               open(2104,file='evap.canada',status='unknown',access='append')
               open(2120,file='solar.canada',status='unknown',access='append')
               open(2121,file='carbon.canada',status='unknown',access='append')
               open(2122,file='soic.canada',status='unknown',access='append')
            end if
!
            open(200,file='ibis.out.annual',status='unknown',access='append')
            open(201,file='ibis.out.global',status='unknown',access='append')
            open(202,file='ibis.out.vegtype',status='unknown',access='append')
#endif
            write(201,*) ' '
            write(201,*) '* * * annual diagnostic fields * * *'
            write(201,*) ' '
            write(201,*) 'year       nee     npp       '//             &
                         'totfall   disturb     co2mic     gleach  '// &
                         'gpp   biomass   scarbon   snitrogen  ' //    &
                         'alitter   blitter   co2soi    prec     ' //  &
                         ' aet    transp    runoff  drainage '

            write (202,*) ' '
            write (202,*) '** annual diagnostic fields by vegetation type **'
            write (202,*) ' '
            write (202,*) 'year    veg           area       nee       npp       gpp   '// &
                          'leach       co2mic  disturb    totfall '//      &
                          'biomass       lai   scarbon    runoff '
         end if ! (nsrest .eq. 0 .or. irestart .eq. 0)
      endif  ! (myid .eq. 0)

      deallocate(temp_pn1)
      deallocate(temp_pn2)

      return
end subroutine ibisini
