#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine ibismap(plon, plat, oro, clat, ixy, jxy, lati, loni, npts, area)
! ---------------------------------------------------------------------
! The land surface model works by gathering all the land points on the
! CCM3 [plon] x [plat] grid into a vector of [lpt] land points.
!
! This subroutine finds [ixy] and [jxy], which are indices for the 
! mapping: [plon] x [plat] grid <-> [lpt] vector of land points 
! ---------------------------------------------------------------------
      use ibispar
      use comwork
#if (defined CAMIBIS)
      use abortutils, only: endrun
#endif

      implicit none
!-----------------------------Arguments---------------------------------
! Input arguments
      integer plat             ! number of latitude points on grid
      integer plon             ! number of longitude points on grid
      real*8 oro(plon,plat)    ! Land/ocean/sea ice flag
      real*8 clat(plat)           ! latitude of model grid [radians]
 
! Output arguments
      integer npts            ! total number of land points
      integer ixy(npoi1)       ! longitude index of lpt land points: 1 to plon100   continue
      integer jxy(npoi1)       ! latitude index of lpt land points: 1 to plat
      real*8 lati(npoi1)         ! latitude of land point on long vector [rads]
      real*8 loni(npoi1)         ! longitude of land point on long vector [rads]
      real*8 area(npoi1)          ! area of land gridcell centered on lati*loni [m^2]
!
!------------------------------Local variables--------------------------
      integer i,j,l           ! loop indices
      real*8 pi                 ! pi
      real*8 re                 ! radius of Earth (m)
      real*8 airtot
      real*8 sinlat(plat)       ! sine of latitudes
      real*8 areagordon(plon,plat)
      real*8 lats(plat+1)
      real*8 lonw(plon+1,plat)
      real*8 edgen, edges,edgew,edgee,global
      integer numlon(plat)
!-----------------------------------------------------------------------
!
      pi = 4.*atan(1.)
      re = 6.371227709e+6

! lonscale and latscale are used by the netcdf input/output subroutine
      do i = 1, plon
         lonscale(i) = real(i-1)/real(plon)* 360.
      end do
      do j = 1, plat
         latscale(j) = clat(j) * 180. /pi
      end do

!     the area of each grid cell is calculated following Gordon's
!     subroutines
      edgen = 90.
      edges = -90.
      edgew = -float(plon)/(2.*360.0)
      edgee = 360.-float(plon)/(2.*360.0)
      do j = 1, plat
         numlon(j) = plon
      end do

      call cellbox(plat, plat+1, plon, plon+1, edgen,edges,edgew,edgee, &
            numlon,lonscale, latscale, lats, lonw)
     
      call cellarea(plat, plat+1, plon, plon+1,edgen,edges,edgew,edgee, &
            numlon,lats, lonw, areagordon)

      if (myid .eq. 0) open(1000,file='land_points',status='unknown')
      l = 0
      npts = 0
      global = 0
      do j=1,plat
         do i=1,plon
            if (oro(i,j) .gt. 0.) then         ! land point
               l = l + 1
               if (l .gt. npoi1) then
                  write(6,*) 'Error LSMMAPIB: Land surface mismatch - index =', l
                  call endrun
               endif
               ixy(l) = i
               jxy(l) = j
               lati(l) = clat(j)
               loni(l) = real(i-1)/real(plon)*2.*pi
               area(l) = oro(i,j)*areagordon(i,j)*1e6
               if (myid .eq. 0) &
                  write(1000,'(3i5,3f12.3)') &
                              l,i,j,loni(l)*180./pi,lati(l)*180./pi, area(l)/1e6
            endif
         end do
      end do

      npts = l

      if (myid .eq. 0) close (1000)

      return
end subroutine ibismap
