#include "inland_config.h"

#ifdef SINGLE_POINT_MODEL
#error "This subroutine should NOT be compiled for 0D IBIS model option."
#endif

! ies-io.f last update 9/9/98 Christine C. Molling, cmolling@facstaff.wisc.edu
! University of Wisconsin - Madison; Institute for Environmental Studies;
! Center for Climatic Research; Climate, People, and Environment Program
!
! You are free to change or distribute my code as long as you
! 1) acknowlege me, Christine C. Molling (cmolling@facstaff.wisc.edu)
! as the author of the original code and
! 2) do not sell it.
! Of course if there is anything wrong with the code, or you somehow
! encounter damage to your reputation/wallet because of its use, you
! are forbidden to sue me or anyone else.
!
! These subroutines can be used as a primary interface to the ies
! format netcdf files.  You can also use the slightly lower level functions
! found in ies.f or the low level netcdf commands (see the netcdf V3
! users guide).
! SUBROUTINE LIST:
! inifile - create a new file with dimensions and variables for
! longitude, latitude, an optional 3rd real dimension, and time
! Time must have units of days since a date!!!
! inifilec - create a new file with dimensions and variables for
! longitude, latitude, a 3rd character-variable dimension, and time
! Time must have units of days since a date!!!
! inivar - initialize a new variable
! endini - end initialization of file: sync file to disk, close file
! writevar - write a 2-, 3-, or 4-dimensional hyperslab of data
! readvar - read a 2-, 3-, or 4-dimensional hyperslab of data
! .....................................................................
! inivar - initialize a new variable
! .....................................................................
subroutine inivar(idies,varname,longname,units,ndims,dimnames,valmissing, &
                  ierror)
      implicit none
!
! INPUT
! idies - integer - id number of a new file from a previous call
! to inifile, inifilec, or iesopen
! varname - charcter*(*) - name for new variable
! longname - character*(*) - descriptive name for variable
! units - character*(*) - units for variable, compatible with udunits
! ndims - integer - number of dimensions for this variable
! dimnames - character*(*)(ndims) - name of each dimension, in order
! valmissing - real - missing value, ignored if valmissing=0.
! OUTPUT
!     ierror - integer - error code, 0 = no error, < 0 = error occurred
      integer :: idies, ndims, ierror
      character*(*) varname, longname, units, dimnames(ndims)
      real*8 :: valmissing

#ifdef GFORTRAN
#include <netcdf.inc>
#else /* GFORTRAN */
      include 'netcdf.inc'
#endif

      integer :: ierr, idvar, iddims(4), i, lenchrib

      ierror = 0

! Put into redefine mode, but don't fail if already in define mode
! ----------------------------------------------------------------
      ierr = NF_REDEF(idies)
      if (ierr .ne. NF_NOERR) then
         print *, 'Warning in inivar'
         print *, NF_STRERROR(ierr)
      end if

! Find id's of dimensions
! -----------------------
      do 100 i = 1, ndims
         ierr = NF_INQ_DIMID(idies,dimnames(i),iddims(i))
         if (ierr .ne. NF_NOERR) then
            print *, 'Error in inivar'
            print *, NF_STRERROR(ierr)
            ierror = -1
            return
         end if
100   continue

! Define variable
! ---------------
      ierr = NF_DEF_VAR(idies,varname,NF_DOUBLE,ndims,iddims,idvar)
      if (ierr .ne. NF_NOERR) then
         print *, 'Error in inivar'
         print *, NF_STRERROR(ierr)
         ierror = -1
         return
      end if

! Define attributes
! -----------------
      ierr = NF_PUT_ATT_TEXT(idies,idvar,'long_name',lenchrib(longname), &
                             longname)
      if (ierr .ne. NF_NOERR) then
         print *, 'Error in inivar'
         print *, NF_STRERROR(ierr)
         ierror = -1
         return
      end if
      ierr = NF_PUT_ATT_TEXT(idies,idvar,'units',lenchrib(units),units)
      if (ierr .ne. NF_NOERR) then
         print *, 'Error in inivar'
         print *, NF_STRERROR(ierr)
         ierror = -1
         return
      end if
      if (valmissing .ne. 0.) then
         ierr = NF_PUT_ATT_DOUBLE(idies,idvar,'missing_value',NF_DOUBLE,1, &
                                  valmissing)
         if (ierr .ne. NF_NOERR) then
            print *, 'Error in inivar'
            print *, NF_STRERROR(ierr)
            ierror = -1
            return
         end if
      end if

! Exit define mode
! ----------------
      ierr = NF_ENDDEF(idies)
      if (ierr .ne. NF_NOERR) then
         print *, 'Error in inivar'
         print *, NF_STRERROR(ierr)
         ierror = -1
         return
      end if

      return
end subroutine inivar
