#include "inland_config.h"
module intermap

#ifndef PCMIBIS
      use mpishorthand
#endif
      use inland_ibisparm
      use inland_lsmmapib

      implicit none
#ifdef PCMIBIS
#include <mpif.h>
#endif
      public
      save

! ccm => ibis
!
! njsnd = number of destination subdomains
! njs_beg = first index for destination
! njs_end = last index for destination
! njs_tot = number of points for destination
! njs_sub = subdomain number of destination
      integer :: njsnd
      integer, dimension(:), allocatable :: njs_beg
      integer, dimension(:), allocatable :: njs_end
      integer, dimension(:), allocatable :: njs_tot
      integer, dimension(:), allocatable :: njs_sub

! njrcv = number of source subdomains
! njr_beg = first index for source
! njr_end = last index for source
! njr_tot = number of points from source
! njr_sub = subdomain number of source
      integer :: njrcv
      integer, dimension(:), allocatable :: njr_beg
      integer, dimension(:), allocatable :: njr_end
      integer, dimension(:), allocatable :: njr_tot
      integer, dimension(:), allocatable :: njr_sub

! ibis => ccm (reverse roles)

! Communication objects
      type t_comm
         real*8, dimension(:,:), pointer :: s
      end type t_comm

      type (t_comm), dimension(:), allocatable :: sjbuf, rjbuf

      integer, dimension(:), allocatable :: mpi_reqj
      integer, dimension(:), allocatable :: mpi_reqn
      integer, dimension(:,:), allocatable :: mpi_statusj
      integer, dimension(:,:), allocatable :: mpi_statusn

contains
      subroutine initmap
         integer :: mynjbeg, mynjend, mynpbeg, mynpend
         integer :: nk1, np1, k

#ifdef SPMD
         allocate(njs_beg(nproc))
         allocate(njs_end(nproc))
         allocate(njs_tot(nproc))
         allocate(njs_sub(nproc))
         allocate(njr_beg(nproc))
         allocate(njr_end(nproc))
         allocate(njr_tot(nproc))
         allocate(njr_sub(nproc))

         njs_beg(:) = 0
         njs_end(:) = 0
         njs_tot(:) = 0
         njs_sub(:) = 0
         njr_beg(:) = 0
         njr_end(:) = 0
         njr_tot(:) = 0
         njr_sub(:) = 0

! Indexing based on singular land grid
         mynjbeg = njbeg1(myid+1)
         mynjend = njend1(myid+1)
         njsnd = 0

         if (mynjbeg .gt. 0) then
            nk1 = procn1(mynjbeg)
            np1 = 1
            njs_beg(np1) = mynjbeg
            njs_sub(np1) = nk1
            do 100 k = mynjbeg+1, mynjend
               if (procn1(k) .ne. nk1) then
                  njs_end(np1) = k-1
                  np1 = np1 + 1
                  njs_beg(np1) = k
                  nk1 = procn1(k)
                  njs_sub(np1) = nk1
               endif
100         continue
            njs_end(np1) = mynjend
            njsnd = np1
            njs_tot(:) = njs_end(:) - njs_beg(:) + 1
         endif

         mynpbeg = npbeg1(myid+1)
         mynpend = npend1(myid+1)
         njrcv = 0
         if (mynpbeg .gt. 0) then
            nk1 = procj1(mynpbeg)
            np1 = 1
            njr_beg(np1) = mynpbeg
            njr_sub(np1) = nk1

            do 200 k = mynpbeg+1, mynpend
               if (procj1(k) .ne. nk1) then
                  njr_end(np1) = k-1
                  np1 = np1 + 1
                  njr_beg(np1) = k
                  nk1 = procj1(k)
                  njr_sub(np1) = nk1
               endif
200         continue
            njr_end(np1) = mynpend
            njrcv = np1
            njr_tot(:) = njr_end(:) - njr_beg(:) + 1
         endif

! Allocate communication buffers
         if (njsnd .gt. 0) then
            allocate(sjbuf(njsnd))
            allocate(mpi_reqn(njsnd))
            allocate(mpi_statusn(MPI_STATUS_SIZE,njsnd))
         endif

         if (njrcv .gt. 0) then
            allocate(rjbuf(njrcv))
            allocate(mpi_reqj(njrcv))
            allocate(mpi_statusj(MPI_STATUS_SIZE,njrcv))
         endif
#endif

         return
      end subroutine initmap

      subroutine ccm_to_ibis(arr_ccm, arr_ibis, londim, beglatv, endlatv, nvars)
         integer :: londim, nvars, beglatv, endlatv
         real*8 :: arr_ccm(londim,beglatv:endlatv,nvars)
         real*8 :: arr_ibis(lbeg:lend,nvars)

#ifndef SPMD
         integer :: k, nl

! Indexing based on multiple land grid
         do 300 nl = 1, nvars
            do 400 k = lbeg, lend
               arr_ibis(k,nl) = arr_ccm(ixy(k),jxy(k),nl)
400         continue
300      continue
#else
         integer :: k, n, tag, nl, k2

! Indexing based on singular land grid
         tag = 81101

! Post receives from source subdomains
         if (njrcv .gt. 0) then
            do 500 n = 1, njrcv
               allocate(rjbuf(n)%s(njr_tot(n),nvars))
               call mpi_irecv(rjbuf(n)%s, nvars*njr_tot(n), MPI_REAL8,      &
                              njr_sub(n)-1, tag, mpicomw, mpi_reqj(n), iermpi)
500         continue
         endif

! Send to destination subdomains
         if (njsnd .gt. 0) then
            do 600 n = 1, njsnd
               allocate(sjbuf(n)%s(njs_tot(n),nvars))
               do 700 nl = 1, nvars
                  do 800 k = njs_beg(n), njs_end(n)
                     sjbuf(n)%s(k-njs_beg(n)+1,nl) = arr_ccm(ixy1(k),jxy1(k),nl)
800               continue
700           continue
              call mpi_send(sjbuf(n)%s, nvars*njs_tot(n), MPI_REAL8, &
                           njs_sub(n)-1, tag, mpicomw, iermpi)
              deallocate(sjbuf(n)%s)
600         continue
         endif
! Confirm receives from source subdomains
         if (njrcv .gt. 0) then
            call mpi_waitall(njrcv, mpi_reqj, mpi_statusj, iermpi)
            do 900 n = 1, njrcv
               do 1000 nl = 1, nvars
                  do 1100 k = njr_beg(n), njr_end(n)
                     do 1200 k2 = (k-1)*mlpt+1, k*mlpt
                            arr_ibis(k2,nl) = rjbuf(n)%s(k-njr_beg(n)+1,nl)
1200                    continue
1100              continue
1000           continue
               deallocate(rjbuf(n)%s)
900         continue
         endif
#endif
         return
      end subroutine ccm_to_ibis

      subroutine ibis_to_ccm(arr_ccm, arr_ibis, londim, beglatv, endlatv, nvars)
         integer :: londim, nvars, beglatv, endlatv
         real*8 :: arr_ccm(londim,beglatv:endlatv,nvars)
         real*8 :: arr_ibis(lbeg:lend,nvars)

#ifndef SPMD
         integer :: k, nl

! Indexing based on multiple land grid
         do 1300 nl = 1, nvars
            do 1400 k = lbeg, lend
               arr_ccm(ixy(k),jxy(k),nl) = arr_ibis(k,nl)
1400        continue
1300      continue
#else
         integer :: k, n, tag, nl, k2
         real*8 :: arrib, rmlpt

! Indexing based on singular land grid
         tag = 81102

! Post receives from source subdomains
         if (njsnd .gt. 0) then
            do 1500 n = 1, njsnd
               allocate(sjbuf(n)%s(njs_tot(n),nvars))
               call mpi_irecv(sjbuf(n)%s, nvars*njs_tot(n), MPI_REAL8,  &
                              njs_sub(n)-1, tag, mpicomw, mpi_reqn(n), iermpi)
1500        continue
         endif

! Send to destination subdomains
         if (njrcv .gt. 0) then
            rmlpt = 1./mlpt
            do 1600 n = 1, njrcv
               allocate(rjbuf(n)%s(njr_tot(n),nvars))
               do 1700 nl = 1, nvars
                  do 1800 k = njr_beg(n), njr_end(n)
                     arrib = 0.
                     do 1900 k2 = (k-1)*mlpt+1, k*mlpt
                        arrib = arrib + arr_ibis(k2,nl)
1900                 continue
                     arrib = rmlpt * arrib

! To get bit-for-bit agreement when comparing mlpt values for
!   uniform vegetation, uncomment the following statement:
!           arrib = arr_ibis(k*mlpt,nl)
                     rjbuf(n)%s(k-njr_beg(n)+1,nl) = arrib
1800              continue
1700           continue
               call mpi_send(rjbuf(n)%s, nvars*njr_tot(n), MPI_REAL8,    &
                             njr_sub(n)-1, tag, mpicomw, iermpi)
               deallocate(rjbuf(n)%s)
1600        continue
         endif
! Confirm receives from source subdomains
         if (njsnd .gt. 0) then
            call mpi_waitall(njsnd, mpi_reqn, mpi_statusn, iermpi)
            do 2000 n = 1, njsnd
               do 2100 nl = 1, nvars
                  do 2200 k = njs_beg(n), njs_end(n)
                     arr_ccm(ixy1(k),jxy1(k),nl) = sjbuf(n)%s(k-njs_beg(n)+1,nl)
2200              continue
2100           continue
               deallocate(sjbuf(n)%s)
2000        continue
         endif
#endif
         return
      end subroutine ibis_to_ccm
end module intermap
