#include "inland_config.h"
!
! #    #    ##       #    #    #
! ##  ##   #  #      #    ##   #
! # ## #  #    #     #    # #  #
! #    #  ######     #    #  # #
! #    #  #    #     #    #   ##
! #    #  #    #     #    #    # OFFLINE! :P
!
! Ported from Sage's IBIS as of 2011-06-02
! ---------------------------------------------------------------

program main !main_offline!
! ---------------------------------------------------------------
! common modules
#ifndef SINGLE_POINT_MODEL
      use inland_ibispar
      use inland_ibisctl, only: jday, iday, imonth, iyear, iyear0, idailyout, &
                                imonthout, iyearout, isimveg, isimfire,       &
                                isimco2, nspinsoil, spinmax, eqyears, istep,  &
                                spincons, spinfrac, ipointout, ccmexist
      use inland_comatm, only: co2conc, o2conc
#else /* SINGLE_POINT_MODEL */
!     TODO: verify if there are variables that should be no longer used - fzm
      use inland_ibisparm, only: npoi, nsoilay, ndaypm, dtime, hvap, lbeg,   &
                                 lend, mpt, ndaypy, nlon, nlat, rhow, xres,  &
                                 yres, indir, infile, datadir, outdir
      use inland_ibisctl, only: jday, iday, imonth, iyear, iday0, imonth0,   &
                                iyear0, idailyout, imonthout, iyearout,      &
                                isimveg, isimfire, isimco2, nspinsoil, istep,&
                                spinmax, eqyears, spincons, spinfrac,        &
                                ipointout, ccmexist, nan
      use inland_com1d, only: firb, fsena, abupi, fvapa, abupd
      use inland_comatm, only: fira, co2conc, o2conc, solai, asuri, solad, asurd
      use inland_comhyd, only: gsuvap, grunof, gdrain, gtrans, ginvap
      use inland_comsoi, only: tg, hsoi, soihfl, poros, upsoil, tsoi, swilt, &
                               wsoi, rhosoi, csoi
      use inland_comsum, only: ayanpptot, wtot
      use inland_comveg, only: cbioltot, cbiortot, cbiowtot, chl, chs, chu,  &
                               sai, tco2mic, tu, ts, tl, lai, tco2root,      &
                               tnpptot, totlit, cbiol, cbior, cbiow, fl, fu, &
                               tgpptot, totcsoi, aleaf, awood, aroot,        &
                               totlail, totlaiu, wliql, wliqs, wliqu,        &
                               tneetot, clitll, clitlm, clitls, clitrl,      &
                               clitrm, clitrs, clitwl, clitwm, clitws,       &
                               topparl, topparu, falll
      use inland_comforc, only: xprec, xcld, xlati, xlin, xsin, xta, xqa,    &
                                xua, dimforc
#endif /* SINGLE_POINT_MODEL */
      use inland_comdiag, only: diagstart, diagend, ndiagpt, nfreq
      use inland_lsmmapib, only: lati, loni

      implicit none

! local variables
      integer istyrd,      & ! first yr daily means exist (from precip file)
              !istyrm,     & ! first yr monthly anoms exist (from file)
              iwest,       & ! 1st lon index for subset
              iy1,         & ! first year for year loop
              iy2,         & ! last year for year loop
              iyrdaily,    & ! year to start reading daily means
              iyrlast,     & ! last year of previous run (for restart)
              iyrmon,      & ! month year of previous run (for restart)
! The below variable has been inherited from CCSM3-ibis.
              imonthlast,  & ! last month of previous run (for restart)
              idiag,       & ! number of diagnostic files requested
              irestart,    & ! 0: normal mode 1: restart mode
              iyrrestart,  & ! Year to restart model
              jnorth,      & ! 1st lat index for subset
              niter,       & ! total number of time iterations per day
              nday,        & ! number of days since the start of the simulation
              nrun,        & ! # of years in this run
              plen,        & ! length of precipitation event in timesteps (see plens)
              plenmax,     & ! upper bound on plen
              plenmin,     & ! lower bound on plen

              seed,        & ! first value used to initialize the random # generator
              seed2,       & ! second value used to initialize the random # generator
              seed3(32),   & ! third value used to initialize the random # generator
              seed4,       & ! fourth value used to initialize the random # generator

              spin,        & ! counter for iterations used to spin up soilbgc
              soilcspin,   & ! 0: no spinup procedure for soil c  1: acceleration procedure used
              lun,         & ! file indices
              timed,       & ! time of day since midnight (in seconds)
              i              ! loop indices

      integer daystart       ! index of when to start the day loop
#ifdef SINGLE_POINT_MODEL
! Imbuzeiro: linenum is created to print the input files (xta, xqa, xua, xprec,
!            xsin, xlin) as an output
      integer linenum,     & ! line number
              monthstart,  & ! index of when to start the month loop
              isoilforc,   & ! 0: fixed soil, 1: dynamic soil
              timel,       & ! local time of day since midnight (in seconds)
              nc             ! 0D specific loop indice

      real *8 rn,          & ! net radiation flux (short and long waves)
              pari,        & ! incoming PAR
              apar,        & ! APAR
              paro,        & ! outgoing PAR
              reflsw         ! reflected solar radiation

! Hewlley's variables as of june, 2007
! FIXME: this is no place for this kind of variables.
!       -fzm
      real*8  swnet,       & ! net shortware radiation
              lwnet,       & ! net longwave radiation
              vegt,        & ! vegetation canopy temperature
              avgsurft,    & ! average surface temperature
              surfalb,     & ! surface albedo
              soilwet,     & ! total soil wetness
              canopint,    & ! total canopy water storage
              rootmoist,   & ! root zone soil moisture
              canheat,     & ! canopy heat storage (J.m^-2)
              canheatl,    & ! last canopy heat storage (J.m^-2)
              delcanheat,  & ! change incanopy heat storage (J.m^-2)
              delsoilmoist,& ! change in soil moisture (kg.h2o.m^-2)
              soilmoistsum,& ! soil moisture in all layers (kg.h2o.m^-2)
              soilmoist(nsoilay), & ! soil moisture in which layer (kg.h2o.m^-2)
              soilmoistl,  & ! last soil moisture (kg.h2o.m^-2)
              autoresp,    & ! autotrophic respiration (kg.m^-2.s^-2)
              surfheat,    & ! surface heat storage (J.m^-2)
              delsurfheat, & ! change in surface heat storage (J.m^-2)
              surfheatl,   & ! last surface heat storage (J.m^-2)
              delsurfstor, & ! change in surface heat storage (J.m^-2)
              wtotl,    & ! total wather stored in soil, vegetation and snow
!                            ! at previous timestep (kg.h2o.m^-2.s^-1)
              totsoilcarb, & ! total soil carbon (kg.C.m^-2)
! FIXME: surfalb or albedo? are them the same? -fzm
              albedo,      & ! surface albedo
              solartot,    & ! total incoming radiation (dir+diff,vis+nIR)
! FIXME: avgsurft or soiltemp? are them the same? -fzm
              soiltemp,    & ! average layer soil temperature (K)
              undefined,   & ! undefined by albedo
              delwtot,     & ! change in total water stored (kg.h2o.m^-2.s^-1)
              fpar,        & ! absorbed fraction of PAR
              totlai,      & ! total ecosystem lai
              totlivbio,   & ! total living biomass (kg.C.m^-2)
              agb            ! above ground biomass
#endif /* SINGLE_POINT_MODEL */

      real*8  co2init,     & ! initial co2 concentration in mol/mol
              o2init,      & ! initial o2 concentration in mol/mol
              dran,        & ! random number from generator
              plens,       & ! length of precipitation event in seconds
              startp,      & ! time to start precipitation event (seconds since midnight)
              endp,        & ! time to end precipitation event (seconds since midnight)
              slope,       & ! rate of decrease of number of iteration for soil C spinup 
              snorth,      & ! north latitude for subsetting std grid
              ssouth,      & ! south latitude for subsetting std grid
              swest,       & ! west longitude for subsetting std grid
              seast,       & ! east longitude for subsetting std grid
              test           ! test on timestep
	      
      character*1024 filen
      character*4    chyear
      character*255  chtmp

#ifndef SINGLE_POINT_MODEL
! External
      real ran2              ! Function : random number generator
#endif /* SINGLE_POINT_MODEL */

! default values for subset grid
      data snorth, ssouth, swest, seast / 90., -90., -180., 180. /

! Ibis - CCSM3 variables
      logical rdlsf        ! set initial data for Antarctica, Greenland if true
      integer loopi,      &! index of little vector in big vector
              kpti,       &! start of vector segment
              kptj         ! end of vector segment

! namelist declaration
#ifndef SINGLE_POINT_MODEL
      namelist/INLAND_GRID/ irestart, iyrrestart, iyear0, nrun, iyrdaily, iyrmon, dtime, soilcspin, isimveg, isimfire, isimco2, co2init, o2init, iyearout, imonthout, idailyout, idiag, snorth, ssouth, swest, seast
#else
      namelist/INLAND_SINGLE_POINT/ irestart, iyear0, nrun, iyrdaily, dtime, soilcspin, isimveg, isimfire, isoilforc, isimco2, co2init, o2init, iyearout, imonthout, idailyout, idiag, snorth, ssouth, swest, seast
#endif /* SINGLE_POINT_MODEL */

! multiproc variables
      integer ithread, numthreads 
#ifdef _OPENMP
#ifndef SINGLE_POINT_MODEL
      !     USE omp_lib
      INTEGER OMP_GET_MAX_THREADS
      INTEGER OMP_GET_NUM_THREADS
      INTEGER OMP_GET_NUM_PROCS
      INTEGER OMP_GET_THREAD_NUM     
#endif
#endif

! time report variables
#ifdef __GFORTRAN__
      real, dimension(2) :: tarray
      real tresult
      integer telapsed
      telapsed = time()
#endif

      ipointout=0

! ---------------------------------------------------------------
!                       j o b   c o n t r o l 
! ---------------------------------------------------------------
!
! multiproc variables - put default values for now
      ithread = 1
      loopi = 1
      numthreads = 1
#ifndef SINGLE_POINT_MODEL
       numlv = numthreads
#endif /* SINGLE_POINT_MODEL */

! model I/O variables
      call rd_env

! open input file (namelist)
! TODO: add option to specify infile file name in command line.
! FIXME: Verify if every variable is allocated or it is needed another
!      'ibisalloc' call here.

! setup defaults - document any changes here in inland-grid.namelist and inland-single_point.namelist
! temporal parameters
      irestart   = 0        ! 0: not a restart run  1: restart run (default 0)
      iyrrestart = 9999     ! year to restart model (default 9999)
      iyear0     = 1981     ! initial year of simulation (don't change for restart) (default 1981)
      nrun       = 2        ! number of years in this simulation (change for restart) (default 2)
      iyrdaily   = 9999     ! year to start reading daily data (ditto) (default 9999)
      iyrmon     = 9999     ! year to start reading montly data (default 9999)
      dtime      = 3600     ! time step in seconds (default 3600)
!
! simulation parameters
      soilcspin  = 0        ! 0: no soil spinup, 1: acceleration procedure used (default 0)
      isimveg    = 0        ! 0: static veg, 1: dynamic veg, 2: dynamic veg-cold start (default 0)
      isimfire   = 0        ! 0: fixed fire, 1: dynam fire (default 0)
      isimco2    = 0        ! 0: fixed co2,  1: ramped co2 (default 0)
      co2init    = 0.000350 ! initial co2 concentration in mol/mol (real) (default 0.000350)
      o2init     = 0.209000 ! initial o2 concentration in mol/mol (real) (default 0.209000)
!
! output parameters
      iyearout   = 1        ! 0: no yearly output, 1: yearly output (default 1)
      imonthout  = 1        ! 0: no monthly output, 1: monthly output (default 1)
      idailyout  = 0        ! 0: no daily output, 1: daily output (default 0)
      idiag      = 0        ! idiag      0: no diagnostic output, 1-10 # of files to output (default 0)
!
! domain parameters - no defaults, use 999.0 to test values were set
      snorth     = 999.0    ! northern latitude for subsetting in/output (no default)
      ssouth     = 999.0    ! southern latitude for subsetting in/output (no default)
      swest      = 999.0    ! western longitude for subsetting in/output (no default)
      seast      = 999.0    ! eastern longitude for subsetting in/output (no default)

! different defaults for single_point
#ifdef SINGLE_POINT_MODEL
      isoilforc  = 0        ! 0: dynamic soil  physics 1: forced soil physics (default 0)
      iyear0     = 1992
      nrun       = 12
      iyearout   = 0
      imonth     = 0
      isimveg    = 1
#endif

!read namelist
       lun = 12
      print *,trim(infile)
      open (lun, status='old', file=trim(infile))
#ifndef SINGLE_POINT_MODEL
      read (lun, end=99, NML=INLAND_GRID)
#else
      read (lun, end=99, NML=INLAND_SINGLE_POINT)
#endif
99    close(lun)

! verify that domain values were read and are correct
      if ( snorth .eq. 999.0 .or. ssouth .eq. 999.0 .or. swest .eq. 999.0 .or. seast .eq. 999.0 ) then
         write (*,*) 'ERROR: domain dimensions not set in '//trim(infile)
         stop 1
      end if
      if ( snorth .gt. 90.0 .or. snorth .lt. -90.0 .or. ssouth .gt. 90.0 .or. ssouth .lt. -90.0 .or. &
           swest .gt. 180.0 .or. swest .lt. -180.0 .or. seast .gt. 180.0 .or. seast .lt. -180.0 ) then
         write (*,*) 'ERROR: domain dimensions exceed bounds (+90N -90S -180W +180E)'
         write (*,9025) snorth, ssouth, swest, seast
         stop 1
      end if
      if ( snorth .lt. ssouth .or. swest .gt. seast ) then
         write (*,*) 'ERROR: domain dimensions must be N>S / W<E'
         write (*,9025) snorth, ssouth, swest, seast
         stop 1
      end if
      

! why was this in the inland.infile section?
#ifdef SINGLE_POINT_MODEL
      linenum = 0
#endif

! we are not coupled. ccmexist (from ibisctl) is 0.
      ccmexist = 0

! tell user about the simulation
      write (*,*) ' '
      write (*,*) '**************************************'
      write (*,*) '* InLand - Surface model for the MBSCG'
      write (*,*) '**************************************'
      write (*,*) '* based on IBIS v2.6b4'
      write (*,*) '* Version ',PACKAGE_VERSION,', November 2012'
#ifdef SINGLE_POINT_MODEL
      write (*,*) '* 0D: Single Point Model version.'
#endif /* SINGLE_POINT_MODEL */
      write (*,*) '**************************************'
      write (*,*) ' '
      if (irestart .eq. 1) then
         write (*,*) 'INFO: running in restart mode'
#ifdef SINGLE_POINT_MODEL
         write (*,*) 'ERROR: Single Point Model (0D) does NOT support restart.' 
         write (*,*) 'Please turn off restart (set irestart to 0) in config file.'
         stop 1
#endif /* SINGLE_POINT_MODEL */
      end if
      write (*,9010) xres, yres
      write (*,9020) nlon, nlat
      write (*,9025) snorth, ssouth, swest, seast
      write (*,*) ' '
      write (*,9041) trim(indir)
      write (*,9042) trim(infile)
      write (*,9043) trim(datadir)
      if ( trim(outdir) .ne. '.' ) then
         write (*,9044) trim(outdir)
      end if
#ifndef SINGLE_POINT_MODEL
      if ( ran2val .ne. 0. ) then
         write (*,9045) ran2val
      end if
#endif /* SINGLE_POINT_MODEL */
      write (*,*) ' '
      write (*,9000) nrun

9000  format (1x,'INFO: length of this simulation (years)   : ',i8)
9006  format (1x,'INFO: number of iterations per day        : ',i8)
9010  format (1x,'INFO: model lon, lat resolution (degrees) : ',2f8.2)
9020  format (1x,'INFO: model domain (nlon x nlat)          :     ',i3,'  x  ',i3)  
9025  format (1x,'INFO: model limits (N,S,W,E)              : ',f8.2,' ',f8.2,' ',f8.2,' ',f8.2)  
9030  format (1x,'INFO: last year run in this sequence      : ',i8)
9040  format (1x,'INFO: number of threads                   : ',i8)
9041  format (1x,'INFO: input directory                     :     ',a)
9042  format (1x,'INFO: input file                          :     ',a)
9043  format (1x,'INFO: data directory                      :     ',a)
9044  format (1x,'INFO: output directory                    :     ',a)
9045  format (1x,'INFO: randomval                           :  ',E15.5)
9055  format (1x,'NOTICE - setting numthreads to ',i2,' instead of OMP_GET_MAX_THREADS ( ',i2,' )')

      if (iyrdaily.eq.iyrmon .and. iyrdaily.ne.9999) then
          write(*,*) 'ERROR: cannot run because iyrdaily:',iyrdaily,' equal irymon:', iyrmon
          stop 2
      endif

! ---------------------------------------------------------------
!                     t i m e   c o n t r o l 
! ---------------------------------------------------------------
! determine the number of timesteps per day
      niter = int(86400.0 / dtime)

! test on length of the time step and total number of iteration per day
      test = dmod(dble(86400.0), dtime)

      if (test .gt. 1.e-20) then
         write (*,*) 'ERROR: dtime ', dtime, ' should be divisible into 86400'  
         stop 3
      else
         write (*,9006) niter
      end if

! check for restart conditions and set "iyrlast" accordingly, open
! ibis.annual in append mode
      if (irestart .eq. 1) then
         write(chyear(1:4),'(i4)') iyrrestart
         filen = trim(outdir)//'/yearsrun.'//chyear//'.dat'
         open(13, status='old', file=filen, err=9050)
         read(13,*) iyrlast
! those variables to turn possible restart of ran2 subroutine
         read(13,*) seed
         read(13,*) seed2
         read(13,*) seed3
         read(13,*) seed4
         close(13)
         goto 9059
  9050   write(*,*) 'ERROR: ibis.infile restart flag=1, indicating restart, but output/yearsrun.',iyrrestart,'.dat not found'
         stop 4
         write(*,*) 'beginning from ', iyear0
         iyrlast = iyear0 - 1
         open(20,file='ibis.out.global',status='unknown',access='append')
         open(30,file='ibis.out.vegtype',status='unknown',access='append')
      else
         iyrlast = iyear0 - 1
9059  end if

      write (*,9030) nrun + iyrlast
      write (*,*) ' '

! lbeg, lend, mpt, kti and kptj now set in readit subroutine

! ---------------------------------------------------------------
!                   i n i t i a l i z a t i o n
! ---------------------------------------------------------------
! Allocate variables needed by rd_param and readit (except those with npoi dim).
      call inland_prealloc

! call RD_PARAM to read in all parameter files
      call rd_param

! initial concentration of co2, o2
      co2conc = co2init
      o2conc  = o2init

! read global boundary condition datasets
      call readit(isimveg,snorth,ssouth,swest,seast,iwest,jnorth)

! multiproc variables
#ifdef _OPENMP
#ifndef SINGLE_POINT_MODEL
      numthreads = OMP_GET_MAX_THREADS() 

      ! make sure numthreads <= 2, unless explicitly set with OMP_NUM_THREADS
      call getenv("OMP_NUM_THREADS", chtmp)
      if ( numthreads .gt. 2 .and. trim(chtmp) .eq. "" ) then
         print *, ''
         write (*,9055) 2, OMP_GET_MAX_THREADS()
         numthreads = 2
         numlv = numthreads
         call OMP_SET_NUM_THREADS(numthreads)
      end if

      ! make sure numthreads <= npoi
      if ( numthreads > npoi ) then
         print *, ''
         write (*,9055) npoi, OMP_GET_MAX_THREADS()
         numthreads = npoi
         call omp_set_num_threads(numthreads)
      end if
      numlv = numthreads
      write (*,9040), numthreads 
#endif
#endif

!-----------------
! Data structures allocation: call ibisalloc
! TODO: maybe we need setting a 'mpt' variable, according to ibisini subroutine.
!      Sometimes you can see 'var(mpt)' and other 'var(lbeg:lend)' to the same
!      sized variables in SAGE IBIS 'var(npoi)'.
! ibisalloc -must- be after readit() for nlonsub+nlatsub to be already defined
! at the time of allocation.
      call ibisalloc

#ifdef SINGLE_POINT_MODEL
! read forcing data
      call readforc(isoilforc)

! FIXME: 'test' variable. No clue what it is for, this is a badly written code.
      test = dimforc
#endif /* SINGLE_POINT_MODEL */

! map coupled model variables based on variables read from data files
      call ibismap_offline(imonthlast, rdlsf, loopi, kpti, kptj, ipointout)

! check if diagnostic output is requested, if so read info from 'diag.infile'
! TODO: not supported yet, can be added as a later feature. Just keep '0' at
!       conf/inland-<model>.infile for 'idiag'.
      !if (idiag .ne. 0) call inidiag(idiag)

#ifndef SINGLE_POINT_MODEL
! preliminary analysis of climate data
      if (irestart.eq.0 .or. isimveg.eq.0) then
         call climanl
      end if

      if (iyrdaily .le. iyrlast+nrun) then
         write(chyear(1:4),'(i4)') iyrdaily
         filen = trim(datadir)//'/daily/prec.daily.'//chyear//'.nc'
         call inird(filen,istyrd)
         if (istyrd .le. 0) istyrd = 9999
      end if

!    If first year of this run/restart is an anomaly year, then read in 
! anomalies for month 12 of previous year and month 1 of this year
! (note: rdanom reads imonth+1) 
      iy2 = iyrlast + nrun
#endif /* SINGLE_POINT_MODEL */

! initialize the model
      call initialib(isimveg, irestart, iyrlast, imonthlast, rdlsf, lati, loni)

#ifndef SINGLE_POINT_MODEL
! initialize random number generator seed
      if(irestart.eq.0) then
         seed = -1
         seed2 = 123456789
         seed3 = 0
         seed4 = 0
         dran = ran2 (seed,seed2,seed3,seed4)
      endif
#endif /* SINGLE_POINT_MODEL */

! ---------------------------------------------------------------
!              s t a r t   o f   t i m e   l o o p s
! ---------------------------------------------------------------
      write (*,*) ' '
      write (*,*) '********************************'
      write (*,*) '* start of the IBIS simulation *'
      write (*,*) '********************************'
      write (*,*) ' '
#ifdef SINGLE_POINT_MODEL
! Initialize the output file from outside the time loops -fzm
! Write half-hourly output
! Imbuzeiro: Added the variable names in the output file
      open(40,file='output/single_point-output.csv',status='unknown')
      write(40,9199) 'time','Swnet','Lwnet','Qle','Qh','Qg','DelCanHeat',   &
                     'DelSurfHeat','Evap','Qs','Qrec','Qsb','Qt',           &
                     'DelSoilMoist','DelSurfStor','DelIntercept',           &
                     'WaterTableD','VegT','BaresoilT','AvgSurfT','Albedo',  &
                     'SurfStor','fPAR','SoilMoist1','SoilMoist2',           &
                     'SoilMoist3','SoilMoist4','SoilMoist5','SoilMoist6',   &
                     'SoilTemp1','SoilTemp2','SoilTemp3','SoilTemp4',       &
                     'SoilTemp5','SoilTemp6','SoilWet','Ecanop','Tveg',     &
                     'Esoil','Ewater','RootMoist','CanopInt','GPP',         &
                     'NPP','NEE','AutoResp','HeteroResp','ANPP','BgResp',   &
                     'LitterFall','CarbPools1','CarbPools2','CarbPools3',   &
                     'CarbPools4','CarbPools5','CarbPools6','CarbPools7',   &
                     'CarbPools8','CarbPools9','CarbPools10','CarbPools11', &
                     'CarbPools12','CarbPools13','TotLivBiom','AbvGrndWood',&
                     'LAI','Tair','Qair','Wind','Rainf','Psurf','Swdown',   &
                     'Lwdown','CO2air'
9199  format (74(a,x))
#endif /* SINGLE_POINT_MODEL */

! reset elapsed number of days, accumulating past days if restart mode
      nday = 0

      if (irestart .eq. 1) then
         do 150 iyear = iyear0, iyrlast
            nday = nday + 365
            if (mod(iyear,4).eq.0) then
               if (mod(iyear,100).ne.0) then
                  nday = nday + 1
               elseif (mod(iyear/100,4).eq.0) then
                  nday = nday + 1
               end if
            end if
150      continue
      end if

!---------------------- YEAR YEAR YEAR -------------------------------------
! start of yearly loop
!---------------------- YEAR YEAR YEAR -------------------------------------
      iy1 = iyrlast + 1
      iy2 = iyrlast + nrun
      do 200 iyear = iy1, iy2
! reset julian date
         jday = 0

!    Determine the calendar for this year.
!    Leap years occur every calendar year that is divisible by 4 (such as
! 1980, 1984, etc.) *except* for years which begin centuries not divisible
! by 400
!    For example, 1700, 1800, and 1900 are *not* leap years, but 1600
! and 2000 *are* leap years
! NOTICE: CCSM3 version was using individual ndaypy on soilbgc, sumnow, sumyear,
!        and wyearly subroutines. The value was fixed at 365. This might bring
!        trouble as we fixed back to SAGE IBIS behavior.
         ndaypm(2) = 28
         ndaypy = 365

! 0D IBIS does not consider leap years.
#ifndef SINGLE_POINT_MODEL
         if (mod(iyear,4).eq.0) then
            if (mod(iyear,100).ne.0) then
               ndaypm(2) = 29
               ndaypy = 366
            elseif (mod(iyear/100,4).eq.0) then
               ndaypm(2) = 29
               ndaypy = 366
            end if
         end if
#endif /* SINGLE_POINT_MODEL */

!---------------------- MONTH MONTH MONTH ----------------------------------
! start of monthly loop
!---------------------- MONTH MONTH MONTH ----------------------------------
#ifdef SINGLE_POINT_MODEL
! Observed, step data, might not start on first month of year.
         if (iyear.eq.iy1) then 
            monthstart = imonth0
         else
            monthstart = 1
         endif
         do 210 imonth = monthstart, 12
! rdanom is not called on single point model.
#else /* SINGLE_POINT_MODEL */
         do 210 imonth = 1, 12
!            call rdanom(iyranom,nanom,iy2,istyrm,iwest,jnorth)
#endif /* SINGLE_POINT_MODEL */

!------------------------- DAY DAY DAY -------------------------------------
! start of daily loop
!------------------------- DAY DAY DAY -------------------------------------
               daystart = 1
#ifdef SINGLE_POINT_MODEL
! Observed, step data, might not start on first day of the month for the first
!   month.
            if ( (iyear.eq.iy1).and.(imonth.eq.imonth0) ) then
               daystart = iday0
            endif
#endif /* SINGLE_POINT_MODEL */

            do 220 iday = daystart, ndaypm(imonth)

! update user on model progress once a day
               write (STDOUT,9100,advance='no') char(13), char(13), iyear, imonth, iday
#ifdef IFORT_NOFLUSH_BUG
               close(STDOUT)
               open(STDOUT)
#endif
#ifdef PGF90_NOFLUSH_BUG
               call flush(STDOUT)
#endif
9100           format (a,79x,a,'Running now: ',i4.4,'/',i2.2,'/',i2.2)
9101           format (a,79x,a,'Running now: ',i4.4,'/',i2.2,'/',i2.2,a)
9102           format (a,79x,a,'Completed ',i4.4,'/',i2.2,'/',i2.2)

! update elapsed number of days and julian date
               nday = nday + 1
               jday = jday + 1

#ifdef SINGLE_POINT_MODEL
               call daily(test,dimforc)
#else /* SINGLE_POINT_MODEL */
! get daily means (if requested)
! and determine today's climatic conditions

! TODO: port the weather generator!
!      maybe we need to sed the kpti-kptj interval to allow parallelizing the
!      weather generator. by now, we just use lbeg:lend but keep your eyes open!
               if (iyear.ge.iyrdaily) then
                  call rdday(jday,istyrd,iwest,jnorth)
                  call daily(seed,seed2,seed3,seed4,1,iyrlast,nrun)
               else
                  if (iyear.ge.iyrmon) then
                     call rdmon(iyear, iwest, jnorth)
                     call dailymon(seed,seed2,seed3,seed4,0,iyrmon,iyrlast,nrun)
                  else
                     call daily(seed,seed2,seed3,seed4,0,iyrlast,nrun)
                  endif
               end if
#endif /* SINGLE_POINT_MODEL */

! determine the daily vegetation cover characteristics
               call pheno(kpti, kptj)

! call soil biogeochemistry model
!
! soil spin up procedure calls soil bgc model spincons times
! for each time step to spin up the process (accelerate C & N pools)
               spinfrac  = 0.75     ! fraction of nspinsoil time used to
!                                   ! spin up soil at maximum spin up rate
!
               spincons  = 40.0     ! number of times soilbgc subroutine is
!                                   ! called for each day in simulation during
!                                   ! acceleration procedure
!
               eqyears   = 50       ! years used past spin up to slowly bring
!                                   ! entire soil to a steady equilibrium
!
               nspinsoil=iyear0+150 ! year of simulation that soil c reaches equilibrium

               if (soilcspin .eq. 1) then
                  if ((iyear-iyear0) .le. (spinfrac*(nspinsoil-iyear0-eqyears))) then
                     spinmax = int(spincons)
                  elseif ((iyear-iyear0) .lt. (nspinsoil-iyear0-eqyears)) then
                     slope   = spincons / ((nspinsoil - iyear0 - eqyears) -   &
                               (spinfrac * (nspinsoil - iyear0 - eqyears)))
                     spinmax = int(spincons - (slope * ((iyear - iyear0) -    &
                               (spinfrac * (nspinsoil - iyear0 - eqyears)))))
                     spinmax = max(spinmax,1)
                  else
                     spinmax = 1
                  endif
               else 
                  spinmax = 1
               endif

               do 230 spin = 1, spinmax
                  call soilbgc(spin,kpti,kptj)
230            continue

#ifndef SINGLE_POINT_MODEL
! determine the length of a precipitation event (between 4 and 24 hours),
! and time to start and end precipitation event. plen is in timesteps, while
! plens, startp, and endp are in seconds
               plenmin = 1 + int((4.0 * 3600. - 1.) / dtime)
               plenmax = max(int(24.0 * 3600. / dtime), plenmin)
               plen = min(plenmax,int(plenmin+ran2(seed,seed2,seed3,seed4) * (plenmax-plenmin+1)))
               plens   = dtime * plen
               startp  = dtime * min(niter-plen, int(ran2(seed,seed2,seed3,seed4)*(niter-plen+1)))
               endp    = startp + plens
#endif /* SINGLE_POINT_MODEL */

!---------------------- HOUR HOUR HOUR -------------------------------------
! start of hourly loop
! if SINGLE_POINT_MODEL (0D), then this is half-hourly loop.
!---------------------- HOUR HOUR HOUR -------------------------------------
               do 240 istep = 1, niter

! calculate the time of day since midnight (in seconds)
                  if (dtime - int(dtime) .ne. 0) then
                     write(*,*) "ERROR: Time slice is not integer!"
                     stop 5
                  endif
                  timed = (istep - 1) * int(dtime)

#ifdef SINGLE_POINT_MODEL
! determine climatic conditions for the given timestep
                  call diurnal(timed, jday, test, dimforc)
! call the land surface model
                  call lsxmain(loopi, kpti, kptj, test, dimforc, isoilforc)
#else /* SINGLE_POINT_MODEL */
! determine climatic conditions for the given timestep
                  call diurnal(timed, jday, plens, startp, endp, seed, seed2,seed3,seed4)

! call the land surface model

! for now openmp is here only, because 80% of work is in lsxmain, 
! and diurnal cannot be parallelized without "fixing" weather generator
#ifdef _OPENMP
#ifndef SINGLE_POINT_MODEL
      !$OMP PARALLEL DEFAULT(SHARED),PRIVATE(loopi,kpti,kptj,istep,timed,ithread,mpt)
      ithread = OMP_GET_THREAD_NUM()+1
      loopi = ithread
      !loopi = 1
      kpti = 1 + (ithread-1)*(npoi/numthreads) 
      kptj = kpti + (npoi/numthreads) - 1  ! test when 1 numthreads
      if ( ithread .eq. numthreads ) then
         if ( kptj .ne. npoi ) then
            kptj = npoi
         end if
      end if 
      mpt = kptj - kpti + 1
      !print *,'ithread=',ithread,'thread=',OMP_GET_THREAD_NUM(),'kpti=',kpti,'kptj=',kptj
#endif
#endif

                  call lsxmain(loopi, kpti, kptj)

#ifdef _OPENMP
#ifndef SINGLE_POINT_MODEL
      !$OMP END PARALLEL
#endif
#endif

#endif /* SINGLE_POINT_MODEL */

! accumulate some variables every timestep
                  call sumnow(kpti,kptj)
                  call sumday  (timed, loopi, kpti, kptj)
                  call summonth(timed, loopi, kpti, kptj)
                  call sumyear (timed, loopi, kpti, kptj)

! write out diagnostics
                  if (idiag .ne. 0) then
                     do 250 i = 1, idiag
                        if (iyear.ge.diagstart(i) .and. iyear.le.diagend(i) &
                            .and. ndiagpt(i).ge.1) then  
                           if (mod(istep,nfreq(i)).eq.0) then
                              call wdiag(i, istep)
                           end if
                        end if
250                  continue
                  end if

#ifdef SINGLE_POINT_MODEL
! FIXME ET: this should not be inside the main loop, but in a function or subroutine...
! energy budget of the surface
                  rn = solad(1,1) * (1-asurd(1,1)) + &
                       solad(1,2) * (1-asurd(1,2)) + &
                       solai(1,1) * (1-asuri(1,1)) + &
                       solai(1,2) * (1-asuri(1,2)) + fira(1) - firb(1)

                  pari = (solad(1,1) + solai(1,1)) * 4.59
                  apar = (topparl(1) + topparu(1)) + 4.59
                  paro = (solad(1,1) * asurd(1,1) + solai(1,1)*asuri(1,1))*4.59

                  swnet = solad(1,1) * (1-asurd(1,1)) + &
                          solad(1,2) * (1-asurd(1,2)) + &
                          solai(1,1) * (1-asuri(1,1)) + &
                          solai(1,2) * (1-asuri(1,2))

                  lwnet = (fira(1) - firb(1))

                  reflsw = solad(1,1)*asurd(1,1) + solad(1,2)*asurd(1,2) + &
                           solai(1,1)*asuri(1,1) + solai(1,2)*asuri(1,2)

                  solartot = solad(1,1) + solad(1,2) + solai(1,1) + solai(1,2)

! Imbuzeiro: Created to replace the NaN values to zero, because
!            in time whitout radiation the ibis write NaN and I
!            excluded the NaN to used the output file in my
!            netcdf program (.txt to .nc)
! In accordance to Hewlley Imbuzeiro, when solartot is 0, albedo must be an 
! 'expected' NaN (or -999.99 defined by the 'nan' variable on ibisctl) - fzm
                  if (solartot.ne.0) then
                     albedo = reflsw / solartot
                  else
                     albedo = nan
                  endif

                  vegt = ((chl*tl(1)) + (chs*ts(1)) + (chu*tu(1))) / &
                         (chs+chl+chu)

                  avgsurft = (tu(1) + tl(1) + tg(1)) / 3

! Surfalb is -999.99 (expected NaN) when solad+solai = 0 (no solar incidence)
                  if ((solad(1,1)+solai(1,1)).ne.0.) then
                     surfalb = ((solad(1,1)*(1-asurd(1,1))) + (solai(1,1) * &
                               (1-asuri(1,1)))) / (solad(1,1) + solai(1,1))
                  else
                     surfalb = nan
                  endif

                  soilwet = ((wsoi(1,1) - swilt(1,1) * hsoi(1) * poros(1,1)) + &
                            (wsoi(1,2) - swilt(1,2) * hsoi(2) * poros(1,2)) +  &
                            (wsoi(1,3) - swilt(1,3) * hsoi(3) * poros(1,3)) +  &
                            (wsoi(1,4) - swilt(1,4) * hsoi(4) * poros(1,4)) +  &
                            (wsoi(1,5) - swilt(1,5) * hsoi(5) * poros(1,5)) +  &
                            (wsoi(1,6) - swilt(1,6) * hsoi(6) * poros(1,6))) / &
                            ((1-swilt(1,1) * poros(1,1) * hsoi(1)) +           &
                            (1-swilt(1,2) * poros(1,2) * hsoi(2)) +            &
                            (1-swilt(1,3) * poros(1,3) * hsoi(3)) +            &
                            (1-swilt(1,4) * poros(1,4) * hsoi(4)) +            &
                            (1-swilt(1,5) * poros(1,5) * hsoi(5)) +            &
                            (1-swilt(1,6) * poros(1,6) * hsoi(6)))

                  canopint = (wliqu(1) * fu(1) * 2 * lai(1,2) + &
                              wliqs(1) * fu(1) * 2 * sai(1,2) + &
                              wliql(1) * fl(1) * 2 * (lai(1,1) + sai(1,1)))

                  rootmoist = (upsoil(1,1) + gsuvap(1))
                  totsoilcarb = tco2root(1) + tco2mic(1)
                  delcanheat = (swnet + lwnet - fvapa(1)*hvap - fsena(1) + &
                                soihfl(1)) * 3600

! Imbuzeiro: Correct the soilmoisture equation
! Turned the code into a do-loop for simplicity - fzm
                  soilmoistsum=0.
                  do 260 i=1,nsoilay
                     soilmoist(i) = (wsoi(1,i) * poros(1,i) * rhow * hsoi(i))
                     soilmoistsum = soilmoistsum + soilmoist(i)
260               continue

                  delsoilmoist = soilmoistsum - soilmoistl
                  soilmoistl = soilmoistsum

                  surfheat=0.
                  do 270 i=1,nsoilay
                     surfheat=surfheat+(csoi(1,i)*rhosoi(1,i)*tsoi(1,i)*hsoi(i))
270               continue

                  delsurfheat = surfheat - surfheatl
                  surfheatl = surfheat
                  
                  delwtot = wtot(1) - wtotl
                  wtotl = wtot(1)

                  autoresp = tgpptot(1) - tnpptot(1)
                  totsoilcarb = totcsoi(1) + totlit(1)

! Imbuzeiro: Created to replace the NaN values to zero, because
!            in time whitout radiation the ibis write NaN and I
!            excluded the NaN to used the output file in my
!            netcdf program (.txt to .nc)
! In accordance to Hewlley Imbuzeiro, when solartot is 0, albedo must be an 
! 'expected' NaN (or -999.99 defined by the 'nan' variable on ibisctl) - fzm
                  if (solad(1,1).ne.0) then
                     fpar = (solad(1,1)*abupd(1) + solai(1,1)*abupi(1)) / &
                            (solad(1,1) + solai(1,1))
                  else
                     fpar = nan
                  endif

! FIXME: make a DO loop for totlivbio, agb, cbio?tot all at once -fzm
                  totlivbio = cbiol(1,1) + cbior(1,1) + cbiow(1,1) + &
                              cbiol(1,2) + cbior(1,2) + cbiow(1,2) + &
                              cbiol(1,3) + cbior(1,3) + cbiow(1,3) + &
                              cbiol(1,4) + cbior(1,4) + cbiow(1,4) + &
                              cbiol(1,5) + cbior(1,5) + cbiow(1,5) + &
                              cbiol(1,6) + cbior(1,6) + cbiow(1,6) + &
                              cbiol(1,7) + cbior(1,7) + cbiow(1,7) + &
                              cbiol(1,8) + cbior(1,8) + cbiow(1,8) + &
                              cbiol(1,9) + cbior(1,9) + cbiow(1,9) + &
                              cbiol(1,10) + cbior(1,10) + cbiow(1,10) + &
                              cbiol(1,11) + cbior(1,11) + cbiow(1,11) + &
                              cbiol(1,12) + cbior(1,12) + cbiow(1,12)

                  agb = cbiol(1,1) + cbiow(1,1) + cbiol(1,2) + cbiow(1,2) + &
                        cbiol(1,3) + cbiow(1,3) + cbiol(1,4) + cbiow(1,4) + &
                        cbiol(1,5) + cbiow(1,5) + cbiol(1,6) + cbiow(1,6) + &
                        cbiol(1,7) + cbiow(1,7) + cbiol(1,8) + cbiow(1,8) + &
                        cbiol(1,9) + cbiow(1,9) + cbiol(1,10) + cbiow(1,10) + &
                        cbiol(1,11) + cbiow(1,11) + cbiol(1,12) + cbiow(1,12)

                  cbioltot(1) = cbiol(1,1) + cbiol(1,2) + cbiol(1,3) + &
                                cbiol(1,4) + cbiol(1,5) + cbiol(1,6) + &
                                cbiol(1,7) + cbiol(1,8) + cbiol(1,9) + &
                                cbiol(1,10) + cbiol(1,11) + cbiol(1,12)

                  cbiortot(1) = cbior(1,1) + cbior(1,2) + cbior(1,3) + &
                                cbior(1,4) + cbior(1,5) + cbior(1,6) + &
                                cbior(1,7) + cbior(1,8) + cbior(1,9) + &
                                cbior(1,10) + cbior(1,11) + cbior(1,12)

                  cbiowtot(1) = cbiow(1,1) + cbiow(1,2) + cbiow(1,3) + &
                                cbiow(1,4) + cbiow(1,5) + cbiow(1,6) + &
                                cbiow(1,7) + cbiow(1,8) + cbiow(1,9) + &
                                cbiow(1,10) + cbiow(1,11) + cbiow(1,12)

                  totlai = totlaiu(1) + totlail(1)

! Imbuzeiro: For the LBA-DMIP protocol I did a spinup but I only wanted
!            write the last 3 years. This information changes for other
!            sites and user specifications
!                  if ( (iyear.ge.2001).and.(iyear.le.2003) ) then
                     linenum = linenum + 1
                     write(40,9200) linenum, swnet, lwnet, -fvapa(1)*hvap,     &
                                    -fsena(1), soihfl(1), delcanheat, nan,     &
                                    -fvapa(1), grunof(1), nan, gdrain(1),      &
                                    gdrain(1)+grunof(1), delsoilmoist, nan,    &
                                    nan, nan, vegT,tsoi(1,1), nan, albedo, nan,&
                                    fpar,(soilmoist(i),i=1,nsoilay),           &
                                    (tsoi(1,i),i=1,nsoilay), soilwet,ginvap(1),&
                                    gtrans(1), gsuvap(1), nan,nan, canopint,   &
                                    tgpptot(1)*0.012,tnpptot(1)*0.012,         &
                                    -tneetot(1)*0.012,autoresp*0.012,          &
                                    tco2mic(1)*0.012,ayanpptot(1), nan,        &
                                    falll(1),cbioltot(1), cbiortot(1),         &
                                    cbiowtot(1),clitll(1), clitlm(1),          &
                                    clitls(1),clitrl(1), clitrm(1), clitrs(1), &
                                    clitwl(1), clitwm(1), clitws(1), agb,      &
                                    totlivbio, cbiowtot(1), totlai,            &
                                    xta(1,linenum)+273.15, xqa(1,linenum),     &
                                    xua(1,linenum), xprec(1,linenum)/1800, nan,&
                                    xsin(1,linenum), xlin(1,linenum), nan
!                  endif                       
9200              format(i10,x,74(f25.16,x))

                  test = test - 1
! end of half-hourly loop
                  if (test.eq.0.) goto 9999
#endif /* SINGLE_POINT_MODEL */

! ---------------------------------------------------------------
!               e n d   o f   t i m e   l o o p s
! ---------------------------------------------------------------

!---------------------- HOUR HOUR HOUR -------------------------------------
! end of the hourly loop
!---------------------- HOUR HOUR HOUR -------------------------------------
240            continue

#ifndef SINGLE_POINT_MODEL
! write out daily output
               if (idailyout.eq.1) then
                  write (STDOUT,9101,advance='no') char(13), char(13), iyear, &
                                                   imonth, iday,              &
                                                   ' *writing daily output*'
#ifdef IFORT_NOFLUSH_BUG
                  close(STDOUT)
                  open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
                  call flush(STDOUT)
#endif /* PFG90_NOFLUSH_BUG */

                  call wdaily(jday,nday) 

               endif

! for long runs, write current day to file
         open(12,file=trim(outdir)//'/daysrun.dat',status='unknown')
         write(12,*) iyear,imonth,iday
         close(12)

#endif /* SINGLE_POINT_MODEL */
!------------------------- DAY DAY DAY -------------------------------------
! end of the daily loop
!------------------------- DAY DAY DAY -------------------------------------
220         continue
!
! write out monthly output (use 1st day of this month because of GrADS bug)
#ifndef SINGLE_POINT_MODEL
            if (imonthout.eq.1) then
               write (STDOUT,9101,advance='no') char(13), char(13), iyear, &
                                                imonth, iday-1,            &
                                                ' *writing monthly output*'
#ifdef IFORT_NOFLUSH_BUG
               close(STDOUT)
               open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
               call flush(STDOUT)
#endif /* PGF90_NOFLUSH_BUG */

               call wmonthly(nday-ndaypm(imonth)+1)
            endif
#endif /* SINGLE_POINT_MODEL */
!---------------------- MONTH MONTH MONTH ----------------------------------
! end of the monthly loop
!---------------------- MONTH MONTH MONTH ----------------------------------
210      continue

#ifndef SINGLE_POINT_MODEL
! recalculate bioclimatic parameters if using anomalies
         if (iyear.ge.iyrdaily) call climanl2
#endif /* SINGLE_POINT_MODEL */

! NOTICE: Although Hewlley's version has CO2 commented out, you can disable it
!        just by not setting isimco2=1 on the infile.
! get new o2 and co2 concentrations for this year
         if (isimco2.eq.1) call co2(co2init)

! perform vegetation dynamics
         if (isimveg.ne.0) call dynaveg

! calculate simple annual diagnostic variables for the globe
         call gdiag

! calculate simple annual diagnostic variables for each vegetation type
         call vdiag

#ifndef SINGLE_POINT_MODEL
! write out annual output
         if (iyearout.eq.1) then
            write (STDOUT,9101,advance='no') char(13),char(13),iyear,imonth-1, &
                                        iday-1, ' *writing yearly output*'
#ifdef IFORT_NOFLUSH_BUG
            close(STDOUT)
            open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
            call flush(STDOUT)
#endif /* PGF90_NOFLUSH_BUG */

            call wyearly(nday)
         endif

! write restart files
         write (STDOUT,9101,advance='no') char(13), char(13), iyear, imonth-1, &
                                     iday-1, ' *writing restart data*'
#ifdef IFORT_NOFLUSH_BUG
         close(STDOUT)
         open(STDOUT)
#endif /* IFORT_NOFLUSH_BUG */

#ifdef PGF90_NOFLUSH_BUG
         call flush(STDOUT)
#endif /* PGF90_NOFLUSH_BUG */

! Now just restart for dinamic vegetation (option 1 only)
        if(isimveg.ge.1) call wrestart(nday, imonthlast)

#endif /* SINGLE_POINT_MODEL */

! update iyrlast value and file yearsrun.dat
         iyrlast = iyrlast + 1
!         open(12,file=trim(outdir)//'/yearsrun.dat',status='unknown')
!         write(filen,'(A16,I4,A4)')trim(outdir)//'/yearsrun.',iyrlast,'.dat'
         write(chyear(1:4),'(i4)') iyrlast
         filen = trim(outdir)//'/yearsrun.'//chyear//'.dat'
         open(12,file=filen,status='unknown')
         write(12,*) iyrlast
! uptade variables to turn possible restart of ran2 subroutine
         write(12,*) seed
         write(12,*) seed2
         write(12,*) seed3
         write(12,*) seed4
         close(12)
         write (STDOUT,9102) char(13), char(13), iyear, imonth-1, iday-1
!---------------------- YEAR YEAR YEAR -------------------------------------
! end of the yearly loop
!---------------------- YEAR YEAR YEAR -------------------------------------
200   continue

#ifdef SINGLE_POINT_MODEL
9999  continue
! If jumped here due to test = 0, then force calling the final g/vdiag.
      if (test.eq.0) then
         call gdiag
         call vdiag
      endif
#endif /* SINGLE_POINT_MODEL */

! end of the simulation
      write (*,*) ' '
      write (*,*) '*** end of run ***'

#ifdef __GFORTRAN__
      call ETIME(tarray, tresult)  
      telapsed = time() - telapsed
      write(*,*) ' '
      write(*,1001) 'elapsed time:', INT(telapsed), REAL(telapsed)/60.0
      write(*,1001) 'run time:    ', INT(tresult), tresult/60.0
      write(*,1001) 'user time:   ', INT(tarray(1)), tarray(1)/60.0
      write(*,1001) 'system time: ', INT(tarray(2)), tarray(2)/60.0
      write(*,*) ' '
#endif

1001           format (a,i10,' s (',f10.1,' m)')
1002           format (a,i4,a,i4,a,i4,'s (',f3.1,' m)')

      stop
end program main !main_offline
