#include "inland_config.h"

#ifdef SINGLE_POINT_MODEL
#error "This subroutine should NOT be compiled for 0D IBIS model option."
#endif

! This subroutine reads in daily fields..
! ---------------------------------------------------------------------
subroutine rdday(jday, istyr, iwest, jnorth)
! ---------------------------------------------------------------------
      use inland_ibispar
      use inland_ibisctl, only: imonth, iyear
      use inland_combcs
      use inland_comwork

      implicit none

! Arguments
      integer jday,   & ! day of the year
              istyr,  & ! 1st year in data files
              iwest,  & ! 1st lon index for subset
              jnorth    ! 1st lat index for subset

! Local variables
      integer i,    & ! loop indice on years after istyr
              istat   ! error flag for netcdf

      character*1024 filen
      character*1024 :: directory
      character*80 :: suffix
      integer istart(4), icount(4)

      data istart / 1,1,1,1 /, icount / nlon,nlat,1,1 /
      istart(1) = iwest
      istart(2) = jnorth
      icount(1) = nlonsub
      icount(2) = nlatsub

      if (iyear .lt. istyr) then
         print *, 'daily data begins in year ', istyr
         print *, 'not reading in daily data'
         stop 1
      end if

! count how many days since beginning of daily data
! daily data begin on Jan 1, istyr
      if (iyear .eq. istyr) then
         istart(4) = jday
      else
         istart(4) = 0
         do 10 i = istyr, iyear-1
            istart(4) = istart(4) + 365
            if (mod(i,4).eq.0) then
               if (mod(i,100).ne.0) then
                  istart(4) = istart(4) + 1
               else if (mod(i/100,4).eq.0) then
                  istart(4) = istart(4) + 1
               end if
            end if
10       continue
        istart(4) = jday
      end if

! read daily precip
      aname = 'prec'
      directory = trim(datadir)//'/daily/'
      write(suffix,'(A,I4,A)') '.daily.',iyear,'.nc'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, prec'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xinprecd(1) )

! read daily temp
      aname = 'temp'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, temp'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xintd(1) )

! read daily trange
!      aname = 'trange'
!      filen = 'input/daily/trange.daily.nc'
!      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
!                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
!                   istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in rdday, trange'
!         if (iyear .gt. 1997) then
!            print *, 'Attempted to read past last day in file?'
!         end if
!         stop 1
!      end if
!      call arr2vec( work, xintrngd(1) )

! read daily cloudiness
      aname = 'cld'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, cld'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xincldd(1) )

! read daily windspeed
      aname = 'wspd'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, wspd'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xinwindd(1) )

! read daily specific humidity
      aname = 'sphum'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, sphum'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xinqd(1) )

! read tmax temperature
      aname = 'tmax'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, tmax'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xintmaxd(1) )

! read tmin temperature
      aname = 'tmin'
      filen = trim(directory)//trim(aname)//trim(suffix)
      call readvar(filen,aname,'level',istart,icount,work,cdummy(1),          &
                   cdummy(nlonsub+1),cdummy(2*nlonsub+1),cdummy(3*nlonsub+1), &
                   istat)
      if (istat .ne. 0) then
         write(*,*) 'ERROR in rdday, tmin'
         if (iyear .gt. 1997) then
            print *, 'Attempted to read past last day in file?'
         end if
         stop 1
      end if
      call arr2vec( work, xintmind(1) )

      return
end subroutine rdday
