#include "inland_config.h"
!----------------------------------------------------------
logical function dir_exists(dir_name)

      implicit none

      character*255 dir_name
      logical :: dir_e

#if defined(__GFORTRAN__)
! gfortran does not support directory 
      inquire( file=trim(dir_name)//'/', exist=dir_e )
#elif defined(PGF90)
      inquire( file=trim(dir_name)//'/', exist=dir_e )
#else
! this works with ifort, test with others
!         inquire( directory=trim(dir_name), exist=dir_e )
     inquire( file=trim(dir_name)//'/', exist=dir_e )
#endif
      dir_exists = dir_e

end function dir_exists


!----------------------------------------------------------
subroutine rd_env
!----------------------------------------------------------
!  Read various environment variables
!
!---------------------------Code history--------------------------------
! Author: Etienne Tourigny
! 
!----------------------------------------------------------

      use inland_ibispar

      implicit none
  
! model I/O variables
      character*255 chtmp
      logical :: dir_e
      logical dir_exists

#ifndef SINGLE_POINT_MODEL
#define INLAND_SUBDIR "offline/grid"
#define INLAND_INFILE "conf/inland-grid.infile"
#else
#define INLAND_SUBDIR "offline/single_point"
#define INLAND_INFILE "conf/inland-single_point.infile"
#endif

! Default path definitions, can be overriden by user at runtime
! FIXME is this ok in GNU only?
! INLAND_INDIR=data
! INLAND_INFILE=$INLAND_INDIR/<INLAND_INFILE>
! INLAND_DATADIR=./input or $INLAND_INDIR/$INLAND_SUBDIR/data
! INLAND_OUTDIR=.

      indir = './data/'//INLAND_SUBDIR
      datadir = './input'
      call getenv("INLAND_INDIR", chtmp)
      if ( trim(chtmp) .ne. "" ) then
         indir = trim(chtmp)//'/'//INLAND_SUBDIR
         datadir = trim(indir)//'/data'
      end if
!      inquire( file=trim(indir)//'/', exist=dir_e )
      dir_e = dir_exists(indir)
      if ( .not. dir_e ) then
         write (*,*) ''
         write (*,*) 'ERROR: input directory '//trim(indir)//' does not exist!'
         write (*,*) 'make sure INLAND_INDIR is set to proper path'
         stop 1
      end if

      infile = trim(indir)//'/'//INLAND_INFILE
      call getenv("INLAND_INFILE", chtmp)
      if ( trim(chtmp) .ne. "" ) then
         infile = trim(chtmp)
      end if
      inquire( file=trim(infile), exist=dir_e )
      if ( .not. dir_e ) then
         write (*,*) ''
         write (*,*) 'ERROR: input file '//trim(infile)//' does not exist!'
         write (*,*) 'make sure INLAND_INDIR or INLAND_INFILE is set to proper path (or not set)'
         stop 1
      end if

      call getenv("INLAND_DATADIR", chtmp)
      if ( trim(chtmp) .ne. "" ) then
         datadir = chtmp
      end if
!      inquire( file=trim(datadir)//"/", exist=dir_e )
      dir_e = dir_exists(datadir)
!      inquire( directory=trim(datadir), exist=dir_e )
      if ( .not. dir_e ) then
         write (*,*) ''
         write (*,*) 'ERROR: data directory '//trim(datadir)//' does not exist!'
         write (*,*) 'make sure INLAND_DATADIR is set to proper path'
         stop 1
      end if

      outdir = 'output'
      call getenv("INLAND_OUTDIR", chtmp)
      if ( trim(chtmp) .ne. "" ) then
         outdir = chtmp
      end if
!      inquire( file=trim(outdir)//"/", exist=dir_e )
      dir_e = dir_exists(outdir)
      if ( .not. dir_e ) then
         call system( 'mkdir -p '//trim(outdir) )
      end if
!      inquire( file=trim(outdir)//"/", exist=dir_e )
      dir_e = dir_exists(outdir)
      if ( .not. dir_e ) then
         write (*,*) ''
         write (*,*) 'ERROR: output directory '//trim(outdir)//' does not exist!'
         write (*,*) 'make sure INLAND_OUTDIR is set to proper path and directory exists'
         stop 1
      end if

! ran2val - set value from INLAND_RANDOMVAL if exists
      ran2val = 0.
      call getenv("INLAND_RANDOMVAL", chtmp)
      if ( trim(chtmp) .ne. "" ) then
         read (chtmp,*) ran2val
      end if

end subroutine rd_env
!*******************************************************************************
