#include "inland_config.h"
!----------------------------------------------------------
subroutine rd_param
!----------------------------------------------------------
!  Read various parameters from ibis.params
!
!---------------------------Code history--------------------------------
! Author: David Price
! 
!----------------------------------------------------------

#ifdef SINGLE_POINT_MODEL
      use inland_comforc ! use everything from comforc.
#endif /* SINGLE_POINT_MODEL */

      use inland_ibispar, only: npoi, npft, nband, ndat, npftu, nsoilay,       &
                                stemae, rootae, rrootpy, rwoodpy, tempvm_nec,  &
                                rgrowth, dispuhf, ialoglhf, ialoguhf, avmuir,  &
                                kfactor, stressfac, indir, datadir
!      use ibiscst
!      use comage    ! npftu
      use inland_comveg, only: clitrl, clitrm, clitrs, clitll, clitlm, clitls, &
                               clitwl, clitwm, clitws, csoislop, csoislon,     &
                               csoipas, alaiml, alaimu, cgrass, chl, chs, chu, &
                               cleaf, cstem, tblowl, tblows, tblowu, tdripl,   &
                               tdrips, tdripu, wliqlmax, wliqsmax, wliqumax,   &
                               woodnorm, wsnolmax, wsnosmax, wsnoumax, specla, &
                               rhoveg, tauveg, dleaf, dstem, aleaf, aroot,     &
                               awood, optis_params

      use inland_comsoi    ! wpudmax
      use inland_compft    ! PFT parameters incl physiological "constants"
      use inland_comtex    ! Soil texture-related parameters
      use inland_combgc    ! Soil biogeochemistry parameters
#ifdef SINGLE_POINT_MODE
      use inland_comveg,   only: fu,fl
#endif /* SINGLE_POINT_MODEL */

      implicit none
  
! Local variables
#ifdef SINGLE_POINT_MODEL
      ! Thiago Veloso (Sep-2012)
      ! Variable below is related to the single-point version
      ! It represents the dimension (number of lines) of the 
      ! input data. It is used later, in subroutine readforc.

!     integer dimforc    ! dimension (number of lines) of input data
#endif /* SINGLE_POINT_MODEL */

      integer*4 parm_unit,& ! file unit assignment for input
                j,     & ! PFT number (in range 1 to npftu)
                npft2, & ! number of PFTs reported in params.veg
                npftu2,& ! number of upper canopy PFTs reported in params.veg
                nsoil2,& ! number of soil texture classes reported in params.soi
                nveg,&   ! number of vegetation classes reported in params.veg
                optis_i  ! index in optis_params array

      real*8    dummyvar ! use this to read in integers that might be pretending
                         ! to be reals. Also use it as a filler when reading 
                         ! non-existent variables in subroutine ReadItems

      character*128 parm_file
      
      parameter (parm_unit = 9)

#ifdef SINGLE_POINT_MODEL
! Thiago on Set/12: Code below loops over all lines of the input file to
!                   retrieve its dimension, which is saved to variable 
!                   'dimforc'. This procedure avoids both the existence 
!                   of variable 'nbforc' and the declaration of variable 
!                   'dimforc' as a parameter.
       dimforc = 0
       open (unit=20,status='unknown',file=trim(datadir)//'/clim-input')
       do
          read (20,*,end=10)
          dimforc = dimforc + 1
       end do
10     close (20)
       if ( dimforc .eq. 0 ) then
          write (*,*) 'ERROR: could not read clim input file '//trim(datadir)//'/clim-input'
          stop 'make sure INLAND_INDIR is set to proper path'
          stop 1
       end if

#endif /* SINGLE_POINT_MODEL */

! ******************************************************************************
! open the parameter file 'params.can' for input; read in canopy parameters...
      parm_file = trim(indir)//'/'//'params/canopy'
      open(UNIT=parm_unit, FILE=parm_file, STATUS='OLD', ERR=9001)

      call readitem(parm_unit, parm_file, tau15)
      call readitem(parm_unit, parm_file, kc15)
      call readitem(parm_unit, parm_file, ko15)
      call readitem(parm_unit, parm_file, cimax)
      call readitem(parm_unit, parm_file, dummyvar)

      npft2 = nint(dummyvar)
      if (npft2 .ne. npft) then
         write (*, 9003) parm_file, npft2, npft
         goto 9006 ! In the circumstances this seems the best thing to do! 
      end if
9003  format ('rd_param Warning: Number of PFTs in ', A10, ' is: ',  &
              I2, ' number in compar.h is: ', I2)  

      call readitem(parm_unit, parm_file, dummyvar)

      npftu2 = nint(dummyvar)
      if (npftu2 .ne. npftu) then
         write (*,9004) parm_file, npftu2, npftu
         goto 9006 ! In the circumstances this seems the best thing to do! 
      end if
9004  format ('rd_param Warning: Number of upper canopy (tree) PFTs ',  &
              'in ', A10, ' is: ', I2, ' number in comage.h is: ', I2)  

! Standard photosynthesis parameters for C3 and C4 physiological pathways
      call readitem(parm_unit, parm_file, alpha3)
      call readitem(parm_unit, parm_file, theta3)
      call readitem(parm_unit, parm_file, beta3)
      call readitem(parm_unit, parm_file, alpha4)
      call readitem(parm_unit, parm_file, theta4)
      call readitem(parm_unit, parm_file, beta4)

! Physiological parameters for broadleaf trees
      call readitems(parm_unit, parm_file, 4, gammaub, coefmub, coefbub, &
                     gsubmin, dummyvar, dummyvar, dummyvar, dummyvar,    &
                     dummyvar, dummyvar)

! Physiological parameters for conifer trees
      call readitems(parm_unit, parm_file, 4, gammauc, coefmuc, coefbuc, &
                     gsucmin, dummyvar, dummyvar, dummyvar, dummyvar,    &
                     dummyvar, dummyvar)

! Physiological parameters for shrubs
      call readitems(parm_unit, parm_file, 4, gammals, coefmls, coefbls,       &
                     gslsmin, dummyvar,dummyvar, dummyvar, dummyvar, dummyvar, &
                     dummyvar)

! Physiological parameters for C4 grasses
      call readitems(parm_unit, parm_file, 4, gammal4, coefml4, coefbl4,       &
                     gsl4min, dummyvar,dummyvar, dummyvar, dummyvar, dummyvar, &
                     dummyvar)

! Physiological parameters for C3 grasses
      call readitems(parm_unit, parm_file, 4, gammal3, coefml3, coefbl3,       &
                     gsl3min, dummyvar,dummyvar, dummyvar, dummyvar, dummyvar, &
                     dummyvar)

!!! DTP 2001/06/05: Note that I have included a new variable here: vmax_pft
!                   This reads in values of vmax assigned initially to each
!                   PFT. These values are then transferred to vmaxub, vmaxuc,
!                   vmaxls, vmaxl4, vmaxl3, in the modified PHYSIOLOGY.F 
!                   module (subroutine STOMATA).

      do j = 1, npft 
         call readitems(parm_unit, parm_file, 8, vmax_pft(j), specla(j), &
                        tauleaf(j), tauroot(j), tauwood0(j), aleaf(j),   &
                        aroot(j), awood(j), dummyvar, dummyvar) 
      end do

      call readitem(parm_unit, parm_file, woodnorm)

      do j = 1, nband
         call readitems(parm_unit, parm_file, 2, rhoveg(j,1), rhoveg(j,2), &
                        dummyvar, dummyvar, dummyvar, dummyvar, dummyvar,  &
                        dummyvar, dummyvar, dummyvar)  
      end do

      do j = 1, nband
         call readitems(parm_unit, parm_file, 2, tauveg(j,1), tauveg(j,2), &
                        dummyvar, dummyvar, dummyvar, dummyvar, dummyvar,  &
                        dummyvar, dummyvar, dummyvar)   
      end do

      call readitem(parm_unit, parm_file, chifuz)
      call readitem(parm_unit, parm_file, chiflz)

      call readitems(parm_unit, parm_file, 2, dleaf(1), dleaf(2), dummyvar,    &
                     dummyvar, dummyvar, dummyvar,dummyvar, dummyvar, dummyvar,&
                     dummyvar)

      call readitems(parm_unit, parm_file, 2, dstem(1), dstem(2), dummyvar,    &
                     dummyvar, dummyvar, dummyvar,dummyvar, dummyvar, dummyvar,&
                     dummyvar)

      call readitem(parm_unit, parm_file, alaimu)
      call readitem(parm_unit, parm_file, alaiml)

      call readitem(parm_unit, parm_file, cleaf)
      call readitem(parm_unit, parm_file, cstem)
      call readitem(parm_unit, parm_file, cgrass)

      call readitem(parm_unit, parm_file, chs)
      call readitem(parm_unit, parm_file, chu)
      call readitem(parm_unit, parm_file, chl)

      call readitem(parm_unit, parm_file, wliqumax)
      call readitem(parm_unit, parm_file, wliqsmax)
      call readitem(parm_unit, parm_file, wliqlmax)

      call readitem(parm_unit, parm_file, wsnoumax)
      call readitem(parm_unit, parm_file, wsnosmax)
      call readitem(parm_unit, parm_file, wsnolmax)

      call readitem(parm_unit, parm_file, tdripu)
      call readitem(parm_unit, parm_file, tdrips)
      call readitem(parm_unit, parm_file, tdripl)

      call readitem(parm_unit, parm_file, tblowu)
      call readitem(parm_unit, parm_file, tblows)
      call readitem(parm_unit, parm_file, tblowl)

! Canopy parameters identified on Optis' efforts - fzm
      call readitem(parm_unit, parm_file, stemae)
      call readitem(parm_unit, parm_file, rootae)
      call readitem(parm_unit, parm_file, rrootpy)
      call readitem(parm_unit, parm_file, rwoodpy)
      call readitem(parm_unit, parm_file, tempvm_nec)
      call readitem(parm_unit, parm_file, rgrowth)
      call readitem(parm_unit, parm_file, dispuhf)
      call readitem(parm_unit, parm_file, ialoglhf)
      call readitem(parm_unit, parm_file, ialoguhf)
      call readitem(parm_unit, parm_file, avmuir)
#ifdef SINGLE_POINT_MODE
      call readitem(parm_unit, parm_file, fu)
      call readitem(parm_unit, parm_file, fl)
#endif /* SINGLE_POINT_MODEL */
      close (parm_unit)

! ******************************************************************************
! open the parameter file 'params.veg' for input; read in vegetation PFT
! parameters...
      parm_file = trim(indir)//'/'//'params/vegetation'
      open(UNIT=parm_unit, FILE=parm_file, STATUS='OLD', ERR=9001)

      do j = 1, npft
         call readitems(parm_unit, parm_file, 4, TminL(j), TminU(j), Twarm(j), &
                        GDD(j),dummyvar, dummyvar, dummyvar, dummyvar,         &
                        dummyvar, dummyvar)
      end do 

      call readitem(parm_unit, parm_file, dummyvar)
      nveg = nint(dummyvar)

!   There's no better place to allocate plai_init than here, as here we know its
! dimension size and we use it right away.
      allocate(plai_init(4,nveg))
      !plai_init(:,:) = 0. ! do we really need to zero it here?

      do j = 1, nveg
         call readitems(parm_unit, parm_file, 4, plai_init(1,j),          &
                        plai_init(2,j), plai_init(3,j), plai_init(4,j),   &
                        dummyvar, dummyvar, dummyvar, dummyvar, dummyvar, &
                        dummyvar)
      end do

      call readitem(parm_unit, parm_file, plaiupper)
      call readitem(parm_unit, parm_file, plailower)
      call readitem(parm_unit, parm_file, xminlai)
      call readitem(parm_unit, parm_file, sapfrac_init)
      call readitem(parm_unit, parm_file, beta1)
      call readitem(parm_unit, parm_file, beta2)

! Parameters identified in Optis' efforts - fzm and et
! Read the values into optis_params, and propagate them in ibisalloc, 
! because for now we don't know npoi
      do optis_i = 1, 12
         call readitem(parm_unit, parm_file, optis_params(optis_i))
      end do
! TODO - what happened to csoislo?

      close (parm_unit)
      
! ******************************************************************************
! open the parameter file 'params.soi' for input; read in soil parameters...
      parm_file = trim(indir)//'/'//'params/soil'
      open(UNIT=parm_unit, FILE=parm_file, STATUS='OLD', ERR=9001)

      do j = 1, nsoilay
         call readitem(parm_unit, parm_file, hsoi(j))
      end do

      call readitem(parm_unit, parm_file, bperm)
      call readitem(parm_unit, parm_file, wpudmax)
      call readitem(parm_unit, parm_file, zwpmax)

      call readitem(parm_unit, parm_file, dummyvar)

      nsoil2 = nint(dummyvar)

      if (nsoil2 .ne. ndat) then ! FIXME: Is this needed????
         write (*, 9031) parm_file, nsoil2
         write (*, 9032) ndat
         goto 9006 ! In the circumstances this seems the best thing to do! 
      end if

9031  format (' rd_param Warning: Number of soil types in ', A10, ' is: ', I2)
9032  format (' Number of soil types in comtex.h is: ', I2)  

      do j = 1, ndat
         call readitems(parm_unit, parm_file, 9, texdat(1,j), texdat(2,j),   &
                        texdat(3,j), porosdat(j), sfielddat(j), swiltdat(j), &
                        bexdat(j), suctiondat(j), hydrauldat(j), dummyvar)
      end do

      call readitem(parm_unit, parm_file, lig_frac)
      call readitem(parm_unit, parm_file, fbsom)
      call readitem(parm_unit, parm_file, effac)

      call readitems(parm_unit, parm_file, 8, cnr(1), cnr(2), cnr(3), cnr(4), &
                     cnr(5), cnr(6), cnr(7), cnr(8), dummyvar, dummyvar)

      call readitems(parm_unit, parm_file, 5, fmax, rconst, cnleaf, cnroot, &
                     cnwood, dummyvar, dummyvar, dummyvar, dummyvar, dummyvar)

      call readitems(parm_unit, parm_file, 9, klm, kls, kll, krm, krs, krl, &
                     kwm, kws, kwl, dummyvar)

      call readitems(parm_unit, parm_file, 7, kbn, kbp, knb, kns, kpb, kps, &
                     ksb, dummyvar, dummyvar, dummyvar)

      call readitems(parm_unit, parm_file, 9, ylm, yrm, ywm, yls, yrs, yws, &
                     yll, yrl, ywl, dummyvar)

      call readitems(parm_unit, parm_file, 7, ybn, ybp, yps, yns, ysb, ypb, &
                     ynb, dummyvar, dummyvar, dummyvar)

! Parameters identified in Optis' efforts
      call readitem(parm_unit, parm_file, stressfac)

! We read wsoi value and add it to optis_params
      call readitem(parm_unit, parm_file, optis_params(optis_i))

      call readitem(parm_unit, parm_file, kfactor)

      close (parm_unit)
! ******************************************************************************
! open the parameter file 'params.hyd' for input; read in vegetation PFT parameters...
!      parm_file = 'params.hyd' ! What is this file supposed to contain???
!      open(UNIT=parm_unit, FILE=parm_file, STATUS='OLD', ERR=9001)
!      close (parm_unit) 
! ******************************************************************************

      write (*,*)'rd_param: All data read in from parameter files successfully.'
      return ! subroutine rd_param

! Error handling targets
9001  write (*,2001) TRIM(parm_file), parm_unit 
2001  format ('rd_param: Error opening parameter file "', A,'" on unit ', I2)      
      stop

9006  write (*,2006) TRIM(parm_file), parm_unit
2006  format ('rd_param: Data inconsistency in parameter file "',A, &
              '" on unit ', I2) 
      close (parm_unit)
      stop
end subroutine rd_param
!*******************************************************************************
