#include "inland_config.h"
! ---------------------------------------------------------------------
subroutine vdiag
! ---------------------------------------------------------------------
      use inland_ibispar
      use inland_ibisctl, only: iyear, iyear0
      use inland_comveg
      use inland_comsum

      implicit none
!------------------------------Variables--------------------------------
! local variables
      integer i, k

      real*8 vtarea(15),   & ! total area of the vegetation type (m**2)
             vnee(15),     & ! vegetation type average nee (kg-c/m**2/yr)
             vnpp(15),     & ! vegetation type average npp (kg-c/m**2/yr)
             vgpp(15),     & ! vegetation type average gpp (kg-c/m**2/yr)
             vleach(15),   & ! vegetation type average leaching (kg-c/m**2/yr)
             vco2mic(15),  & ! vegetation type average micr. respiration (kg-c/m**2/yr)
             vcdist(15),   & ! vegetation type average disturbance (kg-c/m**2/yr)
             vtotfall(15), & ! vegetation type average litterfall (kg-c/m**2/yr)
             vbiomass(15), & ! vegetation type average biomass (kg-c/m**2)
             vlai(15),     & ! vegetation type average lai (m**2/m**2)
             vsoic(15),    & ! vegetation type average soil carbon (kg-c/m**2)
             vrunoff(15)     ! vegetation type average runoff (mm/yr)

#ifdef SPMD
      real*8 vtall1(15,12), vtall2(15,12)
#endif

! initialize variables
      do 100 k = 1, 15
         vnee(k)     = 0.0
         vtarea(k)   = 0.0
         vnpp(k)     = 0.0
         vgpp(k)     = 0.0
         vbiomass(k) = 0.0
         vlai(k)     = 0.0
         vsoic(k)    = 0.0
         vrunoff(k)  = 0.0
         vleach(k)   = 0.0
         vco2mic(k)  = 0.0
         vcdist(k)   = 0.0
         vtotfall(k) = 0.0
100   continue

! sum ecosystem properties over each vegetation type
      do 200 i = lbeg, lend
         k = int (max (dble(1.0), min (dble(15.0), vegtype0(i))))
         vtarea(k) = vtarea(k)    +garea(i)
         vnee(k)     = vnee(k)    +garea(i)*ayneetot(i)
         vnpp(k)     = vnpp(k)    +garea(i)*aynpptot(i)
         vgpp(k)     = vgpp(k)    +garea(i)*aygpptot(i)
         vbiomass(k) = vbiomass(k)+garea(i)*totbiou(i)+garea(i)*totbiol(i)
         vlai(k)     = vlai(k)    +garea(i)*totlaiu(i)+garea(i)*totlail(i)
         vsoic(k)    = vsoic(k)   +garea(i)*aycsoi(i) 
         vrunoff(k)  = vrunoff(k) +garea(i)*aytrunoff(i)
         vleach(k)   = vleach(k)  +garea(i)*yrleach(i)
         vco2mic(k)  = vco2mic(k) +garea(i)*ayco2mic(i)
         vcdist(k)   = vcdist(k)  +garea(i)*cdisturb(i)
         vtotfall(k) = vtotfall(k)+garea(i)*(falll(i)+fallr(i)+fallw(i))
200   continue

#ifdef SPMD
! sum over parallel tasks
      vtall1(:,1) = vtarea(:)
      vtall1(:,2) = vnee(:)
      vtall1(:,3) = vnpp(:)
      vtall1(:,4) = vgpp(:)
      vtall1(:,5) = vbiomass(:)
      vtall1(:,6) = vlai(:)
      vtall1(:,7) = vsoic(:)
      vtall1(:,8) = vrunoff(:)
      vtall1(:,9) = vleach(:)
      vtall1(:,10) = vco2mic(:)
      vtall1(:,11) = vcdist(:)
      vtall1(:,12) = vtotfall(:)

      call MPI_ALLREDUCE(vtall1, vtall2, 180, MPI_REAL8, MPI_SUM, mpicomw, iermpi)

      vtarea(:) = vtall2(:,1)
      vnee(:) = vtall2(:,2)
      vnpp(:) = vtall2(:,3)
      vgpp(:) = vtall2(:,4)
      vbiomass(:) = vtall2(:,5)
      vlai(:) = vtall2(:,6)
      vsoic(:) = vtall2(:,7)
      vrunoff(:) = vtall2(:,8)
      vleach(:) = vtall2(:,9)
      vco2mic(:) = vtall2(:,10)
      vcdist(:) = vtall2(:,11)
      vtotfall(:) = vtall2(:,12)
#endif

!
! calculate area averages
      do 300 k = 1, 15
         vnee(k)     = vnee(k)     / max (dble(1.0), vtarea(k))
         vnpp(k)     = vnpp(k)     / max (dble(1.0), vtarea(k))
         vgpp(k)     = vgpp(k)     / max (dble(1.0), vtarea(k))
         vbiomass(k) = vbiomass(k) / max (dble(1.0), vtarea(k))
         vlai(k)     = vlai(k)     / max (dble(1.0), vtarea(k))
         vsoic(k)    = vsoic(k)    / max (dble(1.0), vtarea(k))
         vrunoff(k)  = vrunoff(k)  / max (dble(1.0), vtarea(k))
         vleach(k)   = vleach(k)   / max (dble(1.0), vtarea(k))
         vco2mic(k)  = vco2mic(k)  / max (dble(1.0), vtarea(k))
         vcdist(k)   = vcdist(k)   / max (dble(1.0), vtarea(k))
         vtotfall(k) = vtotfall(k) / max (dble(1.0), vtarea(k))
300   continue

! write some diagnostic output to history file (file opened in ibisini.F)
      if (myid .eq. 0) then
         do 400 k = 1, 15
         write (202,9000) iyear, k, vtarea(k)/1.0e+06, vnee(k), vnpp(k), &
                          vgpp(k), vleach(k), vco2mic(k), vcdist(k),     &
                          vtotfall(k), vbiomass(k), vlai(k), vsoic(k),   &
                          vrunoff(k)
400      continue
!        call flush (202)
      endif
!
9000  format (1x,i4,5x,i2,5x,1e10.3,11f10.3)
!
! return to main program
!
      return
end subroutine vdiag
