#include "inland_config.h"

#ifdef SINGLE_POINT_MODEL
#error "This subroutine should NOT be compiled for 0D IBIS model option."
#endif

! ---------------------------------------------------------------------
subroutine wdaily (jday,nday)
! ---------------------------------------------------------------------
! writes out daily files
!---------------------------------------------------------------
      use inland_ibispar
      use inland_ibisctl, only: iyear, iyear0
      use inland_comatm
      use inland_comwork
      use inland_comsum
#if (defined CAMIBIS)
      use inland_abortutils, only: endrun
#endif

      implicit none
!-----------------------------------------------------------------------
! input-output variables
      integer jday,    & ! julian day of the simulation
              nday,    & ! number of days since the start of the simulation
              iyearprev  ! iyear - 1.

! local variables
! parameters used for output
      integer mstep,  & ! this time step for netcdf file
              idies,  & !
              istat
      integer istart(4), icount(4) ! for writing vars
      character*8 tmpdate      ! temp. variable to hold date
      character*8 cdate        ! date to use in history attribute in files
      character*10 tdate       ! character date for time step
      character*21 tunits      ! units for time
      character*80 dimnames(4) ! names of dimensions for vars
      character*1024 filen     ! file name
      character*4 chyear

      real*8 ftime, &          ! real form of jday
             tweight           ! number of days in daily average = 1.

      data istart / 1,1,1,1 /, &
           icount / nlon,nlat,1,1 /
      icount(1) = nlonsub
      icount(2) = nlatsub

      iyearprev = iyear - 1

! ---------------------------------------------------------------------
!     current time value, step, time weight
      ftime = nday
      mstep = jday
#ifdef COUPLED
      if (iyear .eq. iyear0 .or. iyear .ne. iyearprev) mstep = 1
#endif /* COUPLED */
      tweight = 1.

      chyear = '0000'
      if (iyear .ge. 1000) then
         write(chyear(1:4),'(i4)') iyear
      elseif (iyear .lt. 10) then
         write(chyear(4:4),'(i1)') iyear
      elseif (iyear .lt. 100) then
         write(chyear(3:4),'(i2)') iyear
      else
         write(chyear(2:4),'(i3)') iyear
      endif

! tdate is julian day (3 char) followed by iyear (4 char)
      tdate='0000000'//char(0)//char(0)//char(0)
      if (jday .lt. 10) then
         write(tdate(3:3),'(i1)') jday
      elseif (jday .lt. 100) then
         write(tdate(2:3),'(i2)') jday
      else
         write(tdate(1:3),'(i3)') jday
      endif
      if (iyear .ge. 1000) then
         write(tdate(4:7),'(i4)') iyear
      elseif (iyear .lt. 10) then
         write(tdate(7:7),'(i1)') iyear
      elseif (iyear .lt. 100) then
         write(tdate(6:7),'(i2)') iyear
      else
         write(tdate(5:7),'(i3)') iyear
      endif

! first time only
      if (mstep.eq.1) then

! initialize snow layer indices, pft names, etc
         call date_and_time(tmpdate)
         cdate(1:2) = tmpdate(5:6)
         cdate(3:3) = '/'
         cdate(4:5) = tmpdate(7:8)
         cdate(6:6) = '/'
         cdate(7:8) = tmpdate(3:4)

! time units is days since Dec 31 of the year before iyear0
         tunits = 'days since 0000-12-31'
         write(tunits(12:15),'(i4)') iyear0-1
         dimnames(1) = 'longitude'
         dimnames(2) = 'latitude'
!        dimnames(3) is set for each variable seperately
!        dimnames(4) not used in this subr for now, but define for future use
         dimnames(4) = 'time'
      endif
 
! dummy variable example, 3-d - copy & modify for new variable.
!
!      filen = 'output/daily/dummyv.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,'daily average dummyv',  &
!                     'ibis wdaily',cdate,nlon,lonscale,nlat,latscale,'none','none', &
!                      \'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'dummyv','average dummyv','dummyvs-units',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      endif
!      call vec2arr(addummyv, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'dummyv',istart,icount,cdummy,ftime,tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wdaily, dummyv'
!         call endrun
!      endif
!

!
! Recording variables in a single file.  
!
      if (myid .eq. 0) then
         filen = trim(outdir)//'/inland-daily-'//chyear//'.nc'
         if (mstep .eq. 1) then
            call inifile(idies,filen,'daily average','ibis wdaily',cdate, &
                         nlonsub,lonscale,nlatsub,latscale,'none','none', &
                         'none','E',1,0.,'none',tunits,'gregorian',istat)
            dimnames(3) = 'time'
            call inivar(idies,'rain','average rainfall','mm/day',3,dimnames, &
                        ocean,istat)
            call inivar(idies,'snow','average snowfall','mm/day',3,dimnames, &
                        ocean,istat)
!            call inivar(idies,'aet','average aet','mm/day',3,dimnames,ocean, &
!                        istat)
!            call inivar(idies,'trunoff','average total runoff','mm/day',3,   &
!                        dimnames,ocean,istat)
            call inivar(idies,'srunoff','average surface runoff','mm/day',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'drainage','average drainage','mm/day',3,      &
                        dimnames,ocean,istat)
            call inivar(idies,'tnpptot','average npp','fraction',3,dimnames, &
                        ocean,istat)
!            call inivar(idies,'wisoi','average soil ice','fraction',3,       &
!                        dimnames,ocean,istat)            
!            call inivar(idies,'snod','average snow depth','meters',3,        &
!                        dimnames,ocean,istat)
!            call inivar(idies,'snof','average snow fraction','fraction',3,   &
!                         dimnames,ocean,istat)
            call inivar(idies,'co2ratio','average co2 ratio','fraction',3,   &
                        dimnames,ocean,istat)
            call inivar(idies,'co2mic','soil microbe carbon flux','kg/m^2',3,&
                        dimnames,ocean,istat)
            call endini(idies,istat)
         end if
      endif

! rainfall
!      if (myid .eq. 0) then
!         filen = 'output/daily/rain-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen,'daily average rainfall','ibis wdaily', &
!                         cdate,nlonsub,lonscale,nlatsub,latscale,'none','none',    &
!                         'none',1,0.,'none',tunits,'gregorian',istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'rain','average rainfall','mm/day',3,dimnames, &
!                        ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(adrain, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'rain',istart,icount,cdummy,ftime,tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wdaily, rain'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif

! snowfall
!      if (myid .eq. 0) then
!         filen = 'output/daily/snow-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen,'daily average snowfall','ibis wdaily', &
!                         cdate,nlonsub,lonscale,nlatsub,latscale,'none','none',    &
!                         'none',1,0.,'none',tunits,'gregorian',istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'snow','average snowfall','mm/day',3,dimnames, &
!                        ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(adsnow, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'snow',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wdaily, snow'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif

! aet
!      filen = 'output/daily/aet-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,'daily average aet','ibis wdaily',cdate,nlon,lonscale, &
!                      nlat,latscale,'none','none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'aet','average aet','mm/day',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      endif
!      call vec2arr(adaet, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'aet',istart,icount,cdummy,ftime,tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wdaily, aet'
!         call endrun
!      endif
!
! trunoff
!      filen = 'output/daily/trunoff-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,'daily average total runoff','ibis wdaily',cdate,nlonsub,lonscale, &
!                       nlat,latscale,'none','none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'trunoff','average total runoff','mm/day',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      endif
!      call vec2arr(adtrunoff, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'trunoff',istart,icount,cdummy,ftime,tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wdaily, trunoff'
!         call endrun
!      endif
!
! srunoff
!      if (myid .eq. 0) then
!         filen = 'output/daily/srunoff-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen,'daily average surface runoff',          &
!                         'ibis wdaily',cdate,nlonsub,lonscale,nlatsub,latscale,     &
!                         'none','none','none',1,0.,'none',tunits,'gregorian', &
!                         istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'srunoff','average surface runoff','mm/day',3,  &
!                        dimnames,ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(adsrunoff, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'srunoff',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wdaily, srunoff'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif

! drainage
!      if (myid .eq. 0) then
!         filen = 'output/daily/drainage-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen,'daily average drainage','ibis wdaily', &
!                         cdate,nlonsub,lonscale,nlatsub,latscale,'none','none',    &
!                         'none',1,0.,'none',tunits,'gregorian',istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'drainage','average drainage','mm/day',3, &
!                        dimnames,ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(addrainage, cdummy)
      if (myid .eq. 0) then
        istart(3) = mstep
        icount(3) = 1
        call writevar(filen,'drainage',istart,icount,cdummy,ftime,tweight,tdate,istat)
        if (istat .ne. 0) then
           write(*,*) 'ERROR in wdaily, drainage'
#ifdef CAMIBIS
           call endrun
#else /* CAMIBIS */
           stop
#endif /* CAMIBIS */
        endif
      endif

! tnpptot
!      if (myid .eq. 0) then
!         filen = 'output/daily/tnpptot-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen,'daily average total npp','ibis wdaily', &
!                         cdate,nlonsub,lonscale,nlatsub,latscale,'none','none',     &
!                         'none',1,0.,'none',tunits,'gregorian',istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'tnpptot','average npp','fraction',3,dimnames,  &
!                        ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(adtnpptot, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'tnpptot',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wdaily, wsoi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif

! wisoi
!      filen = 'output/daily/wisoi-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,'daily average soil ice','ibis wdaily',cdate,nlon,lonscale, &
!                      nlat,latscale,'none','none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'wisoi','average soil ice','fraction',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      endif
!      call vec2arr(adwisoi, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'wisoi',istart,icount,cdummy,ftime,tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wdaily, wisoi'
!         call endrun
!      endif
!
! snod
!      filen = 'output/daily/snod-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,'daily average snow depth','ibis wdaily',cdate,nlon,lonscale, &
!                      nlat,latscale,'none','none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'snod','average snow depth','meters',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      endif
!      call vec2arr(adsnod, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'snod',istart,icount,cdummy,ftime,tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wdaily, snod'
!         call endrun
!      endif
!
! snof
!      filen = 'output/daily/snof-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,'daily average snow fraction','ibis wdaily',cdate,nlon,lonscale, &
!                      nlat,latscale,'none','none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'snof','average snow fraction','fraction',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      endif
!      call vec2arr(adsnof, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'snof',istart,icount,cdummy,ftime,tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wdaily, snof'
!         call endrun
!      endif
!

! co2ratio
!      if (myid .eq. 0) then
!         filen = 'output/daily/co2ratio-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen,                                          &
!                         'daily average ratio of root to total soil co2 flux', &
!                         'ibis wdaily',cdate,nlonsub,lonscale,nlatsub,latscale,      &
!                         'none','none','none',1,0.,'none',tunits,'gregorian',  &
!                         istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'co2ratio','average co2 ratio','fraction',3,     &
!                        dimnames,ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(adco2ratio, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'co2ratio',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wdaily, co2ratio'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif

! co2mic
!      if (myid .eq. 0) then
!         filen = 'output/daily/co2mic-'//chyear//'.nc'
!         if (mstep .eq. 1) then
!            call inifile(idies,filen, 'daily flux of carbon due to soil '//    &
!                         'microbe co2 flux','ibis wdaily',cdate,nlonsub,lonscale, &
!                         nlatsub,latscale,'none','none','none',1,0.,'none',       &
!                         tunits,'gregorian',istat)
!            dimnames(3) = 'time'
!            call inivar(idies,'co2mic','soil microbe carbon flux','kg/m^2',3,  &
!                        dimnames,ocean,istat)
!            call endini(idies,istat)
!         endif
!      endif
!
      call vec2arr(adco2mic, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'co2mic',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wdaily, co2mic'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif
      return
end subroutine wdaily
