#include "inland_config.h"

#ifdef SINGLE_POINT_MODEL
#error "This subroutine should NOT be compiled for 0D IBIS model option."
#endif

! ---------------------------------------------------------------------
subroutine wmonthly(nday)
! ---------------------------------------------------------------------
! writes out monthly files
!---------------------------------------------------------------
      use inland_ibispar
      use inland_ibisctl, only: imonth, iyear, iyear0
      use inland_comsoi
      use inland_comwork
      use inland_comsum
#if (defined CAMIBIS)
      use inland_abortutils, only: endrun
#endif

      implicit none

!----------------------------------------------------------------------
!     input variables 
      integer nday      ! number of days run since iyear0

!-----------------------------------------------------------------------
!     local variables
      integer mstep,  & ! time step in netcdf file
              n, idies, istat, i, zero

      integer istart(4), icount(4) ! for writing vars

      character*8 tmpdate      ! temp. variable to hold date
      character*8 cdate        ! date to use in history attribute in files
      character*10 tdate       ! character date for time step
      character*21 tunits      ! units for time
      character*80 dimnames(4) ! names of dimensions for vars
      character*1024 filen       ! file name
      character*3  chmon(12)   ! month abbreviations
      character*4 chyear

      real*8 depthsoi(nsoilay), & ! soil layer depths
             ftime,             & ! real form of nday
             tweight              ! number of days in monthly average

      data chmon / 'JAN','FEB','MAR','APR','MAY','JUN', &
                   'JUL','AUG','SEP','OCT','NOV','DEC' /
      data istart / 1,1,1,1 /,    &
           icount / nlon,nlat,1,1 /

! FIXME: Better drop 'data' statement than make it and just change it!
      icount(1) = nlonsub
      icount(2) = nlatsub

      zero = 0 ! You tell me!!

! ---------------------------------------------------------------------
!     current time value, step, time weight

      ftime = nday
!     mstep = 12*(iyear-iyear0) + imonth
      mstep = imonth
!     mstep = 1
!      write(*,*) 'wmonthly, test: mstep = 1'
      tweight = ndaypm(imonth)

      if (iyear .lt. 10) then
         write(chyear(1:1),'(i1)') zero
         write(chyear(2:2),'(i1)') zero
         write(chyear(3:3),'(i1)') zero
         write(chyear(4:4),'(i1)') iyear
      elseif (iyear .lt. 100) then
         write(chyear(1:1),'(i1)') zero
         write(chyear(2:2),'(i1)') zero
         write(chyear(3:4),'(i2)') iyear
      elseif (iyear .lt. 1000) then  
         write(chyear(1:1),'(i1)') zero
         write(chyear(2:4),'(i3)') iyear
      else
         write(chyear,'(i4)') iyear
      endif   

! tdate is this month (3 char), followed by this year (4 char)
      tdate=chmon(imonth)//'0000'//char(0)//char(0)//char(0)
      if (iyear .ge. 1000) then
         write(tdate(4:7),'(i4)') iyear
      elseif (iyear .ge. 100) then
         write(tdate(5:7),'(i3)') iyear
      elseif (iyear .ge. 10) then
         write(tdate(6:7),'(i2)') iyear
      else
         write(tdate(7:7),'(i1)') iyear
      endif

! first time only
      if (mstep.eq.1) then

! initialize snow layer indices, pft names, etc
         call date_and_time(tmpdate)
         cdate(1:2) = tmpdate(5:6)
         cdate(3:3) = '/'
         cdate(4:5) = tmpdate(7:8)
         cdate(6:6) = '/'
         cdate(7:8) = tmpdate(3:4)

! time units is days since Dec 31 of the year before iyear0
         tunits = 'days since 0000-12-31'
         write(tunits(12:15),'(i4)') iyear0-1

         depthsoi(1) = hsoi(1)
         do 10 n = 2, nsoilay
            depthsoi(n) = depthsoi(n-1)+hsoi(n)
10       continue

         dimnames(1) = 'longitude'
         dimnames(2) = 'latitude'
!        dimnames(3) is set for each variable seperately
!        dimnames(4) not used in this subr for now, but define for future use
         dimnames(4) = 'time'
      end if

! dummy variable example, 3-d - copy & modify for new variable.
!
!      filen = 'output/monthly/dummyv.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average dummyv',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'dummyv','average dummyv',
!     >    'dummyvs-units',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr (amdummyv, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'dummyv',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, dummyv'
!         call endrun
!      end if
!
!
! evapotranspiration
!
      if (myid .eq. 0) then
         filen = trim(outdir)//'/inland-monthly-'//chyear//'.nc'
         if (mstep .eq. 1) then
            call inifile(idies,filen,'monthly average','ibis wmonthly',cdate, &
                         nlonsub,lonscale,nlatsub,latscale,'none','none',     &
                         'none','E',1,0.,'none',tunits,'gregorian',istat)
            dimnames(3) = 'time'
            call inivar(idies,'aet','average evapotranspiration','mm/day',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'transu','monthly upper canopy transpiration', &
                        'mm/day',3,dimnames,ocean,istat)
            call inivar(idies,'transl','monthly lower canopy transpiration', &
                        'mm/day',3,dimnames,ocean,istat)
            call inivar(idies,'suvap','monthly soil evaporation','mm/day',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'invap','monthly interception loss','mm/day',3,&
                        dimnames,ocean,istat)
            call inivar(idies,'trunoff','average total runoff','mm/day',3,   &
                        dimnames,ocean,istat)
            call inivar(idies,'srunoff','average surface runoff','mm/day',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'drainage','average drainage','mm/day',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'vwc','average volumetric water content',  &
                        'fraction',3,dimnames,ocean,istat)
            call inivar(idies,'awc','average plant available water content', &
                        'cm',3,dimnames,ocean,istat)
            call inivar(idies,'snod','average snow depth','meters',3,dimnames,&
                        ocean,istat)
            call inivar(idies,'snof','average snow fraction','m^2/m^3',3,    &
                        dimnames,ocean,istat)
            call inivar(idies,'albedo','average albedo',&
                        'fraction',3,dimnames,ocean,istat)
            call inivar(idies,'npptot','total npp of carbon', &
                        'kg m-2 month-1',3,dimnames,ocean,istat)
            call inivar(idies,'laiu','average lai upper canopy', &
                        'fraction',3,dimnames,ocean,istat)
            call inivar(idies,'lail','average lai lower canopy', &
                         'fraction',3,dimnames,ocean,istat)
            call endini(idies,istat)
         end if
      endif

      call vec2arr(amaet, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'aet',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, aet'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr (amtransu, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'transu',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
        if (istat .ne. 0) then
           write(*,*) 'ERROR in wmonthly, transu'
#ifdef CAMIBIS
           call endrun
#else /* CAMIBIS */
           stop
#endif /* CAMIBIS */
        end if
      endif
      
      call vec2arr (amtransl, cdummy)
      if (myid .eq. 0) then
        istart(3) = mstep
        icount(3) = 1
        call writevar(filen,'transl',istart,icount,cdummy,ftime, &
                      tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, transl'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr (amsuvap, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'suvap',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, suvap'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
 
      call vec2arr (aminvap, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'invap',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, invap'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! trunoff, srunoff, drainage
!
!      filen = 'runoff-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average total runoff',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'trunoff','average total runoff',
!     >    'mm/day',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'srunoff','average surface runoff',
!     >    'mm/day',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'drainage','average drainage',
!     >    'mm/day',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr (amtrunoff, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'trunoff',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, trunoff'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr (amsrunoff, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'srunoff',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, srunoff'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr (amdrainage, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'drainage',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, drainage'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! soil temperature
!
!      filen = 'tsoi-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average soil temperature',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'soillayer','depth of soil layer bottom',
!     >    'meter',nsoilay,depthsoi,'down',tunits,'gregorian',istat)
!         dimnames(3) = 'soillayer'
!         call inivar(idies,'tsoi','soil temperature per layer',
!     >    'degC',4,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      do k = 1, nsoilay
!         call vec2arr (amtsoil(lbeg,k), cdummy((k-1)*nlon*nlat + 1))
!      end do
!      istart(3) = 1
!      icount(3) = nsoilay
!      istart(4) = mstep
!      icount(4) = 1
!      call writevar(filen,'tsoi',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, tsoi'
!         call endrun
!      end if
!
! soil moisture, ice, volumetric water content, plant available water
!
!      filen = 'wsoi-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average soil moisture',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'soillayer','depth of soil layer bottom',
!     >    'meter',nsoilay,depthsoi,'down',tunits,'gregorian',istat)
!         dimnames(3) = 'soillayer'
!         call inivar(idies,'wsoi','soil moisture fraction per layer',
!     >    'fraction',4,dimnames,ocean,istat)
!         dimnames(3) = 'soillayer'
!         call inivar(idies,'wisoi','soil ice',
!     >    'fraction',4,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'vwc','average volumetric water content',
!     >    'fraction',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'awc',
!     >    'average plant available water content',
!     >    'cm',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      do k = 1, nsoilay
!        call vec2arr (amwsoil(lbeg,k), cdummy((k-1)*nlon*nlat + 1))
!      end do
!      istart(3) = 1
!      icount(3) = nsoilay
!      istart(4) = mstep
!      icount(4) = 1
!      call writevar(filen,'wsoi',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, wsoi'
!         call endrun
!      end if
!      do k = 1, nsoilay
!        call vec2arr (amwisoil(lbeg,k), cdummy((k-1)*nlon*nlat + 1))
!      end do
!      icount(3) = nsoilay
!      istart(4) = mstep
!      icount(4) = 1
!      call writevar(filen,'wisoi',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, wisoi'
!         call endrun
!      end if

      call vec2arr (amvwc, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'vwc',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, vwc'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr (amawc, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'awc',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, awc'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! snow depth
!
!      filen = 'snod-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average snow depth',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'snod','average snow depth',
!     >    'meters',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr (amsnod, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'snod',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, snod'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! snow fraction
!
!      filen = 'snof-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average snow fraction',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'snof','average snow fraction',
!     >    'm^2/m^3',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr (amsnof, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'snof',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, snof'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! solar radiation
!
!      filen = 'solar-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average incident solar radiation',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'solar','average incident solar radiation',
!     >    'W/m^2',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr (amsolar, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'solar',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, solar'
!         call endrun
!      end if
!
! albedo
!
!      filen = 'albedo-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >     'monthly average albedo',
!     >     'ibis wmonthly',cdate,nlon,lonscale,nlat,latscale,'none',
!     >     'none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'albedo','average albedo',
!     >     'fraction',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
 
      call vec2arr (amsnof, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'snof',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, snof'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! normalize the albedo calculation (not at each time step)
      do i = lbeg, lend
        amreflect(i) = amreflect(i) / (amsolar(i) + epsilon)
      end do

      call vec2arr (amreflect, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'albedo',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, albedo'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! downward and upward infrared radiation
!
!      filen = 'ir-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average infrared radiation',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'irdown','average downward IR',
!     >    'W/m^2',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'irup','average upward IR',
!     >    'W/m^2',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr (amirdown, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'irdown',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, irdown'
!         call endrun
!      end if
!      call vec2arr (amirup, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'irup',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, irup'
!         call endrun
!      end if
!
! sensible heat flux
!
!      filen = 'output/monthly/sens-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average sensible heat flux',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'sens','average sensible heat flux',
!     >    'W/m^2',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr (amsens, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'sens',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, sens'
!         call endrun
!      end if
!
! latent heat flux
!
!      filen = 'output/monthly/latent-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly average latent heat flux',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'latent','average latent heat flux',
!     >    'W/m^2',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr (amlatent, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'latent',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, latent'
!         call endrun
!      end if
!
! leaf area index upper and lower
!
!      filen = 'lai-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >   'monthly average leaf area index for upper and lower canopies',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'laiu','average lai upper canopy',
!     >    'fraction',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'lail','average lai lower canopy',
!     >    'fraction',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr (amlaiu, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'laiu',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, laiu'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr (amlail, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'lail',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, lail'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! total net primary productivity
!
!      filen = 'npptot-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly total net primary productivity of carbon',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'npptot','total npp of carbon',
!     >    'kg m-2 month-1',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr (amnpptot, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'npptot',istart,icount,cdummy,ftime, &
                       tweight,tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wmonthly, npptot'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! co2ratio
!
!      filen = 'co2ratio-monthly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'monthly ratio of root respiration to total soil respiration',
!     >    'ibis wmonthly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'co2ratio',
!     >    'ratio of root to soil respiration',
!     >    'fraction',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr (amco2ratio, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'co2ratio',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wmonthly, co2ratio'
!         call endrun
!      end if
     
      return
end subroutine wmonthly
