#include "inland_config.h"

#ifdef SINGLE_POINT_MODEL
#error "This subroutine should NOT be compiled for 0D IBIS model option."
#endif

! ---------------------------------------------------------------------
subroutine wyearly (nday)
! ---------------------------------------------------------------------
! writes out yearly files
! ---------------------------------------------------------------
      use inland_ibispar
      use inland_ibisctl, only: iyear, iyear0
      use inland_comveg
      use inland_comwork
      use inland_comsum
#if (defined CAMIBIS)
      use inland_abortutils, only: endrun
#endif

      implicit none

#ifdef GFORTRAN
#include <netcdf.inc>
#else /* GFORTRAN */
      include 'netcdf.inc'
#endif

! -----------------------------------------------------------------------
! input variables
      integer :: nday      ! number of days run since iyear0

! local variables
      integer :: mstep, idies, istat, i, k , zero
      integer :: istart(4), icount(4)  ! for writing vars
      real*8 :: pindex(npft)    ! index used for pfts and canopies
      character*8 tmpdate       ! temp. variable to hold date
      character*8 cdate         ! date to use in history attribute in files
      character*10 tdate        ! character date for time step
      character*13 canopies(2)  ! canopy definitions
      character*1024 canopies2  ! canopy definitions
      character*21 tunits       ! time units
      character*80 dimnames(4)  ! names of dimensions for vars
      character*60 pftdef(npft) ! plant functional type defs
      character*1024 pftdef2
      character*60 vegtypedef(15) ! vegtype defs (15 currently)
      character*1024 vegtypedef2
      character*1024 filen      ! file name
      character*4 chyear
      real*8 ftime,       & ! real form of nday
             tweight        ! number of days in yearly average

      data istart / 1,1,1,1 /,  &
           icount / nlon,nlat,1,1 /
! ---------------------------------------------------------------------
! FIXME: no point in using 'data' above if we change it right away to a
!       variable that is not compatible with 'data' assignment.
      icount(1) = nlonsub
      icount(2) = nlatsub

      zero = 0 ! you kiddin!

! current time value and step: make ftime Jan 1 of this year
! instead of Dec 31
      ftime = nday - ndaypy + 1
      tweight = ndaypy
!     mstep = iyear - iyear0 + 1
      mstep = 1

      if (iyear .lt. 10) then
         write(chyear(1:1),'(i1)') zero
         write(chyear(2:2),'(i1)') zero
         write(chyear(3:3),'(i1)') zero
         write(chyear(4:4),'(i1)') iyear
      elseif (iyear .lt. 100) then
         write(chyear(1:1),'(i1)') zero
         write(chyear(2:2),'(i1)') zero
         write(chyear(3:4),'(i2)') iyear
      elseif (iyear .lt. 1000) then
         write(chyear(1:1),'(i1)') zero
         write(chyear(2:4),'(i3)') iyear
      else
         write(chyear,'(i4)') iyear
      endif

! tdate is ANN (3 char) followed by this year (4 char)
      tdate='ANN0000'//char(0)//char(0)//char(0)
      if (iyear .ge. 1000) then
         write(tdate(4:7),'(i4)') iyear
      elseif (iyear .lt. 10) then
         write(tdate(7:7),'(i1)') iyear
      elseif (iyear .lt. 100) then
         write(tdate(6:7),'(i2)') iyear
      else
         write(tdate(5:7),'(i3)') iyear
      end if

! first time only
      if (mstep .eq. 1) then
         call date_and_time(tmpdate)
         cdate(1:2) = tmpdate(5:6)
         cdate(3:3) = '/'
         cdate(4:5) = tmpdate(7:8)
         cdate(6:6) = '/'
         cdate(7:8) = tmpdate(3:4)

! time units is days since Dec 31 of the year before iyear0
         tunits = 'days since 0000-12-31'
         write(tunits(12:15),'(i4)') iyear0-1

! dimension names
         dimnames(1) = 'longitude'
         dimnames(2) = 'latitude'
!        dimnames(3) is set for each variable seperately
         dimnames(4) = 'time'

! define plant functional types, canopies with indices
         do 100 i = 1, npft
            pindex(i) = i
100      continue

! pft defs
         pftdef(1)  = ' 1: trbrevtr - tropical broadleaf evergreen trees'
         pftdef(2)  = ' 2: trbrdetr - tropical broadleaf drought-deciduous trees'
         pftdef(3)  = ' 3: wtbrevtr - warm-temperate broadleaf evergreen trees'
         pftdef(4)  = ' 4: tecoevtr - temperate conifer evergreen trees'
         pftdef(5)  = ' 5: tebrdetr - temperate broadleaf cold-deciduous trees'
         pftdef(6)  = ' 6: bocoevtr - boreal conifer evergreen trees'
         pftdef(7)  = ' 7: bocodetr - boreal conifer cold-deciduous trees'
         pftdef(8)  = ' 8: bobrdetr - boreal broadleaf cold-deciduous trees'
         pftdef(9)  = ' 9: evsh - evergreen shrubs'
         pftdef(10) = '10: desh - deciduous shrubs'
         pftdef(11) = '11: c4gr - warm (c4) grasses'
         pftdef(12) = '12: c3gr - cool (c3) grasses'
         write(pftdef2,'(A,A,A,A,A,A,A,A,A,A,A,A,A)') char(10), &
              pftdef(1)//char(10),pftdef(2)//char(10),pftdef(3)//char(10), &
              pftdef(4)//char(10),pftdef(5)//char(10),pftdef(6)//char(10), &
              pftdef(7)//char(10),pftdef(8)//char(10),pftdef(9)//char(10), &
              pftdef(10)//char(10),pftdef(11)//char(10),pftdef(12)//char(0)
         
! vegtype defs from inland_vegmap.F90
         vegtypedef(1)  = ' 1: tropical evergreen forest / woodland'
         vegtypedef(2)  = ' 2: tropical deciduous forest / woodland'
         vegtypedef(3)  = ' 3: temperate evergreen broadleaf forest / woodland'
         vegtypedef(4)  = ' 4: temperate evergreen conifer forest / woodland'
         vegtypedef(5)  = ' 5: temperate deciduous forest / woodland'
         vegtypedef(6)  = ' 6: boreal evergreen forest / woodland'
         vegtypedef(7)  = ' 7: boreal deciduous forest / woodland'
         vegtypedef(8)  = ' 8: mixed forest / woodland'
         vegtypedef(9)  = ' 9: savanna'
         vegtypedef(10) = '10: grassland / steppe'
         vegtypedef(11) = '11: dense shrubland'
         vegtypedef(12) = '12: open shrubland'
         vegtypedef(13) = '13: tundra'
         vegtypedef(14) = '14: desert '
         vegtypedef(15) = '15: polar desert / rock / ice'
         write(vegtypedef2,'(A,A,A,A,A,A,A,A,A,A,A,A,A,A,A,A)') char(10), &
              vegtypedef(1)//char(10),vegtypedef(2)//char(10), &
              vegtypedef(3)//char(10),vegtypedef(4)//char(10), &
              vegtypedef(5)//char(10),vegtypedef(6)//char(10), &
              vegtypedef(7)//char(10),vegtypedef(8)//char(10), &
              vegtypedef(9)//char(10),vegtypedef(10)//char(10), &
              vegtypedef(11)//char(10),vegtypedef(12)//char(10), &
              vegtypedef(13)//char(10),vegtypedef(14)//char(10), &
              vegtypedef(15)//char(0)
         
! canopy defs
         canopies(1) = 'lower canopy'
         canopies(2) = 'upper canopy'
         canopies2 = '1: '//canopies(1)//'/ 2: '//canopies(2)
      end if

! dummy variable example, 4-d var whose 3rd dim is a character dim (pft)
! copy and modify for a new variable
!
!      filen = 'output/yearly/dummyv.nc'
!      if (mstep .eq. 1) then
!         call inifilec(idies,filen,
!     >    'annual dummyv',
!     >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,
!     >    'pft','plant fuctional type','none',npft,80,pftdef,
!     >    tunits,'gregorian',istat)
!         dimnames(3) = 'pft'
!         call inivar(idies,'dummyv','dummyv for each pft',
!     >    'dummyvs-units',4,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      do 5 k = 1, npft
!         call vec2arr(aydummyv(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
! 5    continue
!      istart(3) = 1
!      istart(4) = mstep
!      icount(3) = npft
!      call writevar(filen,'dummyv',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wyearly, dummyv'
!         call endrun
!      end if
!
! dummy variable example, 3-d
!
!      filen = 'output/yearly/dummyv.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'dummyv','annual dummyv',cdate,nlon,lonscale,
!     >    nlat,latscale,'none','none','none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'dummyv','annual dummyv',
!     >    'dummyvs-units',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if
!      call vec2arr(aydummyv, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'dummyv',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wyearly, dummyv'
!         call endrun
!      end if
!
! net primary productivity, by pft and total
      if (myid .eq. 0) then
         filen = trim(outdir)//'/inland-yearly-'//chyear//'.nc'
         if (mstep .eq. 1) then
            call inifile(idies,filen,                                        &
                         'annual net primary productivity of carbon',        &
                         'ibis wyearly',cdate,nlonsub,lonscale,nlatsub,      &
                         latscale, 'pft','plant fuctional type','none','E',  &
                         npft,pindex,'none',tunits,'gregorian',istat)

 ! add global attribute to define pfts, vegtypes and canopies with text
 ! use netcdf low-level commands (must go back into redef mode)

            istat = NF_REDEF(idies)
            istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'canopy_def', &
                 LEN_TRIM(canopies2), canopies2)
            istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'pft_def', &
                 LEN_TRIM(pftdef2), pftdef2)
            istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'vegtype_def', &
                 LEN_TRIM(vegtypedef2), vegtypedef2)
            istat = NF_ENDDEF(idies)
            if (istat .ne. NF_NOERR) then
               print *, 'Error in wyearly'
               print *, NF_STRERROR(istat)
            end if
            
            dimnames(3) = 'pft'
            call inivar(idies,'exist','existence for each pft','none',4, &
                        dimnames,ocean,istat)

            call inivar(idies,'npp','npp of carbon for each pft', &
                        'kg m-2 year-1',4,dimnames,ocean,istat)
 !          call inifile(idies,filen,
 !     >       'annual average quanties',
 !     >       'ibis wyearly',cdate,nlon,lonscale,nlat,
 !     >       latscale,'none','none',
 !     >       'none',1,0.,'none',tunits,'gregorian',istat)
 !
            dimnames(3) = 'time'
            call inivar(idies,'npptot','total npp','kg m-2 year-1',3,dimnames, &
                        ocean,istat)
            call inivar(idies,'anpptot','total above-ground npp', &
                        'kg m-2 year-1',3,dimnames,ocean,istat)
            call inivar(idies,'aet','average evapotranspiration','mm/year',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'trunoff','total runoff','mm/year',3,dimnames,  &
                        ocean,istat)
            call inivar(idies,'srunoff','surface runoff','mm/year',3,dimnames, &
                        ocean,istat)
            call inivar(idies,'drainage','drainage','mm/year',3,dimnames,ocean,&
                        istat)
            call inivar(idies,'wsoi','average soil moisture','fraction',3,     &
                        dimnames,ocean,istat)
            call inivar(idies,'wisoi','average soil ice','fraction',3,dimnames,&
                        ocean,istat)
            call inivar(idies,'vwc','average volumetric water content', &
                        'fraction',3,dimnames,ocean,istat)
            call inivar(idies,'awc','average volumetric water content','cm',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'tsoi','average soil temperature','degrees C',3, &
                        dimnames,ocean,istat)

            dimnames(3) = 'pft'
            call inivar(idies,'plai','leaf area index for each pft','fraction',&
                        4,dimnames,ocean,istat)

            dimnames(3) = 'time'
            call inivar(idies,'totlaiu','total lai for upper canopy', &
                        'fraction',3,dimnames,ocean,istat)
            call inivar(idies,'totlail','total lai for lower canopy', &
                        'fraction',3,dimnames,ocean,istat)

            dimnames(3) = 'pft'
            call inivar(idies,'biomass','biomass for each pft','kg/m^2',4, &
                        dimnames,ocean,istat)

            dimnames(3) = 'time'
            call inivar(idies,'totbiou','total biomass for upper canopy', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totbiol','total biomass for lower canopy', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totfall','total litterfall','kg/m^2',3,dimnames,&
                        ocean,istat)
            call inivar(idies,'rootbio','total live root biomass carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totalit','total above ground litter carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totrlit','total below ground litter carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totcsoi','total soil carbon w/o litter', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totcmic','total microbial carbon','kg/m^2',3, &
                        dimnames,ocean,istat)
            call inivar(idies,'totanlit','total above ground litter nitrogen', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totrnlit','total below ground litter nitrogen', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'totnsoi','total soil nitrogen w/o litter', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'nmintot','total nitrogen mineralization', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'neetot','total net ecosystem echange carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'caccount','end of year carbon correction', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'co2mic','total microbe respiration carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'co2root','total root respiration carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'co2soi','total soil respiration carbon', &
                        'kg/m^2',3,dimnames,ocean,istat)
            call inivar(idies,'vegtype0','vegetation type','none',3,dimnames, &
                        ocean,istat)
            call inivar(idies,'fu','fractional cover of upper canopy', &
                        'fraction',3,dimnames,ocean,istat)
            call inivar(idies,'fl','fractional cover of lower canopy', &
                        'fraction',3,dimnames,ocean,istat)
            call endini(idies,istat)
         end if
      endif

      do 5 k = 1, npft
         call vec2arr(aynpp(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
5     continue

      if (myid .eq. 0) then
         istart(3) = 1
         istart(4) = mstep
         icount(3) = npft
         call writevar(filen,'npp',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, npp'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aynpptot, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'npptot',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, npptot'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(ayanpptot, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'anpptot',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, anpptot'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! evapotranspiration
!
!      filen = 'aet-yearly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'annual average evapotranspiration',
!     >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'aet','average evapotranspiration',
!     >    'mm/year',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr(ayaet, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'aet',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, aet'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! trunoff, srunoff, drainage, rratio, tratio
!
!      filen = 'runoff-yearly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'annual runoff',
!     >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'trunoff','total runoff',
!     >    'mm/year',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'srunoff','surface runoff',
!     >    'mm/year',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'drainage','drainage',
!     >    'mm/year',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'rratio','average runoff ratio',
!     >    'fraction',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'tratio','average transpiration ratio',
!     >    'fraction',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr(aytrunoff, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'trunoff',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, trunoff'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aysrunoff, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'srunoff',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, srunoff'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aydrainage, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'drainage',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, drainage'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

!      call vec2arr(ayrratio, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'rratio',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wyearly, rratio'
!         call endrun
!      end if
!      call vec2arr(aytratio, cdummy)
!      istart(3) = mstep
!      icount(3) = 1
!      call writevar(filen,'tratio',istart,icount,cdummy,ftime,
!     > tweight,tdate,istat)
!      if (istat .ne. 0) then
!         write(*,*) 'ERROR in wyearly, tratio'
!         call endrun
!      end if
!
! soil moisture, soil ice, volumetric water content, plant available water
!
!      filen = 'wsoi-yearly-'//chyear//'.nc'
!      if (mstep .eq. 1) then
!         call inifile(idies,filen,
!     >    'annual average soil moisture',
!     >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!     >    latscale,'none','none',
!     >    'none',1,0.,'none',tunits,'gregorian',istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'wsoi','average soil moisture',
!     >    'fraction',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'wisoi','average soil ice',
!     >    'fraction',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'vwc','average volumetric water content',
!     >    'fraction',3,dimnames,ocean,istat)
!         dimnames(3) = 'time'
!         call inivar(idies,'awc','average volumetric water content',
!     >    'cm',3,dimnames,ocean,istat)
!         call endini(idies,istat)
!      end if

      call vec2arr(aywsoi, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'wsoi',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, wsoi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aywisoi, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'wisoi',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, wisoi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(ayvwc, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'vwc',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, vwc'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif

      call vec2arr(ayawc, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'awc',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, awc'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif
!
! soil temperature
!
!     filen = 'tsoi-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual average soil temperature',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'tsoi','average soil temperature',
!    >    'degrees C',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if

      call vec2arr(aytsoi, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'tsoi',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, tsoi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         endif
      endif
!
! solar radiation
!
!     filen = 'output/yearly/solar-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual average solar incident radiation',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'solar','average solar radiation',
!    >    'W/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(aysolar, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'solar',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, solar'
!        call endrun
!     end if
!
! albedo
!
!    filen = 'output/yearly/albedo-'//chyear//'.nc'
!    if (mstep .eq. 1) then
!       call inifile(idies,filen,
!    >    'annual average albedo',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!       dimnames(3) = 'time'
!       call inivar(idies,'albedo','average albedo',
!    >    'fraction',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!    end if
!    call vec2arr(ayalbedo, cdummy)
!    istart(3) = mstep
!    icount(3) = 1
!    call writevar(filen,'albedo',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!    if (istat .ne. 0) then
!       write(*,*) 'ERROR in wyearly, albedo'
!       call endrun
!    end if
!
! upward and downward infrared radiation
!
!     filen = 'output/yearly/ir-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual average solar infrared radiation',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'irdown','average downward ir',
!    >    'W/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'irup','average upward ir',
!    >    'W/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(ayirdown, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'irdown',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, irdown'
!        call endrun
!     end if
!     call vec2arr(ayirup, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'irup',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, irup'
!        call endrun
!     end if
!
! sensible heat flux
!
!     filen = 'sens-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual average sensible heat flux',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'sens','average sensible heat flux',
!    >    'W/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(aysens, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'sens',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, sens'
!        call endrun
!     end if
!
! latent heat flux
!
!     filen = 'output/yearly/latent-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual average latent heat flux',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'latent','average latent heat flux',
!    >    'W/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(aylatent, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'latent',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, latent'
!        call endrun
!     end if
!
! lai, by pft, total upper canopy, total lower canopy
!
!     filen = 'plai-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual leaf area index',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,
!    >    'pft','plant fuctional type','none',npft,pindex,'none',
!    >    tunits,'gregorian',istat)
! add global attribute to define pfts with text, use netcdf low-level command
!        istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'pft_definition',
!    >    npft*80,pftdef)
!        dimnames(3) = 'pft'
!        call inivar(idies,'plai','leaf area index for each pft',
!    >    'fraction',4,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totlaiu','total lai for upper canopy',
!    >    'fraction',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totlail','total lai for lower canopy',
!    >    'fraction',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
      do 10 k = 1, npft
         call vec2arr(plai(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
10    continue
      if (myid .eq. 0) then
         istart(3) = 1
         istart(4) = mstep
         icount(3) = npft
         call writevar(filen,'plai',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, plai'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      end if

      call vec2arr(totlaiu, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totlaiu',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totlaiu'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(totlail, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totlail',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totlail'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! biomass, by pft, upper canopy, lower canopy
!
!     filen = 'biomass-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual biomass of carbon',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,
!    >    'pft','plant fuctional type','none',npft,pindex,'none',
!    >    tunits,'gregorian',istat)
! add global attribute to define pfts with text, use netcdf low-level command
!        istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'pft_definition',
!    >    npft*80,pftdef)
!        dimnames(3) = 'pft'
!        call inivar(idies,'biomass','biomass for each pft',
!    >    'kg/m^2',4,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totbiou','total biomass for upper canopy',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totbiol','total biomass for lower canopy',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
      do 15 k = 1, npft
         call vec2arr(biomass(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
15    continue

      if (myid .eq. 0) then
         istart(3) = 1
         istart(4) = mstep
         icount(3) = npft
         call writevar(filen,'biomass',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, biomass'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      end if

      call vec2arr(totbiou, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totbiou',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totbiou'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(totbiol, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totbiol',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totbiol'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! soil carbon: rootbio, totalit, totrlit, totcsoi, totcmic
!
!     filen = 'csoi-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total soil carbon',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'rootbio','total live root biomass carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totalit','total above ground litter carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totrlit','total below ground litter carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totcsoi','total soil carbon w/o litter',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totcmic','total microbial carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if

      call vec2arr(ayrootbio, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'rootbio',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, rootbio'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(ayalit, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totalit',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totalit'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
       endif

      call vec2arr(ayblit, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totrlit',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totrlit'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aycsoi, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totcsoi',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totcsoi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aycmic, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totcmic',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totcmic'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! soil nitrogen: totanlit, totrnlit, totnsoi, nmintot
!
!     filen = 'nsoi-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total soil nitrogen',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totanlit',
!    >    'total above ground litter nitrogen',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totrnlit',
!    >    'total below ground litter nitrogen',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totnsoi','total soil nitrogen w/o litter',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'nmintot','total nitrogen mineralization',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if

      call vec2arr(ayanlit, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totanlit',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totanlit'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aybnlit, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totrnlit',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totrnlit'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aynsoi, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totnsoi',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totnsoi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(aynmintot, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'nmintot',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, nmintot'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! total litter
!
!     filen = 'output/yearly/totlit-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total litter carbon',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totlit','total litter carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(totlit, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'totlit',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, totlit'
!        call endrun
!     end if
!
! total wood litter
!
!     filen = 'output/yearly/clitw-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total wood litter carbon',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'clitw','total wood litter carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(clitw, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'clitw',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, clitw'
!        call endrun
!     end if
!
! total litterfall
!
!     filen = 'output/yearly/totfall-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total litterfall carbon',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'totfall','total litterfall',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
      call vec2arr(totfall, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'totfall',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, totfall'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      end if
!
! total soil carbon in slow pool
!
!     filen = 'output/yearly/csoislo-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total soil carbon in slow pool',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'csoislo','total soil carbon in slow pool',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(csoislo, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'csoislo',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, csoislo'
!        call endrun
!     end if
!
! total soil carbon in passive pool
!
!     filen = 'output/yearly/csoipas-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total soil carbon in pasive pool',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'csoipas','total soil carbon in passive pool',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(csoipas, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'csoipas',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, csoipas'
!        call endrun
!     end if
!
! co2 carbon exchange: net ecosystem, microbial resp, root resp, soil resp
!
!     filen = 'co2fluxes-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual total carbon from exchange of co2',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'neetot',
!    >    'total net ecosystem echange carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'co2mic','total microbe respiration carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'co2root','total root respiration carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'co2soi','total soil respiration carbon',
!    >    'kg/m^2',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if

      call vec2arr(ayneetot, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'neetot',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, neetot'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(caccount, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'caccount',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, caccount'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(ayco2mic, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'co2mic',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, co2mic'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(ayco2root, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'co2root',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, co2root'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(ayco2soi, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
       icount(3) = 1
         call writevar(filen,'co2soi',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, co2soi'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

! fire disturbance regime
!
!     filen = 'disturbf-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual fire disturbance regime',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'disturbf','fire disturbance regime',
!    >    'fraction/year',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(disturbf, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'disturbf',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, disturbf'
!        call endrun
!     end if
!
! vegetation type
!
!     filen = 'vegtype-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual vegetation type - ibis classification',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,
!    >    latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'vegtype0','vegetation type',
!    >    'none',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if

      call vec2arr(vegtype0, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'vegtype0',istart,icount,cdummy,ftime,tweight, &
                       tdate,istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, vegtype0'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! fractional cover of upper and lower canopies
!
!     filen = 'fcover-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual fractional cover of canopies','ibis wyearly',
!    >    cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'fu','fractional cover of upper canopy',
!    >    'fraction',3,dimnames,ocean,istat)
!        call inivar(idies,'fl','fractional cover of lower canopy',
!    >    'fraction',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if

      call vec2arr(fu, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'fu',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, fu'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif

      call vec2arr(fl, cdummy)
      if (myid .eq. 0) then
         istart(3) = mstep
         icount(3) = 1
         call writevar(filen,'fl',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, fl'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      endif
!
! sapwood fraction
!
!     filen = 'output/yearly/sapfrac-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual sapwood fraction',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,'none','none',
!    >    'none',1,0.,'none',tunits,'gregorian',istat)
!        dimnames(3) = 'time'
!        call inivar(idies,'sapfrac','sapwood fraction',
!    >    'fraction',3,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     call vec2arr(sapfrac, cdummy)
!     istart(3) = mstep
!     icount(3) = 1
!     call writevar(filen,'sapfrac',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, sapfrac'
!        call endrun
!     end if
!
! bottom and top of vegetation canopies
!
!     filen = 'zcanopy-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual height of vegetation canopies',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,
!    >    'canopy','canopy','none',2,pindex,'up',
!    >    tunits,'gregorian',istat)
! add global attribute to define canopies with text, use netcdf low-level com
!        istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'canopy_def',
!    >    26+5,'1='//canopies(1)//' 2='//canopies(2))
!        dimnames(3) = 'canopy'
!        call inivar(idies,'zbot',
!    >    'bottom heights of lower and upper canopies',
!    >    'meters',4,dimnames,ocean,istat)
!        call inivar(idies,'ztop',
!    >    'top heights of lower and upper canopies',
!    >    'meters',4,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
!     do 20 k = 1, 2
!        call vec2arr(zbot(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
! 20   continue
!     istart(3) = 1
!     istart(4) = mstep
!     icount(3) = 2
!     call writevar(filen,'zbot',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, zbot'
!        call endrun
!     end if
!     do 25 k = 1, 2
!        call vec2arr(ztop(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
! 25   continue
!     istart(3) = 1
!     istart(4) = mstep
!     icount(3) = 2
!     call writevar(filen,'ztop',istart,icount,cdummy,ftime,
!    > tweight,tdate,istat)
!     if (istat .ne. 0) then
!        write(*,*) 'ERROR in wyearly, ztop'
!        call endrun
!     end if
!
! existence of pfts
!
!     filen = 'exist-yearly-'//chyear//'.nc'
!     if (mstep .eq. 1) then
!        call inifile(idies,filen,
!    >    'annual existence of each plant functional type',
!    >    'ibis wyearly',cdate,nlon,lonscale,nlat,latscale,
!    >    'pft','plant fuctional type','none',npft,pindex,'none',
!    >    tunits,'gregorian',istat)
! add global attribute to define pfts with text, use netcdf low-level command
!        istat = NF_PUT_ATT_TEXT(idies,NF_GLOBAL,'pft_definition',
!    >    npft*80,pftdef)
!        dimnames(3) = 'pft'
!        call inivar(idies,'exist','existence for each pft',
!    >    'none',4,dimnames,ocean,istat)
!        call endini(idies,istat)
!     end if
      do 30 k = 1, npft
          call vec2arr(exist(lbeg,k), cdummy((k-1)*nlonsub*nlatsub + 1))
30    continue

      if (myid .eq. 0) then
         istart(3) = 1
         istart(4) = mstep
         icount(3) = npft
         call writevar(filen,'exist',istart,icount,cdummy,ftime,tweight,tdate, &
                       istat)
         if (istat .ne. 0) then
            write(*,*) 'ERROR in wyearly, exist'
#ifdef CAMIBIS
            call endrun
#else /* CAMIBIS */
            stop
#endif /* CAMIBIS */
         end if
      end if
      return
end subroutine wyearly
